/*
    \brief Deklarationen für die Listen der Zutrittskontrolle Version 1.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_ACCESS_CONTROL_LISTS_V1_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_ACCESS_CONTROL_LISTS_V1_H
#endif

#include "df_defines.h"

/*!
    \brief Bibliotheksinterne Datenpuffer für Listendaten der Zutrittskontrolle löschen und initalisieren.

    Die in der Bibliothek bereitgestellten Datenpuffer für die zu übertragenenden Listendaten werden gelöscht und
    für eine Verwendung mittels DFCMakeEntranceList() und DFCLoadEntranceList() initalisiert.

    \ingroup access_control_v1

    \export{_DFCClearEntranceListBuffer@8, DFCClearEntranceListBuffer}

    \warning Diese Funktion setzt eine Firmwareversion \b kleiner 04.01.00 vorraus.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] listNumber Listennummer. Bei Angabe von -1 werden alle vorhandenen Listenpuffer initialisiert.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Listenname</th></tr>
        <tr><td>-1</td><td>Alle Listen</td></tr>
        <tr><td>0</td><td>Zeitzonen</td></tr>
        <tr><td>1</td><td>Zeitmodelle</td></tr>
        <tr><td>2</td><td>Personalstamm</td></tr>
        <tr><td>3</td><td>Feiertage</td></tr>
    </table>
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCClearEntranceListBuffer( int connectionNumber, int listNumber );

/*!
    \brief Kopieren eines aufbereiteten Datenarrays für Listendaten der Zutrittskontrolle in den Biblioteksinternen Datenpuffer.

    Das zu übergebende Datenarray wird in den Bibliotheksinternen Datenpuffer kopiert und für eine spätere
    Übertragung mittels DFCLoadEntranceList() zwischengespeichert.

    \ingroup access_control_v1

    \export{_DFCMakeEntranceList@24, DFCMakeEntranceList}

    \warning Diese Funktion setzt eine Firmwareversion \b kleiner 04.01.00 vorraus.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] listNumber Listennummer.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Listenname</th><th>Größe pro Datensatz in Byte</th><th>max. Datensätze</th></tr>
        <tr><td>0</td><td>Zeitzonen</td><td>22</td><td>16</td></tr>
        <tr><td>1</td><td>Zeitmodelle</td><td>35</td><td>32</td></tr>
        <tr><td>2</td><td>Personalstamm</td><td>19</td><td>5120</td></tr>
        <tr><td>3</td><td>Feiertage</td><td>16</td><td>64</td></tr>
    </table>
    \param[in] recordCount Anzahl der Datensätze im übergebenen Datenarray.
    \param[in] size Anzahl Byte des Datenarray.\n Brechnet sich aus (\a recordCount * Größe pro Datensatz).
    \param[in] buffer Datenarray mit den Daten.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCMakeEntranceList( int connectionNumber, int listNumber, int recordCount, int size, const unsigned char *buffer, int *errorNumber );

/*!
    \brief Übertragen der bibliotheksinternen Datenpuffer zum Gerät.

    Es werden die mit DFCMakeEntranceList() importierten Listendaten in das Gerät geschrieben. Vorhandene Listendaten
    werden hierbei erst nach einer erfolgreichen Übernahme überschrieben.

    \ingroup access_control_v1

    \export{_DFCLoadEntranceList@16, DFCLoadEntranceList}

    \warning Diese Funktion setzt eine Firmwareversion \b kleiner 04.01.00 vorraus.

    \warning Bitte beachten Sie die [Hinweise zur Lebensdauer](\ref dfc_lists_memory) der eingesetzten Flash-Bausteine.
    Diese gelten ebenfalls für Zutrittslisten.

    \note Die Listen besitzen teilweise interne Bezüge zueinander. Um die Datenkonsistenz zu wahren müssen Sie sicherstellen,
    das bei hinzufügen neuer Datensätze die sich über mehrere Listen auswirken diese in einem Bundle übertragen werden. \n
    Importieren Sie hierzu bitte alle zusammengehörenden Listen mittels DFCMakeEntrance2List und überragen diese dann mit
    mit dem Wert -1 für den Parameter number. Es wird dadurch sichergestellt, dass alle Listen erst übernommen werden,
    wenn die Überragung durchweg erfolgreich war.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] listNumber Listennummer. -1 Überträgt alle importierten Listendaten.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Listenname</th></tr>
        <tr><td>-1</td><td>Alle mit DFCMakeEntranceListe importieren Listen</td></tr>
        <tr><td>0</td><td>Zeitzonen</td></tr>
        <tr><td>1</td><td>Zeitmodelle</td></tr>
        <tr><td>2</td><td>Personalstamm</td></tr>
        <tr><td>3</td><td>Feiertage</td></tr>
        <tr><td>-1</td><td>Alle importierten Listenpuffer</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadEntranceList( int connectionNumber, int deviceAddress, int listNumber, int *errorNumber );

#endif
