/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef DATAFOX_COMMUNICATION_CONNECTION_BASE_H
#define DATAFOX_COMMUNICATION_CONNECTION_BASE_H

// Standardtypen
#include <cstdint>
#include <memory>

/////////////////////////////
// Error's
#include "common/error_defines.h"

#ifdef _MSC_VER
#define __func__ __FUNCTION__
#pragma warning(disable : 4996)
#endif

#if defined(_MSC_VER) || defined(__MINGW32__)


typedef int32_t socklen_t;

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <winsock2.h>
#include <string>

#ifndef STRING2
#define STRING2(x) #x
#define STRING(x) STRING2(x)
#define TODO(x) __pragma(message(__FILE__ "(" STRING(__LINE__) ") : todo : " #x))
#define FIXME(x) __pragma(message(__FILE__ "(" STRING(__LINE__) ") : fixme : " #x))
#endif

#else

#include <string>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
typedef int32_t HANDLE;
#define SOCKET int32_t
#define MAX_PATH 260
#define _MAX_PATH MAX_PATH
#define _MAX_EXT 256
#define TRUE 1
#define FALSE 0
#include <termio.h>
#include <sys/file.h>

#define INVALID_HANDLE_VALUE -1

#include <errno.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define INVALID_SOCKET       -1
#define SOCKADDR_IN struct sockaddr_in
#define SOCKADDR struct sockaddr
#define HOSTENT struct hostent
#define WSAGetLastError() h_errno
#define SOCKET_ERROR -1

#define FAR

#ifndef STRING2
#define STRING2(x) #x
#define STRING(x) STRING2(x)
#define TODO(x)
#define FIXME(x)
#endif

#endif

namespace Datafox
{
    namespace Communication
    {
        class TraceBase
        {
        public:
            enum class TraceMode
            {
                off,
                on,
                packets
            };

        public:
            TraceBase( int32_t number );

            void print( const char *format, va_list args );

            inline int32_t number()
            {
                return m_number;
            }
            inline void setNumber( int32_t number )
            {
                m_number = number;
            }

            inline TraceMode mode()
            { 
                return m_mode; 
            }
            inline void setMode( TraceMode mode = TraceMode::on )
            { 
                m_mode = mode;
            }

            inline std::string filePath()
            {
                return m_filePath;
            }
            inline void setFilePath( const std::string &filePath )
            { 
                m_filePath = filePath;
            }

        private:
            int32_t m_number;
            TraceMode m_mode;            // Mode des Trace
            std::string m_filePath;      // Dateiangabe zum Trace
        };

        // Basisklasse fuer pure Kommunikationsaufgaben.
        class ConnectionBase  
        {
        public:
            // Standardwerte für Parameter
            static constexpr uint8_t const defaultConnectionNumber()
            {
                return 0;
            }
            static constexpr uint8_t const defaultReadTrials()
            {
                return 3;
            }
            static constexpr uint8_t const defaultWriteTrials()
            {
                return 3;
            }

        public:
            ConnectionBase( int32_t number = defaultConnectionNumber() );
            virtual ~ConnectionBase();

        public:
            // Typ der Verbindung
            enum class ConnectionType 
            { 
                undefined, 
                serial, 
                socket
            };

            // Oeffnen der Schnittstelle
            virtual int32_t open( ConnectionType type, const std::string &connectionString, uint32_t connectionValue, uint16_t communicationReadTimeout );
            // Schliessen der Schnittstelle.
            virtual int32_t close();

            // Zuweisen von HANDLE oder Socket ohne Schnittstelle selbst zu oeffnen.
            int32_t attach( ConnectionType type, HANDLE connectionHandle, uint16_t communicationReadTimeout );
            // Freigeben der anderweitig zugewiesenen Resourcen
            int32_t detach();

            // Ermitteln ob Schnittstelle geoeffnet.
            bool isOpen();

            // Ermitteln der uebergebenen Parameter an Open.
            inline void openParameters( ConnectionType &type, std::string &connectionString, uint32_t &connectionValue, uint16_t &communicationReadTimeout ) 
            {
                type = m_type;
                connectionString = m_string;
                connectionValue = m_value;
                communicationReadTimeout = m_readTimeout;
            }

        public:
            // Loeschen anstehender Daten.
            void flush();
            int32_t flushSocket();

        public:
            // Oeffnen der Schnittstelle fuer RS232 / RS485 Verbindung.
            int32_t openSerial( const std::string &serialPortName, uint32_t baudRate, uint16_t communicationReadTimeout );
            int32_t closeSerial();
            int32_t changeSerialState( HANDLE connectionSerialPort, uint32_t baudRate, uint16_t communicationReadTimeout );

            // Oeffnen der Schnittstelle fuer TCP/IP.
            int32_t openSocket( const std::string &hostName, uint16_t portNumber, uint16_t communicationReadTimeout );
            int32_t closeSocket();

        protected:
            // Datenempfang
            int32_t readSerial( uint8_t *buffer, uint32_t readCount, uint32_t &readInCount, uint8_t readTrials = defaultReadTrials() );
            int32_t readSocket( uint8_t *buffer, uint32_t readCount, uint32_t &readInCount, uint8_t readTrials = defaultReadTrials() );

        public:
            virtual int32_t readData( uint8_t *buffer, uint32_t readCount, uint32_t &readInCount, uint8_t readTrials = defaultReadTrials() );

        protected:
            // Datenversand
            int32_t writeSerial( const uint8_t *buffer, uint32_t writeCount, uint32_t &writeOutCount, uint8_t writeTrials = defaultWriteTrials() );
            int32_t writeSocket( const uint8_t *buffer, uint32_t writeCount, uint32_t &writeOutCount, uint8_t writeTrials = defaultWriteTrials() );

        public:
            virtual int32_t writeData( const uint8_t *buffer, uint32_t writeCount, uint32_t &writeOutCount, uint8_t writeTrials = defaultWriteTrials() );

        private:
            std::shared_ptr<TraceBase> m_trace;

        public:
            std::shared_ptr<TraceBase> tracePointer()
            {
                return m_trace;
            }
            void setTracePointer( std::shared_ptr<TraceBase> base )
            {
                m_trace = base;
            }

        public:
            virtual void trace( const char *format... )
            {
                if ( m_trace.get() != nullptr && m_trace->mode() >= TraceBase::TraceMode::on )
                {
                    va_list args;
                    va_start( args, format );
                    m_trace->print( format, args );
                    va_end( args );
                }
            }

            TraceBase::TraceMode traceMode()
            {
                TraceBase::TraceMode mode = TraceBase::TraceMode::off;
                if ( m_trace.get() != nullptr )
                {
                    mode = m_trace->mode();
                }

                return mode;
            }
            void setTraceMode( TraceBase::TraceMode mode = TraceBase::TraceMode::on )
            {
                if ( m_trace.get() != nullptr )
                {
                    m_trace->setMode( mode );
                }
            }

            std::string traceFilePath()
            {
                std::string file_path;
                if ( m_trace.get() != nullptr )
                {
                    file_path = m_trace->filePath();
                }

                return file_path;
            }
            void setTraceFilePath( const std::string &filePath )
            {
                if ( m_trace.get() != nullptr )
                {
                    m_trace->setFilePath( filePath );
                }
            }

        public:
            inline int32_t number() 
            { 
                return m_number; 
            }
            inline int32_t setNumber( int32_t id ) 
            { 
                int32_t old = m_number;
                m_number = id; 
                if ( m_trace.get() != nullptr )
                {
                    m_trace->setNumber( id );
                }
                return old; 
            }
        public:
            inline ConnectionType type() 
            { 
                return m_type; 
            }
            inline void setType( ConnectionType type )
            {
                m_type = type;
            }
        public:
            inline HANDLE serialPort()
            {
                return m_serialPort;
            }
            inline void setSerialPort( HANDLE serial_port )
            {
                m_serialPort = serial_port;
            }
        public:
            inline SOCKET socket()
            {
                return m_socket;
            }
            inline void setSocket( SOCKET socket )
            {
                m_socket = socket;
            }
        public:
            inline std::string hostName()
            {
                return m_string;
            }
            inline void setHostName( const std::string &name )
            {
                m_string = name;
            }
        public:
            inline std::string serialPortName()
            {
                return m_string;
            }
            inline void setSerialPortName( const std::string &name )
            {
                m_string = name;
            }
        public:
            inline uint16_t portNumber()
            {
                return static_cast<uint16_t>( m_value );
            }
            inline void setPortNumber( uint16_t value )
            {
                m_value = value;
            }
        public:
            inline uint32_t baud()
            {
                return m_value;
            }
            inline void setBaud( uint32_t value )
            {
                m_value = value;
            }
        public:
            inline uint16_t readTimeout()
            { 
                return m_readTimeout; 
            }
            inline void setReadTimeout( uint16_t timeout )
            { 
                m_readTimeout = timeout; 
            }

        protected:
            int32_t m_number;             // Nummer der Verbindung
            ConnectionType m_type;        // Typ der Verbindung / Schnittstelle
            HANDLE m_serialPort;          // Handle wenn Typ: ct_seriel
            SOCKET m_socket;              // Socket wenn Typ: ct_socket
            std::string m_string;         // Haelt letzten ComPort-String
            uint32_t m_value;             // Baudrate bei Typ Seriell, Portnummer bei Typ Socket
            uint16_t m_readTimeout;       // Lesetimeout fuer Kommunikationsfunktionen
#if !defined(_MSC_VER) && !defined(__MINGW32__)
            struct termios m_termio;
            struct flock  m_flock;
#endif

        protected:
            uint32_t m_sizeWritten;    // Statistik, Anzahl der versendeten Bytes
            uint32_t m_sizeRead;       // Statistik, Anzahl der gelesenen Bytes

        public:
            inline uint32_t sizeWritten()
            {
                return m_sizeWritten;
            }
            inline void setSizeWritten( uint32_t size )
            {
                m_sizeWritten = size;
            }
            inline void addSizeWritten( uint32_t size )
            {
                m_sizeWritten += size;
            }
            inline void subSizeWritten( uint32_t size )
            {
                m_sizeWritten -= size;
            }

            inline uint32_t sizeRead()
            {
                return m_sizeRead;
            }
            inline void setSizeRead( uint32_t size )
            {
                m_sizeRead = size;
            }
            inline void addSizeRead( uint32_t size )
            {
                m_sizeRead += size;
            }
            inline void subSizeRead( uint32_t size )
            {
                m_sizeRead -= size;
            }
        };

    } // namespace Communication
} // namespace Datafox

#endif
