/**
* @file  hardware_info_items.cpp
* @brief Implementierung der Hilfsklassen für FileHardwareInfo.
*/
//////////////////////////////////////////////////////////////////////

#include "file/interface/hardware_info_items.h"

#include <algorithm>
#include <boost/format.hpp>
#include <boost/algorithm/string/replace.hpp>

/*! Representiert einen HardwareConfig Eintrag.
    Von der Gesamtrepresentation gesehen ist das Wurzelelement die HARDWARECONFIGNODE, alle
    Childs des einzigen Wurzelelementes stellen dann jeweils eine HARDWAERCONFIG mit seinen 
    ggf. vorhandenen Untermodulen dar.

    Beispielhafte Baumstruktur:
    HardwareConfigNode (root)
        HardwareConfig ( dieser HardwareConfig folgt keine HARDWARECONFIGSUBNODE in der Serialisierung )
        HardwareConfig ( dieser HardwareConfig folgt eine HARDWARECONFIGSUBNODE in der Serialisierung )
            HardwareConfig
            HardwareConfig
        HardwareConfig ( dieser HardwareConfig folgt keine HARDWARECONFIGSUBNODE in der Serialisierung )

*/
// ######################################################################################

bool ItemHardwareInfoBase::m_autoCorrectModuleDisabled = false;

ItemHardwareInfoBase::ItemHardwareInfoBase( ItemHardwareInfoBase *parent, EnumItemHardwareInfoType typeId )
    :  m_parent( parent ), m_typeId( typeId )
{

}

ItemHardwareInfoBase::~ItemHardwareInfoBase()
{

}

ItemHardwareInfoBase *ItemHardwareInfoBase::parent()
{
    return m_parent;
}

void ItemHardwareInfoBase::setParent( ItemHardwareInfoBase *parent )
{
    m_parent = parent;
}

EnumItemHardwareInfoType ItemHardwareInfoBase::typeId()
{
    return m_typeId;
}

void ItemHardwareInfoBase::setTypeId( EnumItemHardwareInfoType typeId )
{
    m_typeId = typeId;
}

void ItemHardwareInfoBase::setAutoCorrectModuleDisabled( bool state )
{
    m_autoCorrectModuleDisabled = state;
}

bool ItemHardwareInfoBase::autoCorrectModuleDisabled()
{
    return m_autoCorrectModuleDisabled;
}

std::string jsonString( std::string text )
{
    if ( text.find_first_of( '\"' ) != std::string::npos )
    {
        boost::algorithm::replace_all( text, "\"", "\\\"" );
    }

    return "\"" + text + "\"";
}

ItemJumper::ItemJumper()
{
    // Member initalisieren.
    reset();
}

ItemJumper::~ItemJumper()
{

}

void ItemJumper::reset()
{
    // Daten löschen.
    std::fill( m_structData.begin(), m_structData.end(), 0 );
}

bool ItemJumper::writeSerialize( std::vector<uint8_t>::iterator iter )
{
    // Daten als kopie übernehmen.
    std::copy_n( m_structData.begin(), m_structData.size(), iter );

    return true;
}

bool ItemJumper::readSerialize( std::vector<uint8_t>::const_iterator iter )
{
    // Daten als kopie übernehmen.
    std::copy_n( iter, m_structData.size(), m_structData.begin() );

    return true;
}

HARDWARECONFIGJUMPER &ItemJumper::hcj()
{
    return reinterpret_cast<HARDWARECONFIGJUMPER &>( m_structData );
}

std::string ItemJumper::toJson()
{
    std::string text;
    text = "{";
    text += boost::str( boost::format("\"number\": %d,") % static_cast<int>( hcj().number ) );
    text += boost::str( boost::format("\"flags\": %d,") % static_cast<int>( hcj().flags ) );
    text += boost::str( boost::format("\"count\": %d,") % static_cast<int>( hcj().count ) );
    text += boost::str( boost::format("\"start\": %d,") % static_cast<int>( hcj().start ) );
    text += boost::str( boost::format("\"end\": %d,") % static_cast<int>( hcj().end ) );
    text += boost::str( boost::format("\"name\": %s") % jsonString( hcj().name ) );
    text += "}";

    return text;
}

ItemBoard::ItemBoard( ItemHardwareInfoBase *parent, EnumItemHardwareInfoType typeId )
    : ItemHardwareInfoBase( parent, typeId )
{
    // Member initalisieren.
    reset();
}

ItemBoard::~ItemBoard()
{
}

void ItemBoard::reset()
{
    // Alle Childs löschen
    m_childs.clear();

    // Alle Jumper löschen
    m_jumpers.clear();

    // Daten löschen.
    std::fill( m_structData.begin(), m_structData.end(), 0 );
}

bool ItemBoard::writeSerialize( std::vector<uint8_t> &buffer )
{
    // Aktuelle groesse fuer Zugriff halten.
    uint32_t offs = static_cast<uint32_t>( buffer.size() );

    if ( m_childs.size() )
    {
        // Groesse fuer Hinzufuegung anpassen.
        buffer.resize( buffer.size() + sizeof( HARDWARECONFIG )
                       + m_jumpers.size() * (sizeof( uint32_t ) + sizeof( HARDWARECONFIGJUMPER )) + sizeof( HARDWARECONFIGSUBNODE ) + m_childs.size() * sizeof( uint32_t ) );

        // Sind Angaben zu Jumpern vorhanden?
        if ( m_jumpers.size() )
        {        
            // Anzahl Jumperstrukturen eintragen.
            hc().jumperCount = static_cast<uint8_t>( m_jumpers.size() );
            // Zeiger auf HardwareConfigPointer eintragen.
            hc().ptrHardwareConfigJumperArray = offs + sizeof( HARDWARECONFIG );

            // Angaben für Jumper eintragen.
            auto iter = m_jumpers.begin();
            for ( std::size_t i = 0; iter != m_jumpers.end(); ++iter, ++i )
            {
                uint32_t &offs = reinterpret_cast<uint32_t &>( *(buffer.data() + hc().ptrHardwareConfigJumperArray + i * sizeof( uint32_t )) );
                offs = static_cast<uint32_t>( hc().ptrHardwareConfigJumperArray + static_cast<uint32_t>( m_jumpers.size() ) * sizeof( uint32_t ) + i * sizeof( HARDWARECONFIGJUMPER ) );

                (*iter).writeSerialize( buffer.begin() + offs );
            }

            // Zeiger auf HardwareConfigSubNode eintragen.
            hc().ptrHardwareConfigSubNode = hc().ptrHardwareConfigJumperArray + static_cast<uint32_t>( m_jumpers.size() ) * (sizeof( uint32_t ) + sizeof( HARDWARECONFIGJUMPER ));
        }
        else
        {
            // Anzahl der Jumperstrukturen eintragen.
            hc().jumperCount = 0;
            // Zeiger auf HardwareConfigPointer eintragen.
            hc().ptrHardwareConfigJumperArray = 0;

            // Zeiger auf HardwareConfigSubNode eintragen.
            hc().ptrHardwareConfigSubNode = offs + sizeof( HARDWARECONFIG );
        }

        // Eigene Config eintragen.
        std::copy_n( m_structData.begin(), m_structData.size(), buffer.begin() + offs );

        // HardwareConfigSubNode aufbereiten.
        std::array<uint8_t, sizeof( HARDWARECONFIGSUBNODE )> structHcsn;
        structHcsn.fill( 0 );
        HARDWARECONFIGSUBNODE &hcsn= reinterpret_cast<HARDWARECONFIGSUBNODE &>( structHcsn );

        hcsn.count = static_cast<uint16_t>( m_childs.size() );
        hcsn.ptrHardwareConfigArray = hc().ptrHardwareConfigSubNode + sizeof( HARDWARECONFIGSUBNODE );

        // HardwareConfigSubNode eintragen.
        std::copy_n( structHcsn.begin(), structHcsn.size(), buffer.begin() + hc().ptrHardwareConfigSubNode );

        auto iter = m_childs.begin();
        for ( std::size_t i = 0; iter != m_childs.end(); ++iter, ++i )
        {
            uint32_t &offs = reinterpret_cast<uint32_t &>( *(buffer.data() + hcsn.ptrHardwareConfigArray + i * sizeof( uint32_t )) );
            offs = static_cast<uint32_t>( buffer.size() );

            if ( !(*iter).writeSerialize( buffer ) )
                return false;
        }
    }
    else
    {
        // Groesse fuer hinzufuegung anpassen.
        buffer.resize( buffer.size() + sizeof( HARDWARECONFIG ) + m_jumpers.size() * (sizeof( uint32_t ) + sizeof( HARDWARECONFIGJUMPER )) );

        // Sind Angaben zu Jumpern vorhanden?
        if ( m_jumpers.size() )
        {        
            // Anzahl der Jumperstrukturen eintragen.
            hc().jumperCount = static_cast<uint8_t>( m_jumpers.size() );
            // Zeiger auf HardwareConfigPointer eintragen.
            hc().ptrHardwareConfigJumperArray = offs + sizeof( HARDWARECONFIG );

            // Angaben für Jumper eintragen.
            auto iter = m_jumpers.begin();
            for ( std::size_t i = 0; iter != m_jumpers.end(); ++iter, ++i )
            {
                uint32_t &offs = reinterpret_cast<uint32_t &>( *(buffer.data() + hc().ptrHardwareConfigJumperArray + i * sizeof( uint32_t )) );
                offs = static_cast<uint32_t>( hc().ptrHardwareConfigJumperArray + static_cast<uint32_t>( m_jumpers.size() ) * sizeof( uint32_t ) + i * sizeof( HARDWARECONFIGJUMPER ) );

                (*iter).writeSerialize( buffer.begin() + offs );
            }
        }
        else
        {
            // Anzahl der Jumperstrukturen eintragen.
            hc().jumperCount = 0;
            // Zeiger auf HardwareConfigPointer eintragen.
            hc().ptrHardwareConfigJumperArray = 0;
        }

        // Zeiger auf HardwareConfigSubNode sicherstellen.
        hc().ptrHardwareConfigSubNode = 0;

        // Lediglich die eigene Config eintragen.
        std::copy_n( m_structData.begin(), m_structData.size(), buffer.begin() + offs );
    }

    return true;
}

bool ItemBoard::readSerialize( std::vector<uint8_t>::const_iterator iterRead, const std::vector<uint8_t> &buffer )
{
    // Eigene Config lesen.
    std::copy_n( iterRead, m_structData.size(), m_structData.begin() );
    if ( hc().ptrHardwareConfigJumperArray > buffer.size() || hc().ptrHardwareConfigSubNode > buffer.size() )
    {
        return false;
    }


    // Hat diese Config Jumperangaben.
    if ( hc().ptrHardwareConfigJumperArray )
    {
        // Anzahl Jumperelemente anpassen.
        m_jumpers.resize( hc().jumperCount );

        for ( std::size_t i = 0; i < hc().jumperCount; ++i )
        {
            // Offset auf die Daten des Eintrags lesen.
            const uint32_t &offs = reinterpret_cast<const uint32_t &>( *(buffer.data() + hc().ptrHardwareConfigJumperArray + i * sizeof( uint32_t )) );
            if ( offs > buffer.size() )
            {
                return false;
            }

            //const HARDWARECONFIGJUMPER &hcj = reinterpret_cast<const HARDWARECONFIGJUMPER &>( *(buffer.data() + offs) );

            m_jumpers[i].readSerialize( buffer.begin() + offs );
        }
    }

    // Hat diese Config Submodule oder benötigt eine eventuelle Standardangabe.
    if ( hc().ptrHardwareConfigSubNode )
    {
        // HardwareConfigSubNode lesen
        const HARDWARECONFIGSUBNODE &hcsn = reinterpret_cast<const HARDWARECONFIGSUBNODE &>( *(buffer.data() + hc().ptrHardwareConfigSubNode) );
        if ( hcsn.ptrHardwareConfigArray > buffer.size() )
        {
            return false;
        }

        // Anzahl Kindelemente anpassen.
        m_childs.resize( hcsn.count );

        for ( std::size_t i = 0; i < hcsn.count; ++i )
        {
            // Offset auf die Daten des Eintrags lesen.
            const uint32_t &offs = reinterpret_cast<const uint32_t &>( *(buffer.data() + hcsn.ptrHardwareConfigArray + i * sizeof( uint32_t )) );
            if ( offs > buffer.size() )
            {
                return false;
            }

            const HARDWARECONFIG &hc = reinterpret_cast<const HARDWARECONFIG &>( *(buffer.data() + offs) );
            /*switch ( hc.type )
            {
            case 0: // Hauptplatine
                _childs[hc.number] = ItemBoard( data.begin() + offs, data );
                break;

            case 1: // Module
                _childs[hc.number] = ItemBoard();
                break;

            case 2: // Module mit Plätzen
                _childs[hc.number] = ItemBoard();
                break;

            case 3: // SubModul
                _childs[hc.number] = ItemBoard();
                break;
            }
            */

            // Parent auf dieses setzen.
            m_childs[i].setParent( this );
            // Typen zuweisen.
            m_childs[i].setTypeId( static_cast<EnumItemHardwareInfoType>( hc.type ) );
            // Lesen der Strukturdaten
            if ( !m_childs[i].readSerialize( buffer.begin() + offs, buffer ) )
            {
                return false;
            }
        }
    }
        // Oder benötigt eine eventuelle Standardangabe.
    else if ( autoCorrectModuleDisabled() == false && (hc().id == Module015 || hc().id == Module018 || hc().id == Module019 || hc().id == Module030) )
    {
        getDefault( *this );
    }

    return true;
}

/*!
    \brief Liefert eine Standardkonfiguration für fehlende Steckmodule.

    Wenn in der Hardwarekonfiguration bestimmte Auflötmodule ohne Angabe von Steckmodulen vorgefunden werden,
    wird diesen ein Standard-Steckmodul zugewiesen.

    \param[in] id Id des Auflötmoduls welches eine Standardkonfiguration erhalten soll.
*/
void ItemBoard::getDefault( ItemBoard &item )
{
    std::string article;
    std::string name;

    // !! Achtung !!
    // Um eine Unterscheidung zu den Wirklich eingetragenen Modulen zu erhalten,
    // wird bei der Artikelnummer das Erste Zeichen von einem A auf X gesetzt.

    switch ( item.hc().id )
    {
    case Module015: // M015 Mobile Phone Plug
        // Anzahl Kindelemente anpassen.
        item.m_childs.resize( 1 );
        // ModemMC55i eintragen
        m_childs[0].setParent( &item );
        m_childs[0].setTypeId( SubModule );

        article = "X0100050-A";
        name    = "Mobile MC55i";

        m_childs[0].hc().number   = 1;
        m_childs[0].hc().id       = 105001;
        m_childs[0].hc().revision = 10;
        m_childs[0].hc().index    = 1;
        std::copy( article.begin(), article.end(), m_childs[0].hc().article );
        m_childs[0].hc().type     = static_cast<uint8_t>( SubModule );
        std::copy( name.begin(), name.end(), m_childs[0].hc().name );
        break;

    case Module018: // M018 Grafic Adapter
        // Anzahl Kindelemente anpassen.
        item.m_childs.resize( 1 );
        // Display eintragen
        m_childs[0].setParent( &item );
        m_childs[0].setTypeId( SubModule );

        article = "X0600004";
        name    = "Monocrome Display 3.5\"";

        m_childs[0].hc().number   = 1;
        m_childs[0].hc().id       = 110001;
        m_childs[0].hc().revision = 10;
        m_childs[0].hc().index    = 1;
        std::copy( article.begin(), article.end(), m_childs[0].hc().article );
        m_childs[0].hc().type     = static_cast<uint8_t>( SubModule );
        std::copy( name.begin(), name.end(), m_childs[0].hc().name );
        break;

    case Module019: // M019 Grafic Adapter
        // Anzahl Kindelemente anpassen.
        item.m_childs.resize( 2 );
        // Display eintragen
        m_childs[0].setParent( &item );
        m_childs[0].setTypeId( SubModule );

        article = "X1150001";
        name    = "Color TFT 4.3\" 420x272";

        m_childs[0].hc().number   = 1;
        m_childs[0].hc().id       = 110004;
        m_childs[0].hc().revision = 10;
        m_childs[0].hc().index    = 1;
        std::copy( article.begin(), article.end(), m_childs[0].hc().article );
        m_childs[0].hc().type     = static_cast<uint8_t>( SubModule );
        std::copy( name.begin(), name.end(), m_childs[0].hc().name );

        // Touch eintragen
        m_childs[1].setParent( &item );
        m_childs[1].setTypeId( SubModule );

        article = "X0600101";
        name    = "Touchscreen + SMT-C-T10I";

        m_childs[1].hc().number   = 2;
        m_childs[1].hc().id       = 110101;
        m_childs[1].hc().revision = 10;
        m_childs[1].hc().index    = 1;
        std::copy( article.begin(), article.end(), m_childs[1].hc().article );
        m_childs[1].hc().type     = static_cast<uint8_t>( SubModule );
        std::copy( name.begin(), name.end(), m_childs[1].hc().name );
        break;


    case Module030: // M030 Grafic Adapter
        // Anzahl Kindelemente anpassen.
        item.m_childs.resize( 1 );
        // Display eintragen
        m_childs[0].setParent( &item );
        m_childs[0].setTypeId( SubModule );

        article = "X1140001";
        name    = "Color TFT 2.8\" 320x240";

        m_childs[0].hc().number   = 1;
        m_childs[0].hc().id       = 110003;
        m_childs[0].hc().revision = 11;
        m_childs[0].hc().index    = 1;
        std::copy( article.begin(), article.end(), m_childs[0].hc().article );
        m_childs[0].hc().type     = static_cast<uint8_t>( SubModule );
        std::copy( name.begin(), name.end(), m_childs[0].hc().name );
        break;

	}
}

HARDWARECONFIG &ItemBoard::hc()
{
    return reinterpret_cast<HARDWARECONFIG &>( m_structData );
}

listItemBoard ItemBoard::collectModuleIds( const setModuleId &ids )
{
    listItemBoard tags;
    if ( std::find( ids.begin(), ids.end(), hc().id ) != ids.end() )
    {
        tags.push_back( this );
    }

    // Alle Childs nach dieser Id durchsuchen und zurückliefern.
    auto iter = m_childs.begin();
    auto iend = m_childs.end();
    for ( ; iter != iend; ++iter )
    {
        const listItemBoard &subTags = (*iter).collectModuleIds( ids );
        if ( subTags.size() )
        {
            tags.insert( tags.end(), subTags.begin(), subTags.end() );
        }
    }

    return tags;
}

std::string ItemBoard::idGroup( uint32_t id )
{
    std::string name;

    if ( id >= BoardIdGroup::Standard && id <= BoardIdGroup::StandardEnd && id != 11 ) 
    {
        name = "Standardmodul";
    }
    else if ( id >= BoardIdGroup::Hauptplatine && id <= BoardIdGroup::HauptplatineEnd ) 
    {
        name = "Hauptplatine";
    }
    else if ( id >= BoardIdGroup::Tastatur && id <= BoardIdGroup::TastaturEnd ) 
    {
        name = "Tastatur";
    }
    else if ( id >= BoardIdGroup::Transponderleser && id <= BoardIdGroup::TransponderleserEnd ) 
    {
        name = "Transponderleser";
    }
    else if ( id >= BoardIdGroup::Barcode && id <= BoardIdGroup::BarcodeEnd ) 
    {
        name = "Barcode";
    }
    else if ( id >= BoardIdGroup::Fingerprint && id <= BoardIdGroup::FingerprintEnd ) 
    {
        name = "Fingerprint";
    }
    else if ( (id >= BoardIdGroup::Kommunikation && id <= BoardIdGroup::KommunikationEnd)
        || (id >= 100000 && id <= 100005) || id == 11 ) 
    {
        name = "Kommunikation";
    }   
    else if ( id >= BoardIdGroup::Leuchtflaechen && id <= BoardIdGroup::LeuchtflaechenEnd ) 
    {
        name = "Leuchtfläche";
    }
    else if ( (id >= BoardIdGroup::Displays && id <= BoardIdGroup::DisplaysEnd)
        || id == 100007 ) 
    {
        name = "Display";
    }
    else if ( (id >= BoardIdGroup::Touch && id <= BoardIdGroup::TouchEnd)
        || id == 100019 || id == 100020 ) 
    {
        name = "Touch";
    }
    else if ( moduleGPS_all.count( static_cast<EnumModuleGPS>( id ) ) ) 
    {
        name = "GPS";
    }
    else if ( id == 100006 )
    {
        name = "Zutrittskontrolle";
    }
    else if ( id == 100008 ) 
    {
        name = "Speichererweiterung";
    }
    else if ( id >= 100009 && id <= 100010 ) 
    {
        name = "Wechseldatenträger";
    }
    else if ( id >= 100014 && id <= 100015 ) 
    {
        name = "Barcodeleser";
    }
    else if ( id == 100021 ) 
    {
        name = "Batterie";
    }
    else if ( id == 100022 ) 
    {
        name = "Ladeschaltung";
    }
    else if ( id == 100016 ) 
    {
        name = "Analoge Eingänge";
    }
    else if ( id == 100017 ) 
    {
        name = "Digitale Eingänge";
    }
    else if ( id == 100018 ) 
    {
        name = "Digitale Ausgänge";
    }
    else if ( id >= 150000 && id < 150100 )
    {
        name = "Kamera";
    }
    else
    {
        name = "Unbekannt";
    }

    return name;
}

std::string ItemBoard::toJson()
{
    std::string text;
    std::string temp;

    text = "{";

    text += boost::str( boost::format("\"number\": %d,") % hc().number );
    text += boost::str( boost::format("\"id\": %d,") % hc().id );
    text += boost::str( boost::format("\"idGroup\": %s,") % jsonString( idGroup( hc().id ) ) );
    text += boost::str( boost::format("\"revision\": %d,") % static_cast<int>( hc().revision ) );
    text += boost::str( boost::format("\"index\": %d,") % static_cast<int>( hc().index ) );
    text += boost::str( boost::format("\"article\": %s,") % jsonString( hc().article ) );
    text += boost::str( boost::format("\"type\": %d,") % static_cast<int>( hc().type ) );
    text += boost::str( boost::format("\"name\": %s") % jsonString( hc().name ) );

    // Hat diese Konfiguration Jumperelemente?
    if ( m_jumpers.size() )
    {
        temp = ",\"jumpers\": [";
        auto iter = m_jumpers.begin();
        for ( ; iter != m_jumpers.end(); ++iter )
        {
            temp += (*iter).toJson();
            auto nIter = iter;
            // Ausser bei dem letzten ein Komma.
            if ( ++nIter != m_jumpers.end() )
            {
                temp += ",";
            }
        }
        temp += "]";
        text += temp;
    }

    // Hat diese Konfiguration Submodule.
    if ( m_childs.size() )
    {
        temp = ",\"places\": [";
        auto iter = m_childs.begin();
        for ( ; iter != m_childs.end(); ++iter )
        {
            temp += (*iter).toJson();
            auto nIter = iter;
            if ( ++nIter != m_childs.end() )
            {
                temp += ",";
            }
        }
        temp += "]";
        text += temp;
    }

    text += "}";

    return text;
}

ItemBoard::typeChilds &ItemBoard::childs()
{
    return m_childs;
}

ItemBoard::typeJumpers &ItemBoard::jumpers()
{
    return m_jumpers;
}

ItemDeviceHardwareInfo::ItemDeviceHardwareInfo( ItemHardwareInfoBase *parent, EnumItemHardwareInfoType typeId )
    : ItemHardwareInfoBase( parent, typeId )
{
    // Member initalisieren.
    reset();
}

ItemDeviceHardwareInfo::~ItemDeviceHardwareInfo()
{
}

void ItemDeviceHardwareInfo::reset()
{
    // Alle Childs löschen
    m_childs.clear();

    // Daten löschen.
    std::fill( m_structData.begin(), m_structData.end(), 0 );
}

bool ItemDeviceHardwareInfo::writeSerialize( std::vector<uint8_t> &buffer )
{
    // Es muss mindestens ein Child verfuegbar sein.
    if ( m_childs.size() == 0 )
    {
        return false;
    }

    // Ggf. vorhandenen Pufferspeicher löschen.
    if ( buffer.size() )
    {
        buffer.clear();
    }

    // HardwareConfigNode aufbereiten.
    hcn().version = 1;
    hcn().count = static_cast<uint16_t>( m_childs.size() );
    hcn().ptrHardwareConfigArray = sizeof( HARDWARECONFIGNODE );

    // Fuer Hauptstruktur und alle Childs in dem Array einen Offsetwert vorsehen.
    buffer.resize( sizeof( HARDWARECONFIGNODE ) + m_childs.size() * sizeof( uint32_t ) );

    // HardwareConfigNode eintragen.
    std::copy_n( m_structData.begin(), m_structData.size(), buffer.begin() );

    auto iter = m_childs.begin();
    // Alle Childs und rekursiv dessen Childs nach data serialisieren.
    for ( std::size_t i = 0; iter != m_childs.end(); ++iter, ++i )
    {
        // Aktuelle Puffergröße ist der Begin für dieses Child.
        uint32_t &offs = reinterpret_cast<uint32_t &>( *(buffer.data() + hcn().ptrHardwareConfigArray + i * sizeof( uint32_t )) );
        offs = static_cast<uint32_t>( buffer.size() );

        if ( !(*iter).writeSerialize( buffer ) )
        {
            return false;
        }
    }

    return true;
}

/*! Serialisiert das ByteArray in eine Baumstruktur von ItemHardwareConfigs.
    Aus dem zugrunde gelegten ByteArray wird eine Baumartige Speicherrepresentation
    von ItemHardwareConfigs erzeugt. Über diese kann dann auf einfache
    Art mit den Daten gearbeitet werden.
*/
bool ItemDeviceHardwareInfo::readSerialize( const std::vector<uint8_t> &buffer )
{
    // Hauptstruktur lesen.
    std::copy_n( buffer.begin(), m_structData.size(), m_structData.begin() );

    // Grundlegende Werte prufen.
    if ( hcn().version != 1 )
    {
        return false;
    }

    if ( hcn().ptrHardwareConfigArray < sizeof( HARDWARECONFIGNODE ) || hcn().ptrHardwareConfigArray > buffer.size() )
    {
        return false;
    }

    // Es muss mindestens eine HARDWARECONFIG vorliegen.
    if ( hcn().count == 0 )
    {
        return false;
    }

    // Anzahl Kindelemente anpassen.
    m_childs.resize( hcn().count );

    // Alle Childs laden.
    for ( std::size_t i = 0; i < hcn().count; ++i )
    {
        // Offset auf Daten des Eintrags lesen.
        const uint32_t &offs = reinterpret_cast<const uint32_t &>( *(buffer.data() + hcn().ptrHardwareConfigArray + i * sizeof( uint32_t )) );
        if ( offs > buffer.size() )
        {
            return false;
        }

        const HARDWARECONFIG &hc = reinterpret_cast<const HARDWARECONFIG &>( *(buffer.data() + offs) );
        /*switch ( hc.type )
        {
        case 0: // Hauptplatine
            break;

        case 1: // Module
            break;

        case 2: // Module mit Plätzen
            break;

        case 3: // SubModul
            break;
        }*/
        
        // Setzen des Parent
        m_childs[i].setParent( this );
        // Typen zuweisen.
        m_childs[i].setTypeId( static_cast<EnumItemHardwareInfoType>( hc.type ) );
        // Lesen der einzelnen Strukturdaten
        if ( !m_childs[i].readSerialize( buffer.begin() + offs, buffer) )
        {
            return false;
        }
    }

    return true;
}

HARDWARECONFIGNODE &ItemDeviceHardwareInfo::hcn()
{
    return reinterpret_cast<HARDWARECONFIGNODE &>( m_structData );
}

listItemBoard ItemDeviceHardwareInfo::collectModuleIds( const setModuleId &ids )
{
    listItemBoard tags;

    // Alle Childs nach dieser Id durchsuchen und zurückliefern.
    auto iter = m_childs.begin();
    auto iend = m_childs.end();
    for ( ; iter != iend; ++iter )
    {
        const listItemBoard &subTags = (*iter).collectModuleIds( ids );
        if ( subTags.size() )
        {
            tags.insert( tags.end(), subTags.begin(), subTags.end() );
        }
    }

    return tags;
}

listItemBoard ItemDeviceHardwareInfo::moduleRS232Communication( const setModuleRS232Communication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleRS485Communication( const setModuleRS485Communication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::onBoardRS485Communication( const setOnBoardRS485Communication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleUSBCommunication( const setModuleUSBCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleLANCommunication( const setModuleLANCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::onBoardLANCommunication( const setOnBoardLANCommunication &boards )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : boards )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleWLANCommunication( const setModuleWLANCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleMobileCommunication( const setModuleMobileCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleBluetoothCommunication( const setModuleBluetoothCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleMMCCommunication( const setModuleMMCCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleUSBHostCommunication( const setModuleUSBHostCommunication &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleAccessControl( const setModuleAccessControl &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::onBoardAccessControl( const setOnBoardAccessControl &boards )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : boards )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleBiokey( const setModuleBiokey &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleGPS( const setModuleGPS &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleDisplay( const setModuleDisplay &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleTouch( const setModuleTouch &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

listItemBoard ItemDeviceHardwareInfo::moduleTemperatureSensor( const setModuleTemperatureSensor &module )
{
    // Folgende Module abrufen.
    setModuleId ids;
    for ( uint32_t id : module )
    {
        ids.insert( id );
    }

    return collectModuleIds( ids );
}

std::string ItemDeviceHardwareInfo::toJson()
{
    std::string text;
    std::string temp;
    // Bezeichnung des Geraets
    switch ( hcn().device )
    {
    case 0: temp = "PZE-MasterIV"; break;
    case 1: temp = "ZK-MasterIV"; break;
    case 2: temp = "BDE-MasterIV"; break;
    case 3: temp = "Mobil-MasterIV"; break;
    case 4: temp = "MDE-BoxIV"; break;
    case 5: temp = "AE-MasterIV"; break;
    case 6: temp = "TimeboyIV"; break;
    case 7: temp = "Exklusiv-Line IV"; break;
    case 8: temp = "Flex-MasterIV"; break;
    case 9: temp = "Timeboy Mobil-PZE"; break;
        // Bis hier waren die Nummern identisch mit der HardwareInfo-Vorgängerversion.
    case 10: temp = "IPC Embedded-System A"; break;
    case 11: temp = "EVO 2.8 Pure"; break;
    case 12: temp = "EVO 4.3"; break;
    case 13: temp = "IPC Embedded-System B"; break;
    case 14: temp = "IPC Embedded-System C"; break;
    case 15: temp = "KYO Inloc"; break;
    case 16: temp = "IO-Box V4"; break;
    case 17: temp = "Mobil-Box V4"; break;
    case 18: temp = "Dockingstation V2"; break;
    case 19: temp = "Fahrzeugdatenlogger V2"; break;
    case 20: temp = "KYO Oneloc"; break;
    case 21: temp = "EVO 3.5 Universal"; break;
    case 22: temp = "SAMS-OCL-Reader"; break;
    case 23: temp = "EVO 3.5 Pure"; break;
    case 24: temp = "EVO Agera"; break;
    case 25: temp = "KYO Cenloc"; break;
    case 26: temp = "Vario 10"; break;
    case 27: temp = "Vario 12"; break;
    case 28: temp = "Vario 15"; break;
    case 29: temp = "Vario 17"; break;
    case 30: temp = "IPC EVO 12.1"; break;
    case 31: temp = "IPC EVO 15"; break;
    case 32: temp = "IPC EVO 18.5"; break;
    case 33: temp = "IPC EVO 24"; break;
    case 34: temp = "EVO Intera 2"; break;
    case 35: temp = "EVO 4.6 FlexKey"; break;
    case 36: temp = "EVO 5.0 Pure"; break;
    default: temp = "MasterIV"; break;
    }

    text = "{";
    text += boost::str( boost::format("\"version\": %d,") % static_cast<int>( hcn().version ) );
    text += boost::str( boost::format("\"flags\": %d,") % static_cast<int>( hcn().flags ) );
    text += boost::str( boost::format("\"device\": %d,") % hcn().device );
    text += boost::str( boost::format("\"deviceName\": %s,") % jsonString( temp ) );
    text += boost::str( boost::format("\"serial\": %u,") % hcn().serial );
    text += boost::str( boost::format("\"signature\": %u,") % hcn().signature );
    text += boost::str( boost::format("\"signatureSize\": %u,") % hcn().signatureSize );

    text += "\"places\": [";
    // Für jeden Child den Eintrag anfuegen.
    auto iter = m_childs.begin();
    for ( ; iter != m_childs.end(); ++iter )
    {
        temp = (*iter).toJson();
        auto nIter = iter;
        if ( ++nIter != m_childs.end() )
        {
            temp += ",";
        }

        text += temp;
    }
    text += "]";

    text += "}";

    return text;
}

ItemBoard::typeChilds &ItemDeviceHardwareInfo::childs()
{
    return m_childs;
}

