/**
* @file   studio_setup_structs.h
* @author Bernd Ottmann
* @date   15.02.2002
* @brief  Auflistung der Standarddefinitionen und Strukturen
*/
#pragma once
#ifndef STUDIO_SETUP_STRUCTS_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define STUDIO_SETUP_STRUCTS_H
#endif

#include <cstdint>
#include <vector>
#include <map>
#include <string>

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

#pragma pack(push, 1)

/////////////////////////////////////////////////////////////////////
// Angaben über verwendete Speichergrößen / Startadressen

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define EEP_SIZE             0x1000   ///< Gesamtgröße vom EEPROM des Mikrokontroller (4kByte) 
#define EEP_SIZE_SU          0x0200   ///< Größe des EEPROM die in das Geraet uebertragen wird (512Byte) 
#define EEP_SIZE_PT          0x2C     ///< Reservierter EEPROM - Bereich für Pointereintraege (ersten 44Byte)
#define FLASH_SIZE_SU        0x4000   ///< Größe des FLASH - Bereichs für Setupdaten (16kByte) 
#define FLASH_SIZE_SU2       0x8000   ///< Größe des FLASH - Erweiterung des Bereichs für Setupdaten (32kByte) 
#define EXTENDED_SIZE_SU     0x4000   ///< Größe des Erweiterten FLASH - Bereichs für Setupdaten (16kByte) 
#define FLASH_SIZE           (FLASH_SIZE_SU + FLASH_SIZE_SU2 + EXTENDED_SIZE_SU)
#define CONFIG_SIZE_SU       0x64     ///< Größe des Speicherbereichs für Grunddaten des Geraets (100Byte)
#define FLASH_KONFIG_START   0x0FC000 ///< Startadresse, Absoluter Addressoffset für alle Zeigerwerte.
                                      ///< Der absolute Speicheroffset, wurde ursprünglich Aufaddiert um sich dieses in der Firmware zu ersparen.
#define FLASH_LIST_START     0x0E0000 ///< Startadresse des Listenspeichers, es sind somit 100kByte Gesamtspeicher verfuegbar.
#define FLASH_LIST_START_NEW 0x180000 ///< Startadresse des Listenspeichers, ab 04.00.23 Listengroesse auf 244kByte erhoeht. 
#define FLASH_SIZE_LIST      0x018000 ///< Größe des Listenspeichers (98kByte) 
#define FLASH_SIZE_LIST_NEW  0x03D000 ///< Größe des Listenspeichers ab 04.00.23 (ComVersion 5.2) auf 244kByte erhoeht.
#define MAX_FLASH_MEM        0x100000 ///< Größe des eingesetzten Flashspeichers
#endif

/////////////////////////////////////////////////////////////////////
// Automatische Tastenumschaltung im PZE Modus 2

// Aufbau des Datensatztarray
// ++++++++++++++++++++++++++
// 4 Byte Verwaltung (Version etc. Derzeit immer 0)
// 15 * 4 Byte Datensaetze wobei,
//      erstes  Byte: Funktion
//                    0: Nicht gewaehlt
//                  250: Hauptmenue
//                  251: Letzte Taste
//                  252: Letzte Tasten F1, F2
//                 1..6: Funktionstaste F1 .. F6
//      zweites Byte: Stunde
//      drittes Byte: Minute
//      viertes Byte: Wochentag
//                    Bit 0 .. Bit 6: Montag .. Sonntag

/** Enumeration zur Definition der automatisch zu aktivierenden Funktion.
*/
enum class EnumAutoKeyFunction : uint8_t
{
    None = 0,               ///< Keine Funktion auslösen.
    Key_F1 = 1,             ///< Aktivierung der Funktion 1.
    Key_F2 = 2,             ///< Aktivierung der Funktion 2.
    Key_F3 = 3,             ///< Aktivierung der Funktion 3.
    Key_F4 = 4,             ///< Aktivierung der Funktion 4.
    Key_F5 = 5,             ///< Aktivierung der Funktion 5.
    Key_F6 = 6,             ///< Aktivierung der Funktion 6.
    Key_F7 = 7,             ///< Aktivierung der Funktion 7.
    Key_F8 = 8,             ///< Aktivierung der Funktion 8.
    Key_F9 = 9,             ///< Aktivierung der Funktion 9.
    Key_F10 = 10,           ///< Aktivierung der Funktion 10.
    Key_F11 = 11,           ///< Aktivierung der Funktion 11.
    Key_F12 = 12,           ///< Aktivierung der Funktion 12.
    Key_F13 = 13,           ///< Aktivierung der Funktion 13.
    Key_F14 = 14,           ///< Aktivierung der Funktion 14.
    Key_F15 = 15,           ///< Aktivierung der Funktion 15.
    MainMenu = 250,         ///< Springen zum Hauptmenü.
    LastKey = 251,          ///< Letzte ausgewählte Funktion aktivieren.
    LastKey_F1_F2 = 252     ///< Letzte Funktion (F1 oder F2) aktivieren.
};

/** Enumeration zur Definition der Wochentage, an denen automatisch eine Funktionen ausgelöst werden soll.
Die Enumerationen können kombiniert werden. Jedes Enum steht für ein spezifische Bit.
*/
enum class EnumAutoKeyWeekday : uint8_t
{
    Monday = 0x01,      ///< Bit 1 - Montag
    Tuesday = 0x02,     ///< Bit 2 - Dienstag
    Wednesday = 0x04,   ///< Bit 3 - Mittwoch
    Thursday = 0x08,    ///< Bit 4 - Donnerstag
    Friday = 0x10,      ///< Bit 5 - Freitag
    Saturday = 0x20,    ///< Bit 6 - Samstag
    Sunday = 0x40       ///< Bit 7 - Sonntag
};

/** Diese Struktur dient der Beschreibung einer Aktion, die aufgrund eines generierten Tastaturevents ausgelöst wird.
*/
struct StructAutoKey
{
    EnumAutoKeyFunction function{ EnumAutoKeyFunction::None };  ///< Definiert die auszulösende Funktion.
    uint8_t hour{};     ///< Definiert die Stunde (Uhrzeit) zu der das virtuelle Tastaturevent ausgelöst wird.
    uint8_t minute{};   ///< Definiert die Minute (Uhrzeit) zu der das virtuelle Tastaturevent ausgelöst wird.
    EnumAutoKeyWeekday weekdays{ EnumAutoKeyWeekday::Monday};   ///< Definiert die Wochentage, an denen das virtuelle Tastaturevent ausgelöst wird.
}; // 4 Byte

/** Diese Struktur beinhaltet die Definition aller virtuellen Tastaturevents.
*/
struct StructAutoKeys
{
    uint32_t version{};         ///< Beinhaltet die Versionskennung.
    StructAutoKey key[15]{};    ///< Liste der definierten virtuellen Tastaturevents.
                                ///< Maximal 15 Stück (statisch).
}; // 64 Byte


/////////////////////////////////////////////////////////////////////
// Setuptype

/** Da die Setupdateien für unterschiedliche Geräte sein können, wird ein mapping des Gerätetypes auf den Setuptypen benötigt.

Aufgrund der Historie sind die Setuptypen meist der Gerätetyp - 1. Wurde ein Setuptype für unterschiedliche Geräte verwendet
wurde zumeist der erste Gerätetyp verwendet z. B. EVO 2.8 Pure (deviceType = 10)
setupType = 11 gilt auch für EVO 3.5 Pure (deviceType = 23)
*/
enum class SetupType : uint8_t
{
    AEIII_Plus = 0,         ///< AEIII+
    PZE_MasterIV = 1,       ///< PZE-MasterIV & Basic
    ZK_MasterIV = 2,        ///< ZK-MasterIV
    BDE_MasterIV = 3,       ///< BDE-MasterIV
    Mobil_MasterIV = 4,     ///< Mobil-MasterIV
    MDE_BoxIV = 5,          ///< MDE-BoxIV
    AE_MasterIV = 6,        ///< AE-MasterIV
    Timeboy_MasterIV = 7,   ///< TimeboyIV
    Exklusiv_LineIV = 8,    ///< Exklusiv-Line IV
    Flex_MasterIV = 9,      ///< Flex-MasterIV
    Timeboy_Mobil_PZE = 10, ///< Timeboy-Mobil-PZE
    EVO_2_8_Pure = 11,      ///< EVO 2.8 Pure / EVO 3.5 Pure / EVO 5.0 Pure
    EVO_4_3 = 12,           ///< EVO 4.3 / EVO 4.6 FlexKey
    IPC_Extension = 13,     ///< IPC-Extensions (Vario 5.7, Vario 10, Q7, Vario 10 - 17, IPC EVO 12.1 - 24)
    IO_Box = 16,            ///< IO-Box
    KYO_Inloc = 17,         ///< KYO Inloc
    Mobil_Box = 18,         ///< Mobil-Box
    Timeboy_TBDock = 19,    ///< Dockingstation-V2
    Mobil_TBDock = 20,      ///< Fahrzeugdatenlogger-V2
    KYO_Oneloc = 21,        ///< KYO Oneloc
    EVO_3_5_Universal = 22, ///< EVO 3.5 Universal
    KYO_Cenloc = 26         ///< KYO Cenloc
};

#ifndef Q_MOC_RUN
#ifdef Q_DECLARE_METATYPE

// Für die Verwendung mit Qt ist das Enum im Meta-System bekannt zu machen.
Q_DECLARE_METATYPE( SetupType )

QT_BEGIN_NAMESPACE
// Für die Verwendung mit QSet ist zusätzlich eine globale Hashfunktion anzugeben.
static uint8_t qHash( SetupType setupType )
{
    return static_cast<uint8_t>(setupType);
}
QT_END_NAMESPACE
#endif
#endif

static const std::map<SetupType, std::string> SetupTypeName = {
    { SetupType::AEIII_Plus,           "AEIII+" },
    { SetupType::PZE_MasterIV,         "PZE-MasterIV & Basic" },
    { SetupType::ZK_MasterIV,          "ZK-MasterIV" },
    { SetupType::BDE_MasterIV,         "BDE-MasterIV" },
    { SetupType::Mobil_MasterIV,       "Mobil-MasterIV" },
    { SetupType::MDE_BoxIV,            "MDE-BoxIV" },
    { SetupType::AE_MasterIV,          "AE-MasterIV" },
    { SetupType::Timeboy_MasterIV,     "TimeboyIV" },
    { SetupType::Exklusiv_LineIV,      "Exklusiv-Line IV" },
    { SetupType::Flex_MasterIV,        "Flex-MasterIV" },
    { SetupType::Timeboy_Mobil_PZE,    "TimeboyIV Mobil-PZE" },
    { SetupType::EVO_2_8_Pure,         "EVO 2.8 & 3.5 & 5.0 Pure" },
    { SetupType::EVO_4_3,              "EVO 4.3 & 4.6 FlexKey" },
    { SetupType::IPC_Extension,        "IPC Embedded-System" },
    { SetupType::IO_Box,               "IO-Box V4" },
    { SetupType::KYO_Inloc,            "KYO Inloc" },
    { SetupType::Mobil_Box,            "Mobil-Box V4" },
    { SetupType::Timeboy_TBDock,       "Dockingstation V2" },
    { SetupType::Mobil_TBDock,         "Fahrzeugdatenlogger V2" },
    { SetupType::KYO_Oneloc,           "KYO Oneloc" },
    { SetupType::EVO_3_5_Universal,    "EVO 3.5 Universal" },
    { SetupType::KYO_Cenloc,           "KYO Cenloc" }
};

// Angabe der Reihenfolge in der die Typen aufgelistet werden.
// Achtung: AEIII_Plus und Exklusiv_LineIV werden nicht weiter unterstützt und sind somit nicht enthalten.
static const std::vector<SetupType> SetupTypeOrder = {
    SetupType::IPC_Extension,
    SetupType::EVO_4_3,
    SetupType::EVO_2_8_Pure,
    SetupType::EVO_3_5_Universal,
    SetupType::PZE_MasterIV,
    SetupType::AE_MasterIV,
    SetupType::Timeboy_MasterIV,
    SetupType::Timeboy_Mobil_PZE,
    SetupType::KYO_Cenloc,
    SetupType::KYO_Oneloc,
    SetupType::KYO_Inloc,
    SetupType::IO_Box,
    SetupType::Mobil_Box,
    SetupType::Timeboy_TBDock,
    SetupType::Mobil_TBDock,
    SetupType::ZK_MasterIV,
    SetupType::MDE_BoxIV,
    SetupType::Mobil_MasterIV,
    SetupType::BDE_MasterIV,
    SetupType::Flex_MasterIV
};

/////////////////////////////////////////////////////////////////////
// Formate für Zeigerablage

/////////////////////////////////////////////////////////////////////
// Versionsnummern, 
// Zeiger die im EEPROM - Bereich liegen und auf das Flash verweisen.

/** Struktur zur Definition der Flash-Struktur.
Beinhaltet Versionsinformationen und die Zeiger, die im EEPROM-Bereich liegen und auf das Flash verweisen.
*/
struct StructFlashHeader
{
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint16_t unused_0{};                ///< Wird durch DatafoxStudioIV-Versionsstände < 04.03.11 auf 0 gesetzt.
#endif
    uint16_t version{};                 ///< Kommunikationsversionsnummer des Setup. (16bit, seit Studio 04.02.02.08 fix 0x0204)
    uint32_t config{};                  ///< Verwendeter absoluter Adressoffset. Bis Studio 04.03.11 war Big-Endien
    uint8_t headline[18]{};             ///< Kopfzeile im Hauptmenü (17 Zeichen + Nullterminie)
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_1[10]{};
#endif
    uint32_t listStruct{};              ///< Startadresse Listenbeschreibung
    uint32_t list[20]{};                ///< Adresse der Daten der Liste Nr. 0 im Flash. Die weiteren Adressen sind nachfolgend gespeichert. z.B. Adresse von Liste Nr. 4:  P_LIST_0 + 4 * sizeof(uint32_t) 
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_2[8]{};
#endif
    uint32_t dataStruct{};              ///< Kopie von P_DAT_STRUCT, um Fehler von ATmega zu umgehen
    uint32_t flashSizeSUS{};            ///< Größe der Flashdaten
    uint32_t inputChainSizeMDE{};       ///< Größe der MDE-Eingabeketten
    uint32_t inputChainSizeEntrance{};  ///< Größe der Eingabekette der Zutrittskontrolle
    uint8_t listKeyField[20]{};         ///< Für alle 20 möglichen LSBs je ein Byte für ein Schlüsselfeld reservieren. 0=Keines zugewiesen, 1..MAX_FIELDS+1=Feldindex
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_3{};
#endif
    StructAutoKeys autoKeys;        ///< Tastenumschaltung
    uint32_t studioVersion{};       ///< Ab: 03.01.30.00; Es werden in vier Byte die Versionsnummer des Studio mit Buildnumber abgelegt
    SetupType setupType{ SetupType::AEIII_Plus };   ///< Ab: 03.01.31.00; Es wird in einem Byte der Setuptyp abgespeichert, für welche Geräte dieses Setup verwendet wird. 
    uint32_t listAccess{};          ///< Zeiger auf Strukturen der Zutrittskontrolle: FW 04.01.01
    uint32_t inputChainAccess{};    ///< Zeiger auf Strukturen der Zutrittskontrolleingabeketten : FW 04.01.01
    uint16_t fileVersion{};         ///< Ab: 04.01.06.09; Es wird eine Dateiversion der Form 0xDF01 (16kByte Setupspeicher), 0xDF02 (48kByte Setupspeicher) abgelegt.
    uint32_t headerSize{};          ///< Ab: 04.02.03/04; sizeof(TAESHeader) für die aktuell verwendete Headerstruktur.
    uint32_t extendedSize{};        ///< Ab: 04.02.03/04; Größe des erweiterten Speicherbereichs.
    uint32_t libraryVersion{};      ///< Ab: 04.03.14/06; Es wird in vier Byte die Versionsnummer der zur Übertragung benutzten Version der Kommunikationsbibliothek (DFCom) inkl. Buildnummer abgelegt.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_4[256]{};
#endif
}; // 516 Byte

/////////////////////////////////////////////////////////////////////
// Strukturdeklarationen
// Es werden immer erst die defines eingefuehrt, welche in den Strukturen
// und allen weiteren evntl. benoetigt werden. Danach folgt die jeweilige Strukturdeklaration.

///////////////////////////////////////
// Struct: StructTransponder

// Flags für TP StructTransponder.tbModeX
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define BDTP_ACTIVE          0x80 ///< StructTransponder.tpMode: Einstellung für Transponder aktiviert.
//  lesen / schreiben
#define BDTP_WRITE           0x40 ///< StructTransponder.tpMode: Der Wert wird ins Segment geschrieben, ansonsten gelesen.

#define BDTP_DEC             0x00 ///< StructTransponder.tpMode: Behandlung der Daten als Dezimalwert.
#define BDTP_HEX             0x10 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert.
#define BDTP_ASCII           0x20 ///< StructTransponder.tpMode: Behandlung der Daten als ASCII - Zeichenfolge.
#define BDTP_HEX_TURNED      0x30 ///< StructTransponder.tpMode: Behandlung der Daten als Hexadezimalwert aber umgekehrte Bytefolge.
#endif

// Untere 4bit sind Anzahl feste stellen

/** Diese Struktur dient der Spezifikation, bei Angabe eines Hitag-Transponders, der zu verwendenden Segmente, Ablageformate sowie der Länge.

Diese Struktur kommt als Erweiterung in der Struktur @ref StructNumFieldEx zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle in @ref StructConfig abzulegen.
*/
struct StructTransponder
{
    uint8_t tpSegment1{};   ///< Gibt das erste Segment an, aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
    uint8_t tpMode1{};      ///< Gibt den Modus des Wertes an.
                            ///< Oberes Nibble beinhaltet oben definierte Flags, unteres die feste Längenangabe.
    uint8_t tpSegment2{};   ///< Siehe Beschreibung zu @ref tpSegment1. Hier für ein zweites Segment.
    uint8_t tpMode2{};      ///< Siehe Beschreibung zu @ref tpMode1. Hier für einen zweiten Mode.
    uint8_t tpSegment3{};   ///< Siehe Beschreibung zu @ref tpSegment1. Hier für ein drittes Segment.
    uint8_t tpMode3{};      ///< Siehe Beschreibung zu @ref tpMode1. Hier für einen dritten Mode.
}; // 6 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines MIFARE-Transponders, des zu verwendenden Sektors, Ablageformates sowie der Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur @ref StructNumFieldEx zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle in @ref StructConfig abzulegen.
*/
struct StructTransponderMifare
{
    uint8_t sectorBlock{};      ///< Gibt im oberen Nibble den Sektor, im unteren den Block an, aus dem der Wert gelesen / oder in den der Wert geschieben werden soll.
    uint8_t mode{};             ///< Oberes Nibble beinhaltet oben definierte Flags für das Datenformat, unteres die feste Längenangabe.
                                ///< Bei Hex-Format ist der Wert mal 2 zu nehmen.
    uint8_t cut{};              ///< Gibt im oberen Nibble die Startposition des ersten zu lesenden Bytes der 16 möglichen an.
                                ///< Das untere Nibble ist frei.
    uint8_t passwordMode{};     ///< Gibt den zu verwendenden Passwortbereich an.
    uint8_t securityLevel{};    ///< Gibt den zu verwendenden Security-Level des MIFARE Plus Ausweises an.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused{};
#endif
}; // 6 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines HID iClass Transponders, des zu verwendenden Ausweistyps, Ablageformates sowie der Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur @ref StructNumFieldEx zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle in @ref StructConfig abzulegen.
*/
struct StructTransponderHIDiClass
{
    uint8_t type{};     ///< Gibt den zu verwendenden Ausweistyp an.
                        ///< |Typ|Bedeutung         |
                        ///< |:-:|:-----------------|
                        ///< |0  |H10301            |
                        ///< |1  |H10302            |
                        ///< |2  |H10304            |
                        ///< |3  |freies Datenformat|
    uint8_t format{};   ///< Ablageformat.
                        ///< |Format|Bedeutung          |
                        ///< |:----:|:------------------|
                        ///< |0     |Dezimal            |
                        ///< |1     |Hexadezimal        |
                        ///< |2     |-FREI-             |
                        ///< |3     |Hexadezimal gedreht|
    uint8_t length{};   ///< Ablage feste Längenangabe.
                        ///< |Länge|Bedeutung                                  |
                        ///< |:---:|:------------------------------------------|
                        ///< |0    |keine feste Länge                          |
                        ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[3]{};
#endif
}; // 6 Byte

///////////////////////////////////////
// Struct: StructTransponderLegicAdvantOld

/** Diese Struktur dient der Spezifikation, bei Angabe eines Legic Advant Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.

Diese Struktur wird verwendet um globale Angaben für Legic Advant Ausweise in der Firmwareversion
04.02.02 aufzunehmen. Ab Version 04.02.03 wird auf die @ref StructTransponderLegicAdvant ausgewichen.
*/
struct StructTransponderLegicAdvantOld
{
    uint8_t segment{};          ///< Gibt das zu verwendende Segment an.
                                ///< Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
    uint16_t pos{};             ///< Position des zuerst gelesenen Datenbytes.
    uint8_t length{};           ///< Anzahl zu lesender Datenbytes.
    uint8_t format{};           ///< Ablageformat.
                                ///< |Format|Bedeutung          |
                                ///< |:----:|:------------------|
                                ///< |0     |Dezimal            |
                                ///< |1     |Hexadezimal        |
                                ///< |2     |ASCII              |
                                ///< |3     |Hexadezimal gedreht|
    uint8_t formatLength{};     ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
    uint8_t typeFlags{};        ///< Enthält die gewünschten Ausweistypen und zusätzliche Flags.
                                ///< - Bit 8 und 7 (Maske 0xC0) beinhaltet den Ausweistyp
                                ///< |Wert|Bedeutung                 |
                                ///< |:--:|:-------------------------|
                                ///< |0   |nur Advant-Ausweise       |
                                ///< |1   |nur Prime-Ausweise        |
                                ///< |2   |Advant- und Prime-Ausweise|
                                ///< |3   |-FREI-                    |
                                ///< - Bit 6 und 5 (Maske 0x30) CRC-Prüfung
                                ///< |Wert|Bedeutung                 |
                                ///< |:--:|:-------------------------|
                                ///< |0   |keine Prüfung|
                                ///< |1   |8-Bit        |
                                ///< |2   |16-Bit       |
                                ///< |3   |-FREI-       |
                                ///< - Bit 4
                                ///< |Wert|Bedeutung                      |
                                ///< |:--:|:------------------------------|
                                ///< |1   |Access Segment Definition aktiv|
                                ///< - Bit 3
                                ///< |Wert|Bedeutung                  |
                                ///< |:--:|:--------------------------|
                                ///< |1   |ISO14443-Ausweis zugelassen|
                                ///< - Bit 2
                                ///< |Wert|Bedeutung                  |
                                ///< |:--:|:--------------------------|
                                ///< |1   |ISO15693-Ausweis zugelassen|
                                ///< - Bit 1
                                ///< |Wert|Bedeutung                                                 |
                                ///< |:--:|:---------------------------------------------------------|
                                ///< |1   |Seriennummern von Prime-Ausweisen im Advant-Format liefern|
    uint8_t valType{};          ///< Zu Lesender Wertetyp.
                                ///< |Wert|Bedeutung                                  |
                                ///< |:--:|:------------------------------------------|
                                ///< |0   |Seriennummer                               |
                                ///< |1   |freie Dateiauswahl                         |
                                ///< |2   |Kaba Group Header von Prime-Ausweisen lesen|
                                ///< |3   |Segment mittels Suche finden               |
                                ///< |>3  |-FREI-                                     |
    uint16_t crcAddress{};      ///< Position des CRC
    uint8_t kghSearch[13]{};    ///< Zu suchende Bytefolge für den Kaba Group Header
    uint8_t kghSearchLength{};  ///< Länge der zu suchenden Bytefolge des Kaba Group Header
    uint8_t flags{};            ///< Enthält zusätzliche Flags
                                ///< |Bit|Bedeutung (wenn gesetzt)              |
                                ///< |:-:|:-------------------------------------|
                                ///< |1  |Suche nach Kaba Group Header          |
                                ///< |^  |Nur bei Ausweisen vom Typ Legic Prime.|
                                ///< |n  |-FREI-                                |
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[5]{};        ///< Reserve für Erweiterungen
#endif
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderLegicAdvantHeader

/** Diese Struktur enthält zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderLegicAdvantHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur.
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge im Offsetarray, was auf diese Struktur folgt.
    uint8_t mode{};             ///< Enthält die gewünschten Ausweistypen.
                                ///< |Wert|Bedeutung                 |
                                ///< |:--:|:-------------------------|
                                ///< |0   |nur Advant-Ausweise       |
                                ///< |1   |nur Prime-Ausweise        |
                                ///< |2   |Advant- und Prime-Ausweise|
    uint8_t flags{};            ///< Enthält zusätzliche Flags.
                                ///< |Bit|Bedeutung (wenn gesetzt)              |
                                ///< |:-:|:-------------------------------------|
                                ///< |1  |ISO14443-Ausweise zugelassen          |
                                ///< |2  |ISO15693 Ausweise zugelassen          |
                                ///< |3  |Access Segment Definition aktiv       |
                                ///< |4  |Suche nach Kaba Group Header          |
                                ///< |^  |Nur bei Ausweisen vom Typ Legic Prime.|
}; // 6 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines Legic Advant Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur @ref StructNumFieldEx zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle im @ref StructConfig abzulegen.
*/
struct StructTransponderLegicAdvant
{
    uint8_t rwType{};           ///< Zu Lesender Wertetyp.
                                ///< |Wert|Bedeutung                                  |
                                ///< |:--:|:------------------------------------------|
                                ///< |0   |Seriennummer                               |
                                ///< |1   |freie Dateiauswahl                         |
                                ///< |2   |Kaba Group Header von Prime-Ausweisen lesen|
                                ///< |>2  |-FREI-                                     |
    uint8_t rwFlags{};          ///< Enthält zusätzliche Flags.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                   |
                                ///< |:--:|:---------------------------------------------------------|
                                ///< |1   |Seriennummern von Prime-Ausweisen im Advant-Format liefern|
    uint8_t rwSegment{};        ///< Gibt das zu verwendende Segment an.
                                ///< Segment 0 bedeutet, dass der Ausweis nicht segmentiert ist.
    uint16_t rwOffset{};        ///< Position des zuerst gelesenen Datenbytes.
    uint16_t rwLength{};        ///< Anzahl zu lesender Datenbytes.
    uint8_t kghSearch[13]{};    ///< Zu suchende Bytefolge für den Kaba Group Header.
    uint8_t kghSearchLength{};  ///< Länge der zu suchenden Bytefolge des Kaba Group Header.
    uint8_t crcCheck{};         ///< Art der CRC-Prüfung
                                ///< |Wert|Bedeutung    |
                                ///< |:--:|:------------|
                                ///< |0   |keine Prüfung|
                                ///< |1   |8-Bit        |
                                ///< |2   |16-Bit       |
    uint16_t crcAddress{};      ///< Position des CRC
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |ASCII              |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[3]{};
#endif
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderMifareDesfireHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderMifareDesfireHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur.
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge im Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines MIFARE Desfire Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderMifareDesfire
{
    uint8_t rwFlags{};              ///< Gibt durch Flags folgendes an.
                                    ///< |Bit|Bedeutung (wenn gesetzt)                   |
                                    ///< |:-:|:------------------------------------------|
                                    ///< |1  |UID lesen (Seriennummer)                   |
                                    ///< |2  |echte UID bei Random UID Ausweisen lesen   |
                                    ///< |3  |UID auch von anderen MIFARE-Ausweisen lesen|
                                    ///< |4  |UID mit einer Länge von 4 Byte liefern     |
                                    ///< |5-8|-FREI-                                     |
    uint32_t applicationNumber{};   ///< Nummer der zu verwendenden Applikation.
    uint8_t fileNumber{};           ///< Nummer der zu verwendenden Datei.
    uint16_t rwOffset{};            ///< Offset in die Daten der Datei.
    uint16_t rwLength{};            ///< Anzahl der zu lesenden Datenbytes aus der Datei.
    uint8_t fileType{};             ///< Type der Datei.
                                    ///< |Wert|Bedeutung|
                                    ///< |:--:|:--------|
                                    ///< |0   |Standard |
                                    ///< |1   |Backup   |
    uint8_t communicationMode{};    ///< Modus der Kommunikation zwischen Ausweis und Leser.
                                    ///< |Wert|Bedeutung|
                                    ///< |:--:|:--------|
                                    ///< |0   |Plain    |
                                    ///< |1   |MACed    |
                                    ///< |2   |Crypt    |
    uint8_t keyNumber{};            ///< Nummer des zu verwendenden Schlüssels.
    uint8_t keyFlags{};             ///< Passwortflags
                                    ///< |Bit|Bedeutung (wenn gesetzt)                    |
                                    ///< |:-:|:-------------------------------------------|
                                    ///< |1  |Passwort liegt im verschlüsselten Format vor|
                                    ///< |2-8|-FREI-                                      |
    uint8_t applicationKey[32]{};   ///< Zu verwendender Anwendungsschlüssels.
    uint8_t cryptType{};            ///< Typ der Verschlüsselung.
                                    ///< |Wert|Bedeutung                 |
                                    ///< |:--:|:-------------------------|
                                    ///< |0   |DF4    (16 Byte Schlüssel)|
                                    ///< |1   |TDES   (16 Byte Schlüssel)|
                                    ///< |2   |AES    (16 Byte Schlüssel)|
                                    ///< |3   |3KTDES (24 Byte Schlüssel)|
    uint8_t outputFormat{};         ///< Gibt das abzulegende Werteformat an.
                                    ///< |Wert|Bedeutung          |
                                    ///< |:--:|:------------------|
                                    ///< |0   |Dezimal            |
                                    ///< |1   |Hexadezimal        |
                                    ///< |2   |ASCII              |
                                    ///< |3   |Hexadezimal gedreht|
                                    ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};        ///< Gibt die abzulegende Werteformatlänge an.
                                    ///< |Länge|Bedeutung                                  |
                                    ///< |:---:|:------------------------------------------|
                                    ///< |0    |keine feste Länge                          |
                                    ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[10]{};
#endif
}; // 60 Byte


///////////////////////////////////////
// Struct: StructTransponderMifarePlusHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderMifarePlusHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
    uint8_t securityLevel{};    ///< Anzuwendender Sicherheitslevel.
                                ///< |Wert|Bedeutung                            |
                                ///< |:--:|:------------------------------------|
                                ///< |0   |Security Level 0 (Ausweis ist blanko)|
                                ///< |1   |Security Level 1                     |
                                ///< |2   |Security Level 2                     |
                                ///< |3   |Security Level 3                     |
    uint8_t iso14443A{};        ///< Gibt im Security Level 1/2 an welche Ausweistypen von ISO14443A akzeptiert werden.
                                ///< |Bit|Bedeutung (wenn gesetzt)|
                                ///< |:-:|:-----------------------|
                                ///< |1  |ISO14443A Mini          |
                                ///< |2  |ISO14443A Classic 1k    |
                                ///< |3  |ISO14443A Classic 4k    |
                                ///< |4  |ISO14443A Plus S        |
                                ///< |5  |ISO14443A Plus X        |
}; // 6 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines MIFARE Plus Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderMifarePlus
{
    uint8_t rwFlags{};              ///< Gibt durch Flags folgendes an.
                                    ///< |Bit|Bedeutung (wenn gesetzt)                   |
                                    ///< |:-:|:------------------------------------------|
                                    ///< |1  |UID lesen (Seriennummer)                   |
                                    ///< |2  |echte UID bei Random UID Ausweisen lesen   |
                                    ///< |3  |UID auch von anderen MIFARE-Ausweisen lesen|
                                    ///< |4  |UID mit einer Länge von 4 Byte liefern     |
                                    ///< |5-8|-FREI-                                     |
    uint8_t rwSector{};             ///< Nummer des zu verwendenden Sektors.
    uint8_t rwBlock{};              ///< Nummer des zu verwendenden Blocks.
    uint8_t rwOffset{};             ///< Offset in den Datenblock.
    uint16_t rwLength{};            ///< Anzahl der zu lesenden Datenbytes aus dem Sektor/Block.
    uint8_t passwordUsage{};        ///< Passwortverwendung für Lesen / Schreiben.
                                    ///< |Wert|Bedeutung  |
                                    ///< |:--:|:----------|
                                    ///< |0   |KeyA / KeyA|
                                    ///< |1   |KeyA / KeyB|
                                    ///< |2   |KeyB / KeyB|
    uint8_t passwordFlags{};        ///< Passwortflags
                                    ///< |Bit|Bedeutung (wenn gesetzt)                             |
                                    ///< |:-:|:----------------------------------------------------|
                                    ///< |1  |für Lesen auch die Standardpasswörter verwenden      |
                                    ///< |2  |passwordA_Crypto1 liegt im verschlüsselten Format vor|
                                    ///< |3  |passwordB_Crypto1                                    |
                                    ///< |4  |passwordA_AES liegt im verschlüsselten Format vor    |
                                    ///< |5  |passwordB_AES liegt im verschlüsselten Format vor    |
                                    ///< |6  |instID liegt im verschlüsselten Format vor           |
                                    ///< |7-8|-FREI-                                               |
    uint8_t passwordA_Crypto1[6]{}; ///< Zu verwendendes Passwort A für Crypto 1 (nur im Security Level 1/2)
    uint8_t passwordB_Crypto1[6]{}; ///< Zu verwendendes Passwort B für Crypto 1 (nur im Security Level 1/2)
    uint8_t passwordA_AES[16]{};    ///< Zu verwendendes Passwort A im Security level 2 / 3.
                                    ///< Zu verwendender Virtual Card Polling Encryption key im SL 3 bei Random UID
    uint8_t passwordB_AES[16]{};    ///< Zu verwendendes Passwort B.
                                    ///< Zu verwendender Virtual Card Polling MAC key im SL 3 bei Random UID
    uint8_t instID[16]{};           ///< Zu verwendender Installation Identifier im SL 3 bei Random UID
    uint8_t outputFormat{};         ///< Gibt das abzulegende Werteformat an.
                                    ///< |Wert|Bedeutung          |
                                    ///< |:--:|:------------------|
                                    ///< |0   |Dezimal            |
                                    ///< |1   |Hexadezimal        |
                                    ///< |2   |ASCII              |
                                    ///< |3   |Hexadezimal gedreht|
                                    ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};        ///< Gibt die abzulegende Werteformatlänge an.
                                    ///< |Länge|Bedeutung                                  |
                                    ///< |:---:|:------------------------------------------|
                                    ///< |0    |keine feste Länge                          |
                                    ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[9]{};
#endif
}; // 80 Byte


///////////////////////////////////////
// Struct: StructTransponderMifareUltralightHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderMifareUltralightHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines MIFARE Ultralight Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderMifareUltralight
{
    uint8_t rwFlags{};          ///< Gibt durch Flags folgendes an.
                                ///< |Bit|Bedeutung (wenn gesetzt)|
                                ///< |:-:|:-----------------------|
                                ///< |1  |UID lesen (Seriennummer)|
                                ///< |2-8|-FREI-                  |
    uint8_t rwBlock{};          ///< Nummer des zu verwendenden Blocks.
    uint16_t rwOffset{};        ///< Offset in den Block.
    uint16_t rwLength{};        ///< Anzahl der zu lesenden Datenbytes.
    uint8_t keyUsage{};         ///< Verwendung des Schlüssels.
                                ///< |Wert|Bedeutung                                        |
                                ///< |:--:|:------------------------------------------------|
                                ///< |0   |kein Passwort                                    |
                                ///< |1   |Ausweise ohne und mit diesem Passwort akzeptieren|
                                ///< |2   |nur Ausweise mit diesem Passwort akzeptieren     |
    uint8_t keyFlags{};         ///< Passwortflags
                                ///< |Bit|Bedeutung (wenn gesetzt)                    |
                                ///< |:-:|:-------------------------------------------|
                                ///< |1  |Passwort liegt im verschlüsselten Format vor|
                                ///< |2-8|-FREI-                                      |
    uint8_t key[16]{};          ///< Zu verwendender Schlüssel.
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |ASCII              |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[3]{};
#endif
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderiCodeHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderiCodeHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines iCode Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderiCode
{
    uint8_t rwFlags{};          ///< Gibt durch Flags folgendes an.
                                ///< |Bit|Bedeutung (wenn gesetzt)|
                                ///< |:-:|:-----------------------|
                                ///< |1  |UID lesen (Seriennummer)|
                                ///< |2-8|-FREI-                  |
    uint8_t rwBlock{};          ///< Nummer des zu verwendenden Blocks.
    uint16_t rwOffset{};        ///< Offset in den Block.
    uint16_t rwLength{};        ///< Anzahl der zu lesenden Datenbytes.
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |ASCII              |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[6]{};
#endif
}; // 15 Byte


///////////////////////////////////////
// Struct: StructTransponderJavaCardHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderJavaCardHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines JavaCard Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderJavaCard
{
    uint8_t rwFlags{};          ///< Gibt durch Flags folgendes an.
                                ///< |Bit|Bedeutung (wenn gesetzt)|
                                ///< |:-:|:-----------------------|
                                ///< |1  |UID lesen (Seriennummer)|
                                ///< |2-8|-FREI-                  |
    uint8_t rwBlock{};          ///< Nummer des zu verwendenden Blocks.
    uint16_t rwOffset{};        ///< Offset in den Block.
    uint16_t rwLength{};        ///< Anzahl der zu lesenden Datenbytes.
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |ASCII              |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[6]{};        ///< Reserve für Erweiterungen
#endif
}; // 15 Byte


///////////////////////////////////////
// Struct: StructTransponderMyDHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderMyDHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
    uint8_t isoFlags{};         ///< Enthält die zugelassenen Ausweistypen.
                                ///< |Bit|Bedeutung (wenn gesetzt)    |
                                ///< |:-:|:---------------------------|
                                ///< |1  |ISO14443-Ausweise zugelassen|
                                ///< |2  |ISO15693-Ausweise zugelassen|
                                ///< |3-8|-FREI-                      |
}; // 5 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines MyD Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderMyD
{
    uint8_t rwFlags{};          ///< Gibt durch Flags folgendes an.
                                ///< |Bit|Bedeutung (wenn gesetzt)|
                                ///< |:-:|:-----------------------|
                                ///< |1  |UID lesen (Seriennummer)|
                                ///< |2-8|-FREI-                  |
    uint16_t rwBlock{};         ///< Nummer des zu verwendenden Blocks.
    uint16_t rwOffset{};        ///< Offset in den Block.
    uint16_t rwLength{};        ///< Anzahl der zu lesenden Datenbytes.
    uint8_t keyUsage{};         ///< Verwendung des Schlüssels.
                                ///< |Wert|Bedeutung                                        |
                                ///< |:--:|:------------------------------------------------|
                                ///< |0   |kein Passwort                                    |
                                ///< |1   |Ausweise ohne und mit diesem Passwort akzeptieren|
                                ///< |2   |nur Ausweise mit diesem Passwort akzeptieren     |
    uint8_t keyFlags{};         ///< Passwortflags
                                ///< |Bit|Bedeutung (wenn gesetzt)                |
                                ///< |:-:|:---------------------------------------|
                                ///< |1  |keyA liegt im verschlüsselten Format vor|
                                ///< |2  |keyB liegt im verschlüsselten Format vor|
                                ///< |3-8|-FREI-                                  |
    uint8_t keyA[8]{};          ///< Zu verwendender Schlüssel A.
    uint8_t keyB[8]{};          ///< Zu verwendender Schlüssel B.
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |ASCII              |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[2]{};
#endif
}; // 30 Byte


///////////////////////////////////////
// Struct: StructTransponderISOTagsHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderISOTagsHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines ISO Tags Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderISOTags
{
    uint8_t isoType{};      ///< Gibt durch Flags an, welche ISO Ausweise gelesen werden sollen.
                            ///< |Bit|Bedeutung (wenn gesetzt)|
                            ///< |:-:|:-----------------------|
                            ///< |1  |ISO14443A               |
                            ///< |2  |ISO14443B               |
                            ///< |3  |ISO14443-2              |
                            ///< |4  |ISO14443-3              |
                            ///< |5  |ISO15693                |
                            ///< |6-8|-FREI-                  |
    uint8_t iso14443A{};    ///< Gibt an welche Ausweistypen von ISO14443A gelesen werden sollen.
                            ///< |Bit|Bedeutung (wenn gesetzt)|
                            ///< |:-:|:-----------------------|
                            ///< |1  |ISO14443A Mini          |
                            ///< |2  |ISO14443A Classic 1k    |
                            ///< |3  |ISO14443A Classic 4k    |
                            ///< |4  |ISO14443A Plus S        |
                            ///< |5  |ISO14443A Plus X        |
                            ///< |6  |ISO14443A Ultralight    |
                            ///< |7  |ISO14443A DESFire       |
                            ///< |8  |ISO14443A LegicAdvant   |
    uint8_t iso14443A_2{};  ///< Gibt an welche Ausweistypen von ISO14443A gelesen werden sollen.
                            ///< |Bit|Bedeutung (wenn gesetzt)        |
                            ///< |:-:|:-------------------------------|
                            ///< |1  |ISO14443A Andere (SmartMX, usw.)|
                            ///< |2-8|-FREI-                          |
    uint8_t iso15693{};     ///< Gibt an welche Ausweistypen von ISO15693 gelesen werden sollen.
                            ///< |Bit|Bedeutung (wenn gesetzt)            |
                            ///< |:-:|:-----------------------------------|
                            ///< |1  |ISO15693 iCode SL                   |
                            ///< |2  |ISO15693 Tag-it HF-I                |
                            ///< |3  |ISO15693 My-D vicinity              |
                            ///< |4  |ISO15693 Andere (LegicAdvant usw.)  |
                            ///< |5  |ISO15693 24LR16 / 24LR64 von ST     |
                            ///< |6  |ISO15693 Fujitsu MB89R118 / MB89R119|
                            ///< |7  |ISO15693 LRI2k / LRI64k von ST      |
                            ///< |8  |-FREI-                              |
    uint8_t outputFormat{}; ///< Gibt das abzulegende Werteformat an.
                            ///< |Wert|Bedeutung          |
                            ///< |:--:|:------------------|
                            ///< |0   |Dezimal            |
                            ///< |1   |Hexadezimal        |
                            ///< |2   |-FREI-             |
                            ///< |3   |Hexadezimal gedreht|
                            ///< |4   |Dezimal gedreht    |
    uint8_t outputLength{}; ///< Gibt die abzulegende Werteformatlänge an.
                            ///< |Länge|Bedeutung                                  |
                            ///< |:---:|:------------------------------------------|
                            ///< |0    |keine feste Länge                          |
                            ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[4]{};
#endif
}; // 10 Byte


///////////////////////////////////////
// Struct: StructTransponderWiegandClockDataHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderWiegandClockDataHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines ISO Tags Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.

Diese Struktur kommt als Erweiterung in der Struktur @ref StructNumFieldEx zur Anwendung. Sie wird auch verwendet
um die globale Angabe sowie die Angabe bei der Zutrittskontrolle in @ref StructConfig abzulegen.
*/
struct StructTransponderWiegandClockData
{
    uint8_t flags{};                        ///< Flags, die die nachfolgenden Eigenschaften angeben.
                                            ///< |Bit|Bedeutung                                                                             |
                                            ///< |:-:|:-------------------------------------------------------------------------------------|
                                            ///< |1  |Steuerleitung (CLS) vorhanden. Gesetzt => vorhanden; Nicht gesetzt => nicht vorhanden.|
                                            ///< |2  |Parität prüfen: Gesetzt => prüfen; Nicht gesetzt => nicht prüfen.                     |
                                            ///< |3  |Firmencode prüfen: Gesetzt => prüfen; Nicht gesetzt => nicht prüfen.                  |
                                            ///< |4-8|-FREI-                                                                                |
    uint8_t clsWireActiveState{};           ///< Gibt den Zustand der Steuerleitung an, wenn diese aktiv ist (sofern vorhanden; siehe flags-Bit1).
                                            ///< |Wert|Bedeutung |
                                            ///< |:--:|:---------|
                                            ///< |0   |Low-Aktiv |
                                            ///< |1   |High-Aktiv|
    uint8_t bitFormat{};                    ///< Gibt das Bit-Format (Anzahl der vom Transponder gelieferten Bits) des Transponders an.
                                            ///< |Wert|Bedeutung                                                                                  |
                                            ///< |:--:|:------------------------------------------------------------------------------------------|
                                            ///< |0   |Individuelles Einstellen der Bitanzahl und der Eigenschaften Parität bzw. der Paritätsbits.|
                                            ///< |1   |26-Bit-Standardformat                                                                      |
                                            ///< |2   |32-Bit-Standardformat                                                                      |
                                            ///< |3   |34-Bit-Standardformat                                                                      |
                                            ///< |4   |35-Bit-Standardformat                                                                      |
                                            ///< |5   |37-Bit-Standardformat                                                                      |
                                            ///< |6   |38-Bit-Standardformat                                                                      |
                                            ///< |7   |42-Bit-Standardformat                                                                      |
                                            ///< |8   |66-Bit-Standardformat                                                                      |
    uint16_t numberOfBits{};                ///< Gibt die Anzahl der Bits an, die eingestellt werden können, wenn das individuelle Bitformat gewählt ist (siehe @ref bitFormat Wert 0).

    uint8_t parityFlags01{};                ///< Flags, die die nachfolgenden Eigenschaften angeben.
                                            ///< |Bit|Bedeutung                                                                             |
                                            ///< |:-:|:-------------------------------------------------------------------------------------|
                                            ///< |1  |Paritätsberechnung wird verwendet (wenn die Position des Paritätsbits größer Null ist)|
                                            ///< |^  |Gesetzt => Parität prüfen; Nicht gesetzt => Parität nicht prüfen                      |
                                            ///< |2  |Gibt an, ob es sich um eine (un-)gerade Parität handelt                               |
                                            ///< |^  |Gesetzt => ungerade Parität; Nicht gesetzt => gerade Parität                          |
                                            ///< |3-8|-FREI-                                                                                |
    uint16_t parityBitPos01{};              ///< Gibt die Position des Paritätsbits an.
    uint16_t parityStart01{};               ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich beginnt.
    uint16_t parityEnd01{};                 ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich endet.

    uint8_t parityFlags02{};                ///< Flags, die die nachfolgenden Eigenschaften angeben.
                                            ///< |Bit|Bedeutung                                                                             |
                                            ///< |:-:|:-------------------------------------------------------------------------------------|
                                            ///< |1  |Paritätsberechnung wird verwendet (wenn die Position des Paritätsbits größer Null ist)|
                                            ///< |^  |Gesetzt => Parität prüfen; Nicht gesetzt => Parität nicht prüfen                      |
                                            ///< |2  |Gibt an, ob es sich um eine (un-)gerade Parität handelt                               |
                                            ///< |^  |Gesetzt => ungerade Parität; Nicht gesetzt => gerade Parität                          |
                                            ///< |3-8|-FREI-                                                                                |
    uint16_t parityBitPos02{};              ///< Gibt die Position des Paritätsbits an.
    uint16_t parityStart02{};               ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich beginnt.
    uint16_t parityEnd02{};                 ///< Gibt die Position des Bits an, bei welchem der Paritätsbereich endet.

    uint8_t facilityCode[8]{};              ///< Gibt den Wert des Firmencodes an, gegen den der Wert des Firmencodes vom Transponder geprüft werden soll.
    uint8_t facilityCodeFormat{};           ///< Gibt das Format des eingegebenen Firmencodes an.
                                            ///< |Wert|Bedeutung          |
                                            ///< |:--:|:------------------|
                                            ///< |0   |Dezimal            |
                                            ///< |1   |Hexadezimal        |
                                            ///< |2   |ASCII              |
                                            ///< |3   |Hexadezimal gedreht|
                                            ///< |4   |Dezimal gedreht    |
                                            ///< |5   |Binär              |
    uint16_t facilityCodeBitPos{};          ///< Gibt die Position des Startbits des Firmencodes an.
    uint16_t facilityCodeNumberOfBits{};    ///< Gibt die Anzahl der Bits des Firmencodes an.
    uint16_t cardDataBitPos{};              ///< Gibt die Position des Startbits der Kartendaten an.
    uint16_t cardDataNumberOfBits{};        ///< Gibt die Anzahl der Bits der Kartendaten an.
    uint8_t outputData{};                   ///< Gibt an, welche Daten von dem Transponder übernommen werden sollen.
                                            ///< |Wert|Bedeutung                |
                                            ///< |:--:|:------------------------|
                                            ///< |0   |nur die Kartennummer     |
                                            ///< |1   |Firmencode + Kartennummer|
    uint8_t outputFormat{};                 ///< Gibt das Ablageformat der Daten von dem Transponder an.
                                            ///< |Wert|Bedeutung          |
                                            ///< |:--:|:------------------|
                                            ///< |0   |Dezimal            |
                                            ///< |1   |Hexadezimal        |
                                            ///< |2   |ASCII              |
                                            ///< |3   |Hexadezimal gedreht|
                                            ///< |4   |Dezimal gedreht    |
                                            ///< |5   |Binär              |
    uint8_t outputLength{};                 ///< Gibt die Länge der Ausgabedaten an.
                                            ///< |Länge|Bedeutung                                  |
                                            ///< |:---:|:------------------------------------------|
                                            ///< |0    |keine feste Länge                          |
                                            ///< |1-11 |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t tmp[27]{};
#endif
}; // 66 Byte


///////////////////////////////////////
// Struct: StructTransponderETSITagsHeader

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderETSITagsHeader
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines ETSI Tag Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderETSITags
{
    uint8_t strength{};         ///< Gibt die Feldstärke in % an.
    uint8_t rwFlags{};          ///< Gibt durch Flags folgendes an.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                          |
                                ///< |:-:|:---------------------------------------------------------------------------------|
                                ///< |1  |EPC lesen. (ansonsten angegebene Bank lesen)                                      |
                                ///< |2  |Mehrachbuchung bei EPC                                                            |
                                ///< |^  |Für bis zu 10 im Feld befindliche Ausweise wird eine separate Buchung durchgeführt|
                                ///< |3-8|-FREI-                                                                            |
    uint8_t rwBank{};           ///< Nummer der zu verwendenden Bank (0 - 3).
    uint16_t rwOffset{};        ///< Offset in den Datenblock.
    uint16_t rwLength{};        ///< Anzahl der zu lesenden Datenbytes aus dem Sektor/Block.
    uint32_t password{};        ///< Passwortverwendung für Lesen / Schreiben.
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |-FREI-             |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[4]{};
#endif
}; // 18 Byte


///////////////////////////////////////
// Struct: StructTransponderQ5Header

/** Diese Struktur enthaelt zusätzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen für die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von @ref count Offsets, die die Offsets der
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhängig.
*/
struct StructTransponderQ5Header
{
    uint8_t structSize{};       ///< Größe dieser Struktur
                                ///< Diese stellt gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray dar.
    uint8_t structFlags{};      ///< Flags für die strukturübergreifende Verarbeitung.
                                ///< |Bit|Bedeutung (wenn gesetzt)                                                 |
                                ///< |:-:|:------------------------------------------------------------------------|
                                ///< |1  |Passwörter der Strukturen werden mittels RC4 und DatafoxKey verschlüsselt|
                                ///< |n  |-FREI-                                                                   |
    uint8_t transponderType{};  ///< Typ des Leseverfahrens.
    uint8_t count{};            ///< Anzahl der Einträge in dem Offsetarray, was auf diese Struktur folgt.
    uint8_t isoFlags{};         ///< Enthält die zugelassenen Ausweistypen.
                                ///< |Bit|Bedeutung (wenn gesetzt)   |
                                ///< |:-:|:--------------------------|
                                ///< |1  |ISO14443-Ausweis zugelassen|
                                ///< |2  |ISO15693-Ausweis zugelassen|
                                ///< |3-8|-FREI-                     |
}; // 5 Byte

/** Diese Struktur dient der Spezifikation, bei Angabe eines Q5 Transponders in den globalen Einstellungen,
des zu verwendenden Segments, Ablageformates sowie der Auswahl.
*/
struct StructTransponderQ5
{
    uint8_t rwFlags{};          ///< Gibt durch Flags folgendes an.
                                ///< |Bit|Bedeutung (wenn gesetzt)|
                                ///< |:-:|:-----------------------|
                                ///< |1  |UID lesen (Seriennummer)|
                                ///< |2-8|-FREI-                  |
    uint16_t rwBlock{};         ///< Nummer des zu verwendenden Blocks.
    uint16_t rwOffset{};        ///< Offset in den Block.
    uint16_t rwLength{};        ///< Anzahl der zu lesenden Datenbytes.
    uint8_t keyUsage{};         ///< Verwendung des Schlüssels.
                                ///< |Wert|Bedeutung                                        |
                                ///< |:--:|:------------------------------------------------|
                                ///< |0   |kein Passwort                                    |
                                ///< |1   |Ausweise ohne und mit diesem Passwort akzeptieren|
                                ///< |2   |nur Ausweise mit diesem Passwort akzeptieren     |
    uint8_t keyFlags{};         ///< Passwortflags
                                ///< |Bit|Bedeutung (wenn gesetzt)                     |
                                ///< |:-:|:--------------------------------------------|
                                ///< |1  |@ref keyA liegt im verschlüsselten Format vor|
                                ///< |2  |@ref keyB liegt im verschlüsselten Format vor|
                                ///< |3-8|-FREI-                                       |
    uint8_t keyA[8]{};          ///< Zu verwendender Schlüssel A.
    uint8_t keyB[8]{};          ///< Zu verwendender Schlüssel B.
    uint8_t outputFormat{};     ///< Gibt das abzulegende Werteformat an.
                                ///< |Wert|Bedeutung          |
                                ///< |:--:|:------------------|
                                ///< |0   |Dezimal            |
                                ///< |1   |Hexadezimal        |
                                ///< |2   |ASCII              |
                                ///< |3   |Hexadezimal gedreht|
                                ///< |4   |Dezimal gedreht    |
    uint16_t outputLength{};    ///< Gibt die abzulegende Werteformatlänge an.
                                ///< |Länge|Bedeutung                                  |
                                ///< |:---:|:------------------------------------------|
                                ///< |0    |keine feste Länge                          |
                                ///< |n    |Anzahl Zeichen ggf. mit Vornullen auffüllen|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[2]{};
#endif
}; // 30 Byte

///////////////////////////////////////
// Struct: StructTransponderHitag1FeigHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count uint16_t Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderHitag1FeigHeader
{
    uint8_t structSize{};         ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    uint8_t structFlags{};        ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
    uint8_t transponderType{};    ///< Typ des Leseverfahrens.
    uint8_t count{};              ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines Hitag1 Feig Transponders in den Globalen Einstellungen,
die zu verwendenden Einstellungen, Ablageformat sowie Auswahl.

*/
struct StructTransponderHitag1Feig
{
    uint8_t flags{};              ///< Flags, die die nachfolgenden Eigenschaften angeben.
                                  ///< Bit1: Firmencode prüfen: Gesetzt => prüfen; Nicht gesetzt => nicht prüfen.
                                  ///< Bit2-8: Unused.
    uint8_t facilityCodeStart{};  ///< Gibt die Position der Start-Hex-Ziffer des Firmencodes an.
    uint8_t facilityCodeCount{};  ///< Gibt die Anzahl der Hex-Ziffern des Firmencodes an.
    uint8_t facilityCode[6]{};    ///< Gibt den Wert des Firmencodes an, gegen den der Wert des Firmencodes vom Transponder geprüft werden soll.
    uint8_t cardNumberStart{};    ///< Gibt die Position der Start-Hex-Ziffer der Kartennummer an.
    uint8_t cardNumberCount{};    ///< Gibt die Anzahl der Hex-Ziffern der Kartennummer an.
    uint8_t outputData{};         ///< Gibt an, welche Daten von dem Transponder übernommen werden sollen.
                                  ///< 0 => Nur die Kartennummer.
                                  ///< 1 => Firmencode + Kartennummer.
    uint8_t outputFormat{};       ///< Gibt das Ablageformat der Daten von dem Transponder an.
                                  ///< 1 => Hexadezimal.
    uint8_t outputLength{};       ///< Gibt die Länge der Ausgabedaten an.
                                  ///< 0 => Keine feste Länge.
                                  ///< n => Anzahl Zeichen zwischen 1 und 24; Ggf. mit Vornullen auf die angegebene Länge bringen.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[16]{};         ///< Reserve fuer Erweiterungen.
#endif

}; // 25 Byte

///////////////////////////////////////
// Struct: StructTransponderPreconfiguredReaderHeader

/** Diese Struktur enthaelt zusaetzliche Informationen zu den Transponderkonfigurationen.

In dieser Struktur sind Informationen fuer die Transponderkonfigurationen abgelegt.
Dieser Struktur folgt ein OffsetArray von count uint16_t Offsets, die die Offsets auf die
jeweiligen Folgestrukturen beinhalten. Die Folgestrukturen sind von den Transponderverfahren
abhaengig.
*/
struct StructTransponderPreconfiguredReaderHeader
{
    uint8_t structSize{};         ///< Groesse dieser Struktur (ist gleichzeitige eine "Art" Version und Grundoffset auf das folgende Offsetarray)
    uint8_t structFlags{};        ///< Flags fuer die Strukturuebergreifende Verarbeitung.
                                  ///< Bit1: Passwoerter der Strukturen werden mittels RC4 und DatafoxKey verschluesselt.
                                  ///< Bit2-8: Unused
    uint8_t transponderType{};    ///< Typ des Leseverfahrens.
    uint8_t count{};              ///< Anzahl der Eintraege in dem Offsetarray, was auf diese Struktur folgt.
}; // 4 Byte

/** Diese Struktur dient bei Angabe eines vorkonfigurierten Lesers in den Globalen Einstellungen,
die zu verwendenden Einstellungen, Ablageformat sowie Auswahl.

*/
struct StructTransponderPreconfiguredReader
{
    uint8_t flags{};              ///< Unused
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[29]{};         ///< Reserve fuer Erweiterungen.
#endif
}; // 30 Byte

///////////////////////////////////////
// Struct: StructRecordDescription / StructTable / StructTable2
// Beschreibung einer Tabelle

#define MAX_RECORD_DESCRIPTION  20  ///< Maximale Anzahl der Datensatzbeschreibungen und auch max. Listenbeschreibungen
#define MAX_FIELDS              25  ///< Maximale Anzahl Felder in einem Datensatz.
#define MAX_RECORD_SIZE        241  ///< Maximale Größe eines Datensatzes (255 - DS_KOPF_CRC)
#define MAX_SUBPOINTS          100  ///< Maximale Anzahl Unterpunkte im Baum (Bedienung) 

/** Diese Struktur beschreibt mit ihren Grundangaben eine Tabellendefinition.

Die Informationen einer Datensatz- sowie Listenbeschreibung, werden in dieser Struktur abgelegt.
Die Ablagereihenfolge in der Setupdatei (*.aes) ist wie folgt:
Es kommt zuerst ein Zeiger-Array einer Länge von MAX_RECORD_DESCRIPTION * uint32_t,
diese Zeiger verweisen jeweils auf eine der maximal MAX_RECORD_DESCRIPTION möglichen.
*/
struct StructRecordDescription
{
    uint8_t fields{};   ///< Gibt die Anzahl der Tabellenfelder an.
    char name[17]{};    ///< Gibt den verwendeten Tabellenname an.
                        ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint32_t ptr{};     ///< Gibt den Zeiger auf das Array der Zeiger für die Feldbeschreibungen (Typ @ref StructRecordField) an.
#endif
}; // 22 Byte

/** Diese Struktur ist eine Übergangsstruktur von @ref StructRecordDescription auf StructTable2.

Nähere Informationen siehe @ref StructTable2.
*/
struct StructTable
{
    uint8_t count{};    ///< Anzahl der Felder im Datensatz
    char name[17]{};    ///< Name der Datensatzbeschreibung
                        ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
    uint32_t ptr{};     ///< Zeiger auf Array für Zeiger auf Feldstrukturen
    uint8_t number{};   ///< Nummer der Tabelle im System
    uint8_t key{};      ///< Schlüsselspalte
    uint8_t sort{};     ///< Sortierspalte
    uint16_t flags{};   ///< Flags
}; // 27 Byte

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define TF_SORT   0x0001 // Tabelle soll lexiographisch sortiert werden und -sort- gibt nullindex der Spalte an
#define TF_SORTED 0x0002 // Listendaten sind lexiographisch nach spaltenindex -sort- sortiert.
#endif

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellendefinition.

Diese Struktur ersetzt die nicht mehr ausreichende @ref StructRecordDescription.
Eine Beschreibung zur Ablage der Daten in der Setupdatei (*.aes) ist dort gegeben.
*/
struct StructTable2
{
    uint8_t number{};   ///< Nummer der Tabelle im System
    char name[32]{};    ///< Name der Tabelle.
                        ///< Der Name darf maximal 31 Zeichen (ASCII) lang sein.
    uint8_t count{};    ///< Anzahl Tabellenfelder
    uint8_t key{};      ///< Schlüsselspalte
    uint8_t sort{};     ///< Sortierspalte
    uint16_t flags{};   ///< Flags
}; // 38 Byte


///////////////////////////////////////
// Struct: StructRecordField / StructTableField / StructTable2Field
// Beschreibung eines Tabellenfeldes 

//	Feldtypen mit Längenangabe
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define TF_ULONG                                           1 ///< StructRecordField.type: uint32_t (4 Byte)
#define TF_ULONG_LEN                                       4 ///< StructRecordField.count: uint32_t (4 Byte)
#define TF_DT                                              2 ///< StructRecordField.type: DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
#define TF_DT_LEN                                          7 ///< StructRecordField.count: DateTime (7 Byte, 4 Byte Date, 3 Byte Time)
#define TF_NUM                                             3 ///< StructRecordField.type: ASCII (nur Ziffern)
#define TF_NUM_LEN_MIN                                     2 ///< StructRecordField.type: Minimale Feldlänge inkl. Nullterminierung für TF_NUM
#define TF_NUM_LEN_MAX                                    41 ///< StructRecordField.type: Maximale Feldlänge inkl. Nullterminierung für TF_NUM
#define TF_A_NUM                                           4 ///< StructRecordField.type: ASCII
#define TF_A_NUM_LEN_MIN                                   2 ///< StructRecordField.type: Minimale Feldlänge inkl. Nullterminierung für TF_A_NUM
#define TF_A_NUM_LEN_MAX                                  41 ///< StructRecordField.type: Maximale Feldlänge inkl. Nullterminierung für TF_A_NUM
#define TF_DATE                                            5 ///< StructRecordField.type: Date (4 Byte; Y100, Y, M, D)
#define TF_DATE_LEN                                        4 ///< StructRecordField.count: Date (4 Byte; Y100, Y, M, D)
#define TF_TIME                                            6 ///< StructRecordField.type: Time (3 Byte; h, m, s)
#define TF_TIME_LEN                                        3 ///< StructRecordField.count: Time (3 Byte; h, m, s)
#define TF_FINGERTEMPLATE_DIN                              7 ///< StructRecordField.type: Fingertemplate (161 Byte; DIN V66400)
#define TF_FINGERTEMPLATE_DIN_LEN                        161 ///< StructRecordField.count: Fingertemplate (161 Byte; DIN V66400)
#define TF_FINGERTEMPLATE_IDENCOM_COMPACT                  8 ///< StructRecordField.type: Fingertemplate (216 Byte; Idencom Compact)
#define TF_FINGERTEMPLATE_IDENCOM_COMPACT_LEN            216 ///< StructRecordField.count: Fingertemplate (216 Byte; Idencom Compact)
#define TF_BINARY                                          9 ///< StructRecordField.type: Binaere Daten
#define TF_BINARY_LEN_MIN                                  1 ///< StructRecordField.type: Minimale Feldlänge für TF_BINARY
#define TF_BINARY_LEN_MAX                                220 ///< StructRecordField.type: Maximale Feldlänge für TF_BINARY
#define TF_DATA                                           10 ///< StructRecordField.type: Beliebige Daten bis 240 Byte, im Ersten Feldbyte ist die Längenangabe der gültigen Folgebytes abgelegt.
#define TF_DATA_LEN_MIN                                    2 ///< StructRecordField.type: Minimale Feldlänge für TF_DATA
#define TF_DATA_LEN_MAX                                  241 ///< StructRecordField.type: Maximale Feldlänge für TF_DATA
#endif

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellenfelddefinition.

Die Informationen einer Datensatzfeld- sowie Listenfeldbeschreibung, werden in dieser Struktur abgelegt.
Die Ablagereihenfolge in der Setupdatei (*.aes) ist wie folgt:
Es kommt zuerst ein Zeiger-Array einer Länge von @ref StructRecordDescription.fields * uint32_t,
diese Zeiger verweisen jeweils auf eine der @ref StructRecordDescription.fields möglichen Beschreibungen.
*/
struct StructRecordField
{
    uint8_t type{};     ///< Feldtyp
    uint8_t count{};    ///< Feldlänge inkl. abschließender Null bei Strings
    char name[17]{};    ///< Feldname.
                        ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
}; // 19 Byte

/** Diese Struktur ist eine Übergangsstruktur von @ref StructRecordField auf @ref StructTable2Field.

Näheres siehe @ref StructTable2Field.
*/
struct StructTableField
{
    uint8_t type{};     ///< Feldtyp 
    uint8_t size{};     ///< Feldlänge inkl. abschließender Null bei Strings
    char name[17]{};    ///< Feldname
                        ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t number{};   ///< Nummer des Tabellenfeldes
    uint16_t flags{};   ///< Flags
    char format[32]{};  ///< Formatstring für Feldwertmöglichkeit
                        ///< Das Format darf maximal 31 Zeichen (ASCII) lang sein.
}; // 54 Byte

/** Diese Struktur beschreibt mit ihren Grunddaten eine Tabellenfelddefinition.

Diese Struktur ersetzt die nicht mehr ausreichende @ref StructRecordField.
Eine Beschreibung zur Ablage der Daten in der Setupdatei (*.aes) ist dort gegeben.
*/
struct StructTable2Field
{
    uint8_t number{};   ///< Nummer des Tabellenfeldes
    char name[32]{};    ///< Feldname
                        ///< Der Name darf maximal 31 Zeichen (ASCII) lang sein.
    uint8_t type{};     ///< Feldtyp
    uint8_t size{};     ///< Feldlänge inkl. abschließender Null bei Strings
    uint16_t flags{};   ///< Flags
    char format[32]{};  ///< Formatstring für Feldwertmöglichkeit
                        ///< Das Format darf maximal 31 Zeichen (ASCII) lang sein.
}; // 69 Byte


/////////////////////////////////////////////////////////////////////
// Bediensteuerung 
// Hautpmenue mit Menueeintraegen, Eingabeketten und Feldern

///////////////////////////////////////
// Struct: StructOperating

// Typen der Bedienung 
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define TB_MENUE        1   ///< StructOperating.type: Folgestruktur ist von Typ: StructMenu
#define TB_DATENSATZ    2   ///< StructOperating.type: Folgestruktur ist von Typ: StructReadRecord
#define TB_SYSTEM       3   ///< StructOperating.type: Folgestruktur ist von Typ: Undefiniert
#define TB_KONFIG_ROOT  4   ///< StructOperating.type: Folgestruktur ist von Typ: Undefiniert
#define TB_LISTE        5   ///< StructOperating.type: Folgestruktur ist von Typ: Undefiniert
#define TB_CLR_LAST     6   ///< StructOperating.type: Folgestruktur ist von Typ: Undefiniert
#endif

/** Informationsstruktur zur Datensteuerung der Bedienung.

In der Gerätebedienung kann von festen Informationskonstrukten gesprochen werden, an stellen wo
mehrere Wege eingeschlagen werden könnten, kommt diese Struktur zum Einsatz. Es kann auf die
Bedienungswurzel z. B. sowohl eine Menüstruktur oder ein anderer Typ folgen. In einem Menü selbst sind
hingegen nur Menüeinträge erlaubt. In diesen jedoch wieder verschiednene Typen wie Eingabeketten etc.
Um Mehrdeutigkeiten aufzulösen kommt somit vor jeder @ref StructMenu und @ref StructReadRecord eine @ref StructOperating.
*/
struct StructOperating
{
    uint8_t type{}; ///< Typ der Folgestruktur
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t d1{};
    uint8_t d2{};
    uint32_t ptr{}; ///< Zeiger auf Folgestruktur, welche durch @ref type angegeben wird.
#endif
}; // 7 Byte


///////////////////////////////////////
// Struct: StructMenu

/** Diese Struktur beinhaltet Informationen zum Hauptmenü / Untermenü's.
*/
struct StructMenu
{
    uint8_t flags{};            ///< Flags wie Zutrittskontrollabfrage aktiviert usw.
                                ///< |Bit|Bedeutung (wenn gesetzt)             |
                                ///< |:-:|:------------------------------------|
                                ///< |1  |zweispaltige Anzeige beim AE-MasterIV|
                                ///< |2  |Menüauswahl per Barcode zulassen     |
                                ///< |3  |Menüauswahl per Tranponder zulassen  |
    uint8_t numberOfEntries{};  ///< Anzahl der Einträge
    char heading1[17]{};        ///< Überschrift 1 des Menü's.
                                ///< Die Überschrift darf maximal 16 Zeichen (ASCII) lang sein.
    char heading2[17]{};        ///< Überschrift 2 des Menü's.
                                ///< Die Überschrift darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t header1Text{};      ///< Text in Kopfzeile 1.
                                ///< Bit 0...3 Nummer-Globale-Variable nur für das Hauptmenü gültig sonst 254
    uint8_t header2Text{};      ///< Text in Kopfzeile 2.
                                ///< Bit 0...3 Nummer-Globale-Variable nur für das Hauptmenü gültig sonst 254
    uint8_t globVar{};          ///< Globale Variable in die der Wert des Barcodes oder Transponders zur Auswahl des Menüeintrags kopiert werden soll.
    uint8_t behaviourAtEsc{};   ///< Verhalten bei ESC-Taste.
                                ///< Sprungziel (Bedeutung siehe @ref StructReadRecord)
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[4]{};
    uint32_t ptr{};             ///< Zeiger auf Liste der Einträge
#endif
}; // 48 Byte


///////////////////////////////////////
// Struct: StructConfigRoot

// Ein Varhalten 
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define EVERH_AKTUELL 0      ///< StructConfigRoot.behaviourAfterSwitchOn: Nach Ein auf der aktuellen Pos vor Aus bleiben 
#define EVERH_MENUE   1      ///< StructConfigRoot.behaviourAfterSwitchOn: Nach Einschalten zurück in das Menü
                             ///<                                          Eintrag aus Hauptmenü starten
#endif

/** Diese Struktur beinhaltet Informationen zum Haupteintrag der Bedienung.

Dieser Struktur folgt ein Zeigerarray mit numberOfEntries * sizeof(uint32_t) Zeigern, die auf folgende
@ref StructMenu verweisen.
*/
struct StructConfigRoot
{
    uint16_t autoSwitchOff{};           ///< Zeit in Sekunden bis Timeboy abgeschaltet wird, wenn keine Bedienung erfolgt.
                                        ///< 0 --> keine automatische Abschaltung
    uint8_t numberOfEntries{};          ///< Anzahl Einträge
    uint8_t behaviourAfterSwitchOn{};   ///< Verhalten nach Einschalten
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[8]{};
    uint32_t ptr{};                     ///< Zeiger auf Zeigerarray mit @ref numberOfEntries Einträgen
#endif
}; // 16 Byte


///////////////////////////////////////
// Struct: StructMenuItem

// Flags für Menüeintrag
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define BDME_MDE_0                0x00 ///< StructMenuItem.flags: Normal
#define BDME_MDE_1                0x01 ///< StructMenuItem.flags: Anmeldung
#define BDME_MDE_2                0x02 ///< StructMenuItem.flags: Abmeldung
#define BDME_MDE_3                0x03 ///< StructMenuItem.flags: Taste klemmen
#define BDME_MDE_4                0x04 ///< StructMenuItem.flags: Taste klemmen und kummulieren
#define BDME_MDE_5                0x05 ///< StructMenuItem.flags: Buchung
#define BDME_MDE_ONLY_BY_ORDER    0x80 ///< StructMenuItem.flags: Ausführung ist nur bei angemeldetem Auftrag möglich.
#endif

/** Diese Struktur enthält Informationen zu einem Menüeintrag.

Dieser Struktur folgt ein Zeigerarray mit numberOfEntries * sizeof(uint32_t) Zeigern, die auf folgende
@ref StructReadRecord oder @ref StructMenu verweisen.
*/
struct StructMenuItem
{
    char name[17]{};            ///< Menütext
                                ///< Der Text darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t numberOfEntries{};  ///< Anzahl der Einträge
    uint16_t password{};        ///< Passwort
    uint8_t flags{};            ///< Parameter für MDE
    uint8_t machine{};          ///< Zugeordnete Maschine bei Taste Klemmen
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[4]{};
    uint32_t ptr{};             ///< Zeiger auf Aktion nach Auswahl des Menuepunktes 
#endif
}; // 30 Byte


///////////////////////////////////////
// Struct: StructReadRecord

#ifndef DOXYGEN_SHOULD_SKIP_THIS
// Einlesemethoden für Datensaetze 
#define TBDA_NORM                    0 ///< StructReadRecord.Typ: Standard 

// Flags Datensatz einlesen 
#define BDEF_W_UEBER              0x01 ///< StructReadRecord.flags: Bei Schleifen bereits eingegebene Werte als 
                                       ///<                                Vorgabe für Eingabe übernehmen 
#define BDEF_NICHT_SPEICHERN      0x02 ///< StructReadRecord.flags: Datensatz nicht speichern (nur Globale Variable) 
#define BDEF_ZUSATZ_RETURN        0x04 ///< StructReadRecord.flags: Abfrage "Speichern ?" am Ende der Eingabekette 
#define BDEF_RUECKSPRUNG_MENUE    0x08 ///< StructReadRecord.flags: Nach Einlesen eines Datensatzes Ruecksprungmenue anzeigen 
#define BDEF_ENTRANCE_ACCESS      0x10 ///< StructReadRecord.flags: Zutrittskontrolle vor Abarbeitung der Eingabekette durchführen.
#define BDEF_RELAIS               0x20 ///< StructReadRecord.flags: Relais für bestimmte Zeit schalten.
#define BDEF_RECORD_DOUBLE        0x40 ///< StructReadRecord.flags: Eingabekette vor evntl. Eingabe von Werten ausfuehren, danach "Normale" Abarbeitung. 
#define BDEF_NO_SERVER_ONLINE     0x80 ///< StructReadRecord.flags: Im Modus "Server online" wird das Verhalten, dass auf die Antwort und Sendmessage gewaretet wird, für diesen Datensatz deaktiviert.
#endif

/** Diese Struktur enthält Informationen zu einer Eingabekette.

Dieser Struktur folgt ein Zeigerarray mit @ref fields * sizeof(uint32_t) Zeigern die auf folgende
@ref StructReadField verweisen. Ihr selbst ist immer eine @ref StructOperating vorangestellt.
*/
struct StructReadRecord
{
    uint8_t number{};           ///< Nummer der zugeordneten Datensatzbeschreibung (ab 0, 255 = Keine Zuordnung) 
    uint8_t fields{};           ///< Anzahl der Felder
    uint8_t behaviourAtCR{};    ///< Verhalten bei Enter-Taste, Sprungziel.
                                ///< |Wert   |Bedeutung                                            |
                                ///< |:-----:|:----------------------------------------------------|
                                ///< |0      |übergeordnetes U-Menü                                |
                                ///< |1      |Hauptmenü (war mal zum Ausschalten)                  |
                                ///< |253    |reserviert (nächstes Eingabekettenfeld)              |
                                ///< |254    |Hauptmenü                                            |
                                ///< |255    |nächste Eingabekette bzw. U-Menü                     |
                                ///< |2-111  |(1-109)-tes Eingabefeld der Eingabekette             |
                                ///< |112-127|(1-15)-te F–Taste des Hauptmenü                      |
                                ///< |200-229|(1-30)-te E-Kette bzw. U-Menü im gleichen Menüeintrag|
                                ///< |230-252|reserviert (wird zu 0, wenn Bedeutung unbekannt)     |
    char name[17]{};            ///< Überschrift.
                                ///< Die Überschrift darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t type{};             ///< Bedien-Typ des Einlesens
    uint8_t flags{};            ///< Diverse Flags definiert als BDEF_...
    uint8_t header1Text{};      ///< Text in Kopfzeile 1.
                                ///< |Bit  |Bedeutung                   |
                                ///< |:---:|:---------------------------|
                                ///< |1...4|Nummer der globalen Variable|
                                ///< |5...8|=0 -> Konstanter Text       |
                                ///< |^    |>0 -> globale Variable      |
    uint8_t behaviourAtESC{};   ///< Verhalten bei ESC-Taste, Sprungziel
                                ///< |Wert   |Bedeutung                                            |
                                ///< |:-----:|:----------------------------------------------------|
                                ///< |0      |übergeordnetes U-Menü                                |
                                ///< |1      |Hauptmenü (war mal zum Ausschalten)                  |
                                ///< |253    |reserviert (nächstes Eingabekettenfeld)              |
                                ///< |254    |Hauptmenü                                            |
                                ///< |255    |nächste Eingabekette bzw. U-Menü                     |
                                ///< |2-111  |(1-109)-tes Eingabefeld der Eingabekette             |
                                ///< |112-127|(1-15)-te F–Taste des Hauptmenü                      |
                                ///< |200-229|(1-30)-te E-Kette bzw. U-Menü im gleichen Menüeintrag|
                                ///< |230-252|reserviert (wird zu 0, wenn Bedeutung unbekannt)     |
    uint8_t vehicleRelease{};   ///< Fahrzeugfreigabe
                                ///< |Bit|Bedeutung (wenn gesetzt)                        |
                                ///< |:-:|:-----------------------------------------------|
                                ///< |1-5|Freigabetimeout. 0 -> Unbegrenzt, 1...24 Stunden|
                                ///< |6  |-FREI-                                          |
                                ///< |7  |Freigabe soll erfolgen (ansonsten Rücknahme)    |
                                ///< |8  |Fahrzeugfreigabe aktiv                          |
    uint8_t relayNum{};         ///< Nummer des anzusprechenden Relais.
                                ///< |Wert |Bedeutung                         |
                                ///< |:---:|:---------------------------------|
                                ///< |0-3  |interne Relais 1...4              |
                                ///< |4-11 |externe Relais des IO-Moduls 1...8|
                                ///< |12-23|interne Relaise 5...16            |
    uint16_t relayTime{};       ///< Zeit für die das ausgewählte Relais schliesst.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused{};
    uint32_t ptr{};             ///< Zeiger auf Liste der Strukturen für Einlesen der Felder.
                                ///< (Anzahl und Reihenfolge muß mit Datensatzbeschreibung übereinstimmen)
#endif
}; // 33 Byte


///////////////////////////////////////
// Struct: StructConfirmationEx

/** Diese Struktur enthält die Textzeilen eines Bestätigungsfeldes.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructConfirmationEx
{
    char text1[17]{};   ///< Hinweistext der ersten Zeile.
                        ///< Der Text darf maximal 16 Zeichen (ASCII) lang sein.
    char text2[17]{};   ///< Hinweistext der zweiten Zeile.
                        ///< Der Text darf maximal 16 Zeichen (ASCII) lang sein.
    char text3[17]{};   ///< Hinweistext der dritten Zeile.
                        ///< Der Text darf maximal 16 Zeichen (ASCII) lang sein.
}; // 51 Byte

///////////////////////////////////////
// Struct: StructTextEx

/** Diese Struktur enthält Text einer Feldfunktion. Z. B. SMS versenden.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructTextEx
{
    char text[51]{};    ///< Text der ersten Zeile.
                        ///< Der Text darf maximal 50 Zeichen (ASCII) lang sein.
}; // 51 Byte


///////////////////////////////////////
// Struct: StructCompare

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define FECT_NONE            0x00 ///< StructCompare.type: Kein Sprung
#define FECT_FORMAT_STRING   0x01 ///< StructCompare.type: Sprung mit Formatstring konstant.
#define FECT_FORMAT_GV       0x02 ///< StructCompare.type: Sprung mit Formatstring in globaler Variablen
#define FECT_FORMAT_STRING_2 0x03 ///< StructCompare.type: Sprung mit Formatstring konstant. FW >= 04.01.04
#define FECT_FORMAT_GV_2     0x04 ///< StructCompare.type: Sprung mit Formatstring in globaler Variablen. FW >= 04.01.04
#endif

/** Diese Struktur dient mit Ihren Informationen der Angabe eines Sprunges in einer Eingabekette.

Diese Struktur kommt als Erweiterung der Struktur @ref StructNumFieldEx zur Anwendung.
*/
struct StructCompare
{
    uint8_t type{};             ///< Typ für die Vergleichsart (Formatstring oder globale Variable)
    uint8_t formatGlobalVar{};  ///< Spezifikation der globalen Variable (wenn Typ globale Variable)
                                ///< |Bit|Bedeutung (wenn gesetzt)         |
                                ///< |:-:|:--------------------------------|
                                ///< |1-4|Arraynummer der globalen Variable|
                                ///< |5-8|Index der Auswahlliste (Combobox)|
    char formatString[17]{};    ///< Definition des Formats (wenn Typ Formatstring).
                                ///< Die Formatdefinition darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t negate{};           ///< Verzweigen wenn Vergleich NICHT zutraf.
    uint8_t jumpDestination{};  ///< Sprungziel, als Feldindex (0...n), von Eingabekette.
}; // 21 Byte

///////////////////////////////////////
// Struct: StructNumFieldEx

// Quellen für Eingabe von Daten (Maske in Var. Flag) 
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define NFF_TASTATUR    0x80 ///< StructNumFieldEx.flags: Werteingabe über Tastatur möglich.
#define NFF_BARCODE     0x40 ///< StructNumFieldEx.flags: Werteingabe über Barcode möglich.
#define NFF_TRANSPONDER 0x20 ///< StructNumFieldEx.flags: Werteingabe über Transponder möglich.
#define NFF_NO_RET      0x01 ///< StructNumFieldEx.flags: CR nach Barcodeeingabe notwendig.
#define NFF_ANZEIGEN    0x0E ///< StructNumFieldEx.flags: 3 Bit Maske für 8 Werte der Anzeigedauer, wenn kein Return notwendig.
#define NFF_BUZZER      0x10 ///< StructNumFieldEx.flags: Kein aktustisches Signal zur Bestätigung
#endif

/** Diese Struktur dient, in Eingabekettenfeldern bei allen Einlesemethoden außer Liste und Bestätigung,
der Angabe weiterer Informationen.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructNumFieldEx
{
    uint8_t flags{};                    ///< Flags z.B. Einlesen über Barcode erlaubt
    StructTransponder tag;              ///< Transpondereinlesen bei Hitag, bei diesem Feld abaendern. Abweichend von der globalen Einstellung.
    union
    {
        uint8_t param1{};               ///< Abhängig von der genutzten Feldfunktion kann einer der nachfolgenden Bezeichner verwendet werden.
        uint8_t fingerprintParam;       ///< Parametervariable für Fingerprint
                                        ///< |Bit|Bedeutung (wenn gesetzt)                     |
                                        ///< |:-:|:--------------------------------------------|
                                        ///< |1-2|Destination: (0)Fingerprint modul, (1)Ausweis|
                                        ///< |3-4|Delete Method: (0)Fingertemplate, (1)PID     |
                                        ///< |5-6|Fingertemplate: (0)Nr 1, (1)Nr 2             |
        uint8_t inputModuleTimeout;     ///< Zeitangabe wie lange der Barcode- oder Transponder-Leser eingeschaltet bleibt.
        uint8_t operantLeft;            ///< Linker Operant bei @ref TBFE_OPERATION
        uint8_t relayNumberExtended;    ///< Nummer des Relais.
                                        ///< Wird verwendet bei @ref TBFE_SWITCH_RELAY, wenn in relayNumber die Nummer des Relais >= 12 ist.
        uint8_t digitalStatusAsDecimal; ///< Bei der Feldfunktion @ref TBFE_DIGITAL_WERT soll der aktuelle Status der digitalen Eingänge als 32bit Wert geliefert werden.
    };
    union
    {
        uint8_t param2{};               ///< Abhängig von der genutzten Feldfunktion kann einer der nachfolgenden Bezeichner verwendet werden.
        uint8_t fingerprintPID;         ///< Index der globalen Variable für PID-Quelle
        uint8_t operantRight;           ///< Rechter Operant bei @ref TBFE_OPERATION.
    };

    union
    {
        uint8_t param3{};               ///< Abhängig von der genutzten Feldfunktion kann einer der nachfolgenden Bezeichner verwendet werden.
        uint8_t fingerprintTeachFlags;  ///< Zusätliche Funktionen zum Saturn-Flächensensor ( z.B. Übungsmodus, Optimierungsmodus, Traingsmodus ) 8 Bit frei somit
        uint8_t operation;              ///< mathematische oder logische Operation ausführen (für @ref TBFE_OPERATION).
                                        ///< Aufbau
                                        ///< |Bit|Bedeutung (wenn gesetzt)                     |
                                        ///< |:-:|:--------------------------------------------|
                                        ///< |1-2|rechter Operant|
                                        ///< |3-6|Operation      |
                                        ///< |7-8|linker Operant |
                                        ///< Operant
                                        ///< |Wert|Bedeutung      |
                                        ///< |:-:|:---------------|
                                        ///< |0  |Konstante       |
                                        ///< |1  |Feld            |
                                        ///< |2  |globale Variable|
                                        ///< Operation
                                        ///< |Wert|Bedeutung|
                                        ///< |:-:|:---------|
                                        ///< |0  |+         |
                                        ///< |1  |-         |
                                        ///< |2  |not       |
    };
    //uint8_t relayFlagsNum;              ///< Flags und verwendete Relaisnummer (für @ref TBFE_SWITCH_RELAY, @ref TBFE_SWITCH_LED, @ref TBFE_SWITCH_BUZZER)
    uint8_t relayNumber : 4;            ///< Unteres Nibble zur Definition des Relais / Buzzer / LED
                                        ///< |Typ   |Inhalt         |
                                        ///< |:-----|:--------------|
                                        ///< |Relais|Relaisnummer   |
                                        ///< |LED   |Maschinennummer|
                                        ///< |Buzzer|Lautstärke in % (10er Schritte)|
    uint8_t relayAction : 4;            ///< Oberes Nibble zur Definition der Aktion des Relais / Buzzer / LED
                                        ///< |Bit|Bedeutung (Abfolge aufeinanderfolgender Aktionen [2]) |
                                        ///< |:-:|:-----------------------------------------------------|
                                        ///< |1-2|Aktion nach Zeitablauf (was passiert nach dem Timeout)|
                                        ///< |3-4|Aktion vor Zeitablauf (was passiert bis zum Timeout)  |
                                        ///< @n
                                        ///< |Aktion (Relais / Buzzer)||
                                        ///< |:----------------------:||
                                        ///< |Wert|Bedeutung           |
                                        ///< |0   |Ausschalten         |
                                        ///< |1   |Einschalten         |
                                        ///< |2   |Umschalten          |
                                        ///< |3   |Keine               |
                                        ///< @n
                                        ///< |Aktion (LED)||
                                        ///< |:----------------------:||
                                        ///< |Wert|Bedeutung           |
                                        ///< |0   |rot und grün aus    |
                                        ///< |1   |rot an, grün aus    |
                                        ///< |2   |rot blinkt, grün aus|
                                        ///< |3   |rot aus, grün an    |
                                        ///< |4   |rot und grün an     |
    uint16_t relayTime{};               ///< Zeit für Schaltdauer in Sekunden.
    char contentOfConstant[17]{};       ///< Wert der Konstante bei der Einlesemethode "Konstante übernehmen".
                                        ///< Der Inhalt des konstanten Wertes darf maximal 16 Zeichen (ASCII) lang sein.
    StructCompare compare;              ///< Sprungziels in eine Eingabekette definieren.
}; // 51 Byte


///////////////////////////////////////
// Struct: StructListSelectionEx

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define BDLB_SEL_BARCODE          0x01 ///< StructListSelectionEx.selectionBarcode: Selection in Liste mit Barcode freigeben
#define BDLB_BESTAETIGEN          0x02 ///< StructListSelectionEx.selectionBarcode: Ausgewaehlten Eintrag bestätigen mit Return
#define BDLB_AUSBLENDEN           0x04 ///< StructListSelectionEx.selectionBarcode: Automatisches Ausblenden der Liste nach n Sekunden.

#define BDLB_SEL_TRANSPONDER      0x01 ///< StructListSelectionEx.flags: Selection in Liste mit Transponder freigeben
#define BDLB_SEL_TASTATUR         0x02 ///< StructListSelectionEx.flags: Selection in Liste mit Tastatur freigeben
#define BDLB_DONT_SHOW_BIG_WATCH  0x04 ///< StructListSelectionEx.flags: Bei Data on Card die Große Uhr ausblenden und Nachricht für Schreiben auf Ausweis anzeigen.
#define BDLB_INCREMENTAL_SEARCH   0x10 ///< StructListSelectionEx.flags: Inkrementelle Suche in Liste (TimeboyIV)
#endif

/** Diese Struktur dient in Eingabekettenfeldern bei der Einlesemethode Liste,
zur Angabe von weiteren Informationen.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructListSelectionEx
{
    uint8_t selection{};            ///< Flag Listenauswahl mit Selektion
    uint8_t selectionRecordField{}; ///< Feld im Datensatz für Selektion (nur wenn @ref selection = TRUE)
    uint8_t selectionListField{};   ///< Feld in der Liste nach dem selektiert wird (nur wenn @ref selection = TRUE)
    uint8_t listNumber{};           ///< Nummer der Liste
    uint8_t selectionField{};       ///< Feld in der Liste nach dem der Benutzer auswählt
    uint8_t dataField{};            ///< Feld in der Liste das im Datensatz gespeichert wird
    uint8_t selectionBarcode{};     ///< Die unteren drei Bit sind Flags, die oberen der Index der ausgewählten Anzeigedauer.
    uint8_t flags{};                ///< Eingabequellen: Transponder, Tastatur
    uint8_t noValueCheck{};         ///< Wert nicht in Liste
    uint8_t jumpSelectionNoEntry{}; ///< Sprungziel wenn @ref selectionNoEntry == 3
    uint8_t inputModuleTimeout{};   ///< Auszeit wie lange das Modul angeschaltet bleibt, wenn aktiviert.
    uint8_t transponderConfig{};    ///< Zu verwendende Konfiguration für lesen/schreiben auf Ausweise.
                                    ///< Oberes Nibble ist Index zum Lesen eines Wertes,
                                    ///< unters Nibble ist Index zum Schreiben eines Wertes.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_1[2]{};
#endif
    uint8_t doubleColumn{};             ///< War mal: MapFKey - Die ersten zehn Listeneinträge auf Funktionstasten legen.
                                        ///< Ist Jetzt: Zweispaltige Anzeige beim AE-MasterIV
    uint8_t selectionRecordField2{};    ///< wie @ref selectionRecordField
    uint8_t selectionListField2{};      ///< wie @ref selectionListField
    uint8_t selectionNoEntry{};         ///< Verhalten wenn kein Eintrag vorhanden ist.
    uint8_t usedFontSize{};             ///< Zu verwendente Schriftgröße bei der Listenanzeige.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_2[11]{};
#endif
    StructCompare compare;          ///< Sprungziels in eine Eingabekette definieren.
}; // 51 Byte


///////////////////////////////////////
// Struct: StructDataEx

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructDataEx
{
    uint8_t type{};     ///< Typ der Hintergrundbeleuchtung
                        ///< |Wert|Bedeutung       |
                        ///< |:-:|:----------------|
                        ///< |0  |Transponder      |
                        ///< |1  |Logo             |
                        ///< |2  |Tastatur         |
                        ///< |3  |benutzerdefiniert|
                        ///< |4  |Fingerprint      |
    uint8_t flags{};    ///< Flags zur Steuerung des Aktivierungszustandes der Hintergrundbeleuchtung
                        ///< |Bit|Bedeutung                                               ||
                        ///< |:-:|:----------------------------------------------------:|::|
                        ///< |   |nicht gesetzt|gesetzt                                    |
                        ///< |1  |einfarbig    |2-farbig (Ok, Esc)                         |
                        ///< |2  |zeitabhängig |bis das nächste Eingabefeld beendet wird   |
    uint16_t time{};    ///< Dauer in ms (500er Schritte)
                        ///< |Wert|Bedeutung              |
                        ///< |:--:|:----------------------|
                        ///< |0   |immer eingeschaltet    |
                        ///< |>0  |Aktivierungsdauer in ms|
    uint8_t redOK{};    ///< Rotanteil für Ok- und Standard
    uint8_t greenOK{};  ///< Grünanteil für Ok und Standard
    uint8_t blueOK{};   ///< Blauanteil für Ok und Standard
    uint8_t whiteOK{};  ///< Weißanteil für Ok und Standard
    uint8_t redESC{};   ///< Rotanteil für Esc
    uint8_t greenESC{}; ///< Grünanteil für Esc
    uint8_t blueESC{};  ///< Blauanteil für Esc
    uint8_t whiteESC{}; ///< Weißanteil für Esc
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[39]{};
#endif
}; // 51 Byte

///////////////////////////////////////
// Struct: StructGlobVarToHIDEx

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructGlobVarToHIDEx
{
    uint8_t nullByte{};         ///< Die Feldfunktion wurde zuvor mit @ref StructTextEx implementiert.
                                ///< Zur Abwärtskompatibilität wird hier eine Nullterminierung als Text vorgegeben.
    uint8_t ending{};           ///< Abschließendes Zeichen der globalen Variablen hinzufügen.
                                ///< |Wert|Bedeutung|
                                ///< |:--:|:--------|
                                ///< |0   |keine    |
                                ///< |1   |CR       |
                                ///< |2   |CR/LF    |
                                ///< |3   |Tabulator|
    uint8_t interfacePort{};    ///< Gewählte Schnittstelle über die versendet werden soll.
                                ///< |Wert|Bedeutung            |
                                ///< |:--:|:--------------------|
                                ///< |0   |HID                  |
                                ///< |1   |zusätzlicher COM-Port|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[48]{};
#endif
}; // 51 Byte

///////////////////////////////////////
// Struct: StructSwitchModuleEnergieEx
// Die Daten sind gleich mit der Struktur PowerModule.

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructSwitchModuleEnergieEx
{
    uint8_t module{};       ///< Gewähltes Modul
                            ///< |Wert|Bedeutung         |
                            ///< |---:|:-----------------|
                            ///< |1   |CPU               |
                            ///< |2   |Timer             |
                            ///< |3   |Eingänge          |
                            ///< |4   |global alles      |
                            ///< |5   |12V extern        |
                            ///< |6   |5V extern         |
                            ///< |7   |Kommunikation     |
                            ///< |8   |Displaybeleuchtung|
                            ///< |9   |Displaytext       |
                            ///< |10  |Biokey            |
                            ///< |11  |Transponder       |
                            ///< |12  |GPS               |
                            ///< |13  |USB-Host          |
                            ///< |14  |MMC               |
    uint8_t item{};         ///< Auswahl der Energieoption
                            ///< |Wert|Bedeutung                                                                                                                                 |
                            ///< |---:|:-----------------------------------------------------------------------------------------------------------------------------------------|
                            ///< |0   |Aus                                                                                                                                       |
                            ///< |1   |Ein                                                                                                                                       |
                            ///< |2   |Beibehalten aus vorausgegangenem Schema                                                                                                   |
                            ///< |3   |Sleep-Mode                                                                                                                                |
                            ///< |4   |Bedienung-TimeOut                                                                                                                         |
                            ///< |5   |EIN, wenn Modul in aktiver Feldfunktion benutzt wird                                                                                      |
                            ///< |6   |Beleuchtung Display Dunkel                                                                                                                |
                            ///< |7   |Kommunikation "Ein" wenn Datensätze vorhanden sind, "Aus" nach Timeout                                                                    |
                            ///< |8   |Kommunikation Zyklisch "Ein", nach Datenübertragung wieder "Aus" nach TimeOut, Aktivierungszeit in h oder als Uhrzeit                     |
                            ///< |9   |Nur Ein im BIOS-Menü                                                                                                                      |
                            ///< |10  |Zyklischer Betrieb Ein bis gültige Daten vorhanden sind und Zeitwert für Aus muss angegeben werden, Aktivierungszeit in h oder als Uhrzeit|
                            ///< |11  |Digitaler Eingang wird verwendet                                                                                                          |
                            ///< |12  |Maschine steht(Stopp)                                                                                                                     |
                            ///< |13  |Timeboy Steckevent EIN und nach TimeOut z.B. 10h wieder AUS                                                                               |
                            ///< |14  |EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.|
                            ///< |15  |EIN, wenn digitaler Eingang x min. 1s HIGH ist und eingestellter Timeout abgelaufen. AUS wenn digitaler Eingang x wieder min. 1s LOW ist. |
                            ///< |16  |Modul wird nur bei Bedarf aktiviert. Kann eventl. Verzögerungen hervorrufen.                                                              |
                            ///< |17  |EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.|
    uint32_t time{};        ///< Wert der Zeit in Sekunden
    uint8_t value{};        ///< Wert für Helligkeit oder andere Größen
    uint32_t interval{};    ///< Wert des Einschaltintervall der Zeit in Sekunden (value = 0) oder als Uhrzeit (value = 1)
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[40]{};
#endif
}; // 51 Byte

///////////////////////////////////////
// Struct: StructSystemMessageEx

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructSystemMessageEx
{
    uint8_t value{};    ///< Wert der übernommen werden soll.
                        ///< |Wert|Bedeutung         |
                        ///< |---:|:-----------------|
                        ///< |0   |Art|
                        ///< |1   |Id|
                        ///< |2   |id als Text|
                        ///< |3   |Parameter 1|
                        ///< |4   |Parameter 2|
                        ///< |5   |Parameter 3|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[50]{};
#endif
}; // 51 Byte

///////////////////////////////////////
// Struct: StructTakePhotoEx

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im Union-Teil der Struktur @ref StructReadField zur Anwendung.
*/
struct StructTakePhotoEx
{
    uint8_t flags{};    ///< Flags für Optionen.
                        ///< |Bit|Bedeutung (wenn gesetzt)                                                                                                |
                        ///< |:-:|:-----------------------------------------------------------------------------------------------------------------------|
                        ///< |1  |Wird die Kamera im Stichprobenmodus betrieben, kann durch setzten des Flags die Aufnahme erzwungen werden.              |
                        ///< |2  |Wird eine Nachricht angezeigt, wird bei ESC die Bildaufnhame verworfen.                                                 |
                        ///< |3  |Die Textnachricht wird so lange angezeigt, bis sie abgeschlossen wurde.                                                 |
                        ///< |^  |ShowTime wird mit 0 Sekunden angegeben. Das Speichern des Bildes kann je nach gewähltem Format bis zu 8 Sekunden dauern.|
                        ///< |4-8|-FREI-                                                                                                                  |
    uint8_t showTime{}; ///< Anzeigedauer der Nachricht.
                        ///< |Wert|Bedeutung                                                                   |
                        ///< |:---|:---------------------------------------------------------------------------|
                        ///< |0   |Keine Anzeige oder Warten bis Bildaufnahme abgeschlossen (@ref flags Bit 3).|
                        ///< |1-15|Sekunden                                                                    |
                        ///< |255 |Bestätigung durch ESC oder Enter.                                           |
    uint8_t showPeep{}; ///< Piepsignal, Wert wie bei Textmeldung anzeigen.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[8]{};
#endif
    char text[40]{};    ///< Meldungstext.
                        ///< Der Text darf maximal 39 Zeichen (ASCII) lang sein.
}; // 51 Byte

///////////////////////////////////////
// Struct: StructFeverControlEx

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructFeverControlEx
{
    uint8_t flagUnitFahrenheit : 1;              ///< Verwendete Einheit ist Fahrenheit, ansonsten Celsius.
    uint8_t flagShowTemperaturValue : 1;         ///< Temperaturwert in den Meldungen anzeigen.
    uint8_t flagValidMeasureRepeatable : 1;      ///< Erfolgreiche Messung durch Rückfrage Wiederholen können.
    uint8_t flagInvalidMeasureRepeatable : 1;    ///< Fehlerhafte Messung durch Rückfrage Wiederholen können.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t flagsFree : 4;                       /// Nicht verwendet
#endif
    float measureMin{};                          ///< Zulässige Mindesttemperatur für die Körpertemperaturmessung.
    float measureMax{};                          ///< Zulässige Maximaltemperatur für die Körpertemperaturmessung.
    float measureFever{};                        ///< Temperatur für Fieber bei der Körpertemperaturmessung.
    uint8_t indexGlobVar{};                      ///< Globale Variable für Ablage des Ergebnisstatus.
    uint8_t indexRecordField{};                  ///< Datensatzfeld für die Abalge des Ergebnisstatus.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[36]{};                        ///< Unbenutzt
#endif
}; // 51 Byte

///////////////////////////////////////
// Struct: StructExMultiInput

/** Diese Struktur enthält zusätzliche Daten zu einer Feldfunktion.

Diese Struktur kommt im union-Teil der Struktur StructFeldEinlesen zur Anwendung.
*/
struct StructMultiInputSourceEx
{
    uint8_t childCount;                          ///< Anzahl der abgelegten Untereinträge
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint32_t ptr;                                ///< Basisoffset auf das Offsetarray der Untereinträge
#endif
    uint8_t keypadInput : 1;                     ///< Tastatur als Eingabequelle.
    uint8_t barcodeInput : 1;                    ///< Barcode als Eingabequelle.
    uint8_t transponderInput : 1;                ///< Transponder als Eingabequelle.
    uint8_t fingerprintIdentificationInput : 1;  ///< Fingerprint Identifikation als Eingabequelle.
    uint8_t fingerprintVerificationInput : 1;    ///< Fingerprint Verifikation als Eingabequelle.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t inputFree : 3;                       /// Nicht verwendet
#endif
    uint8_t indexGlobaleVariable{};              ///< Globale Variable für Ablage der Eingabequellekennung.
    uint8_t indexRecordField{};                  ///< Datensatzfeld für die Abalge der Eignabequellekennung.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[22]{};                        ///< Unbenutzt
#endif
    StructCompare compare;                       ///< Sprungziels in eine Eingabekette definieren.
}; // 51 Byte

///////////////////////////////////////
// Struct: StructReadField

// Einlesemethoden für Felder
#define TBFE_NORM                         1 ///< Standardmethode.
#define TBFE_GET_TIME                     2 ///< Aktuelles Datum-Uhrzeit übernehmen.
#define TBFE_LIST_SELECT_GET              3 ///< Aus Liste auswählen um Wert von Liste zu übernehmen.
#define TBFE_KONSTANTE                    4 ///< Konstante in den Datensatz schreiben 
#define TBFE_BESTAETIGUNG                 5 ///< Bestätigung abfragen (füllt kein Datenfeld).
#define TBFE_COPY_GV_TO_FIELD             6 ///< Globale Variable in Feld kopieren.
#define TBFE_SERIENNUMMER                 7 ///< Seriennummer übernehmen 
#define TBFE_ANALOG_WERT                  8 ///< Analogen Messwert in Feld übernehmen.
#define TBFE_DIGITAL_WERT                 9 ///< Digitalen Messwert in Feld übernehmen.
#define TBFE_ZAEHLER_WERT                10 ///< Zählerstand in Feld übernehmen.
#define TBFE_ENTRANCE_PNR                11 ///< Eingelesene Personalnummer übernehmen.
#define TBFE_ENTRANCE_STATUS             12 ///< Status von Zutritt übernehmen.
#define TBFE_FUNC                        13 ///< Funktionswert übernehmen.
#define TBFE_SPS_WERT                    14 ///< Wert von SPS übernehmen.
#define TBFE_GPRS_ALIVE_COUNTER          15 ///< Aufgelaufene Alivedatensätze (Zähler) übernehmen.
#define TBFE_FIRMWARE_VERSION            16 ///< Firmware-Version übernehmen.
#define TBFE_ONLINE_STATUS               17 ///< String "online" oder "offline" als -Server Online- Zustand übernehmen
#define TBFE_ENTRANCE_ZM_NUM             18 ///< Zutrittsmasternummer übernehmen.
#define TBFE_ENTRANCE_READER_NUM         19 ///< Lesernummer übernehmen.
#define TBFE_SUMMER_WINTER_STATUS        20 ///< (S) für Sommerzeit oder (W) für Winterzeit übernehmen.
#define TBFE_GPS_RECORD_FIX              21 ///< GPS - Daten übernehmen. Feste Vorgabe des Aufbaus
#define TBFE_ENTRANCE2_IDENTIFICATION    22 ///< Zutrittsprüfung durchführen. 
#define TBFE_COPY_FIELD_TO_GV            23 ///< Feldwert in globale Variable kopieren.
#define TBFE_FINGERPRINT_SCANN           24 ///< Finger scannen und Fingertemplate liefern
#define TBFE_FINGERPRINT_CARD            25 ///< Fingertemplate's von Karte lesen.
#define TBFE_FINGERPRINT_TEACH           26 ///< Finger einlernen
#define TBFE_FINGERPRINT_IDENTIFICATION  27 ///< Identifikation durchführen
#define TBFE_FINGERPRINT_VERIFICATION    28 ///< Verifikation durchführen
#define TBFE_FINGERPRINT_DELETE          29 ///< Fingertemplate löschen
#define TBFE_TIMEBOY_COPY_GV_TO_FIELD    30 ///< Globale Variable von Timeboy übernehmen
#define TBFE_ANALOG_STATUS               31 ///< Status eines Analogen Eingangs übernehmen
#define TBFE_LIST_SELECT_SET             32 ///< Aus Liste auswählen um Wert in Liste zu übernehmen.
#define TBFE_OPERATION                   33 ///< Mathematisch oder logische Operation ausführen.
#define TBFE_TIMEBOY_COPY_FIELD_TO_GV    34 ///< Feldwert in globale Variable des Timeboy übernehmen.
#define TBFE_SWITCH_RELAY                35 ///< Relais schalten
#define TBFE_GPS_RECORD_VARIABLE         36 ///< GPS - Daten übernehmen. Variable Auswahl aus Daten.
#define TBFE_TIMER                       37 ///< Timer starten/stoppen
#define TBFE_TRANSPONDER_WRITE           38 ///< Transponderwert auf Karte schreiben
#define TBFE_SWITCH_LED                  39 ///< LED einer Maschine schalten
#define TBFE_GPS_DISTANCE                40 ///< Zurückgelegte Entfernung in m übernehmen.
#define TBFE_MOBILE_PARAMETER            41 ///< Parameter des Mobilfunks übernehmen. Ordsbestimmung.
#define TBFE_SWITCH_BUZZER               42 ///< Buzzer schalten
#define TBFE_KONSTANTE_TYPE_DEVICE_NAME  43 ///< Gerätename übernehmen.
#define TBFE_DATA_ON_CARD                44 ///< Datenarray auf Transponderkarten schreiben.
#define TBFE_SEND_SMS                    45 ///< SMS versenden mit maximal 50 Zeichen Text.
#define TBFE_SCRIPT                      46 ///< Scripttext mit maximal 50 Zeichen.
#define TBFE_MOBILMASTERIV_MENUENTRY     47 ///< Menüeintrag für TimeboyIV bei einer Mobil-MasterIV Anbindung.
#define TBFE_SWITCH_COMMUNICATION        48 ///< Hautpkommunikation umschalten.
#define TBFE_GV_TO_INTERFACE             49 ///< Werte von globaler Variable über Schnittstelle (HID, COM, etc.) senden. 
#define TBFE_SWITCH_BACKLIGHT            50 ///< Hintergrundbeleuchtung schalten.
#define TBFE_SHOW_MESSAGE                51 ///< Nachricht anzeigen.
#define TBFE_SWITCH_MODULE_ENERGIE       52 ///< Energiezustand eines Moduls einstellen.
#define TBFE_SYSTEM_MESSAGE              53 ///< Einen der Werte von einer Systemmeldung übernehmen.
#define TBFE_TAKE_PHOTO                  54 ///< Ein Photo erstellen.
#define TBFE_TIMER_STATE                 55 ///< Status der Zeitschaltuhr übernehmen
#define TBFE_FINGERPRINT_ADMNISTARTION   56 ///< Für das Saturmodul benötig da hier alle Funktionen von dem Gerät übernommen werden
#define TBFE_VIBRATION_VALUE             57 ///< Vibrationszähler übernehmen
#define TBFE_READ_VARIABLE               58 ///< Setup- oder Systemvariable lesen.
#define TBFE_WRITE_VARIABLE              59 ///< Setup- oder Systemvariable schreiben.
#define TBFE_SIGNATURE                   60 ///< Unterschrift
#define TBFE_MULTI_INPUT_SOURCE          61 ///< Multieingabequelle. Es werden mehrere Feldfunktionen für die Werteingabe in einer Kumuliert.
#define TBFE_FEVER_CONTROL               62 ///< Fiebermessung am Handgelenk durchführen
#define TBFE_LONG_BARCODE                63 ///< Langen Barcode lesen und in eine Datendatei wegschreiben.

#define TBFE_MAX                         64 ///< Maximale Anzahl Funktionen fuer Arraygroesse

#define TBFE_DEBUG                      255 ///< Einlesemethode für interne Zwecke

#ifndef DOXYGEN_SHOULD_SKIP_THIS
// FeldEinlesenFlag
#define FEF_APPEND_TO_FIELD            0x01 ///< StructReadField.flags: Der ermittelte Wert an den Feldwert anfuegen.
#define FEF_APPEND_TO_GV               0x02 ///< StructReadField.flags: Der ermittelte Wert an die globale Variable anfuegen.

// Verhalten am Ende einer Eingabekette oder bei Eingabekettenfeld. 
#define EV_UPPER_MENUE                    0 ///< Uebergeordnetes Untermenue 
#define EV_NEXT_FIELD                   253 ///< Naechstes Eingabekettenfeld
#define EV_MAIN_MENUE                   254 ///< Hauptmenue
#define EV_NEXT_SIBLING                 255 ///< Naechste Eingabekette bzw. U-Menue. 
#endif

/** Diese Struktur enthält Informationen zu einem Eingabekettenfeld.

Der enthaltene Union-Teil wird abhängig von dem verwendeten Typ genutzt.
*/
struct StructReadField
{
    uint8_t type{};                 ///< Typ der Einlesemethode "TBFE..." 
    uint8_t recordFieldNumber{};    ///< Nummer des Datensatzfeldes (ab 0) 
    uint8_t behaviourAtEsc{};       ///< Verhalten bei ESC-Taste.
                                    ///< Sprungziel (Bedeutung siehe @ref StructReadRecord)
    char name[17]{};                ///< Überschrift.
                                    ///< Die Überschrift darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t header2Text{};          ///< Text in Kopfzeile 2.
                                    ///< |Bit  |Bedeutung                   |
                                    ///< |:---:|:---------------------------|
                                    ///< |1...4|Nummer der globalen Variable|
                                    ///< |5...8|=0 -> Konstanter Text       |
                                    ///< |^    |>0 -> globale Variable      |
    uint8_t globalVariable{};       ///< Feldwert in globale Variable kopieren.
                                    ///< Aufbau wie @ref header2Text
    uint8_t systemVariable{};       ///< Systemvariable
                                    ///< |Bit  |Bedeutung (wenn gesetzt)                                               |
                                    ///< |:---:|:----------------------------------------------------------------------|
                                    ///< |1    |Barcode oder Transponder sofort einschalten                            |
                                    ///< |2    |Durch den Buzzer die Transponder- oder Barcodeaktivierung signalisieren|
                                    ///< |3-4  |0 -> Keine Vorbelegung                                                 |
                                    ///< |^    |1 -> Vorbelegung durch Feld                                            |
                                    ///< |^    |2 -> Vorbelegung durch globale Variable                                |
                                    ///< |5    |Wert von Bit 7 liegt als String vor. *1)                               |
                                    ///< |6    |Wert von Bit 8 liegt als String vor.                                   |
                                    ///< |7    |Wert einer Systemvariablen wird geschrieben                            |
                                    //           [0 = Timeout, 1 = Zählerteiler, 2 = Zyklus, 3 = Kavität, 4 = Hubzahl]
                                    ///< |^    |Der Offset, welcher Typ geschrieben wird steht in @ref globalVariable. |
                                    ///< |^    |Die Adresse der Systemvariable steht in @ref variable2.                |
                                    ///< |8    |Wert einer Systemvariablen wird in Zeile 4 der Anzeige angezeigt       |
                                    //           [0 = Timeout, 1 = Zählerteiler, 2 = Zyklus, 3 = Kavität, 4 = Hubzahl]
                                    ///< |^    |Der Offset, welcher Typ angezeigt wird steht in @ref header2Text.      |
                                    ///< |^    |Die Adresse der Systemvariable steht in @ref variable1.                |
                                    ///< *1) Ansonsten handelt es sich um eine Binärarray.
                                    ///<        Timeout  Zählerteiler
                                    ///<     E1[00000001 00000002 ect.]
                                    ///< Hierbei wird in @ref variable2 der Grundoffset E1 angegeben und @ref globalVariable.
    uint8_t barcodeRangeFrom{};     ///< Startwert des relevanten Barcodebereichs (Wert ausschneiden)
    uint8_t barcodeRangeUpTo{};     ///< Endwert des relevanten Barcodebereichs (Wert ausschneiden)
                                    ///< ###barcodeRangeFrom == 0
                                    ///< Ist @ref barcodeRangeUpTo == 0, dann ist das Ausschneiden deaktiviert, ansonsten werden von rechts @ref barcodeRangeUpTo Zeichnen des Wertes geliefert.
                                    ///< ###barcodeRangeFrom == 255
                                    ///< Es werden @ref barcodeRangeUpTo Zeichen von Links aus dem Wert geliefert.
                                    ///< ###barcodeRangeFrom > 0 && barcodeRangeUpTo < 255
                                    ///< Es werden von der Position @ref barcodeRangeFrom bis zur Position @ref barcodeRangeUpTo die Zeichen mitten aus dem Wert geliefert.
    uint8_t inputTypeCount{};       ///< Beschreibt bei Methode *Normal* die Feldlänge und den Typ.
    uint8_t variable1{};            ///< Variable, die abhängig von der jeweiligen Einlesemethode verwendet wird.
    uint8_t variable2{};            ///< Variable, die abhängig von der jeweiligen Einlesemethode verwendet wird.

    union
    {
        StructTextEx text;
        StructConfirmationEx confirmation;
        StructNumFieldEx numField;
        StructListSelectionEx listSelection;
        StructDataEx data;
        StructGlobVarToHIDEx globVarToHID;
        StructSwitchModuleEnergieEx switchModuleEnergie;
        StructSystemMessageEx systemMessage;
        StructTakePhotoEx takePhoto;
        StructFeverControlEx feverControl;
        StructMultiInputSourceEx multiInputSource;
    } ex = { {} };
}; // 79 Byte


///////////////////////////////////////
// Struct: StructGlobalVar

/** Diese Struktur enthält die Namen der definierten globalen Variablen.

Diese Struktur zeigt mit @ref ptr auf den nächsten Eintrag. Sie selbst ist standardmäßig die erste Struktur
in den Setupdaten.
*/
struct StructGlobalVar
{
    char names[8][17]{};    ///< Namen der globalen Variablen
                            ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
    uint32_t ptr{};         ///< Zeiger auf Root @ref StructOperating
}; // 140 Byte


///////////////////////////////////////
// Struct: StructConfig

// Transpondertypen: transponderType
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define TRANSPONDER_NONE                     0x3F
#define TRANSPONDER_UNIQUE                   0
#define TRANSPONDER_HITAG2                   1
#define TRANSPONDER_LEGIC                    2
#define TRANSPONDER_HITAG1                   3
#define TRANSPONDER_HITAGS_H32               4
#define TRANSPONDER_MIFARE                   5
#define TRANSPONDER_Q5                       6
#define TRANSPONDER_HITAGS_H56               7
#define TRANSPONDER_HITAGS_H48               8
#define TRANSPONDER_IBUTTON                  9
#define TRANSPONDER_SIMONSVOSS              10
#define TRANSPONDER_NEDAP                   11
#define TRANSPONDER_TITAN                   12
#define TRANSPONDER_HITAG1_CRYPT            13
#define TRANSPONDER_HITAG1_SECURITY         14
#define TRANSPONDER_HID_PROXKEY             15
#define TRANSPONDER_HID_ICLASS              16
#define TRANSPONDER_LEGIC_ADVANT            17
#define TRANSPONDER_MIFARE_DESFIRE          18
#define TRANSPONDER_MIFARE_PLUS             19
#define TRANSPONDER_MIFARE_ULTRALIGHT       20
#define TRANSPONDER_ICODE                   21
#define TRANSPONDER_MYD                     22
#define TRANSPONDER_ISO_TAGS                23
#define TRANSPONDER_BLUEBOX_ID_TRONIC       24
#define TRANSPONDER_UNIQUE_FIT              25
#define TRANSPONDER_JAVACARD                26
#define TRANSPONDER_OCL                     27
#define TRANSPONDER_WIEGAND                 28
#define TRANSPONDER_CLOCK_DATA              29
#define TRANSPONDER_HITAG1_FEIG             30
#define TRANSPONDER_PRECONFIGURED_READER    31

enum class TransponderType : uint8_t
{
    Undefined              = 0xFF,
    None                   = 0x3F,
    Unique                 = 0,
    Hitag_2                = 1,
    Legic_prime            = 2,
    Hitag_1                = 3,
    Hitag_S_H32            = 4,
    Mifare_classic         = 5,
    Q5                     = 6,
    Hitag_S_H56            = 7,
    Hitag_S_H48            = 8,
    IButton                = 9,
    Simons_voss            = 10,
    Nedap                  = 11,
    Titan                  = 12,
    Hitag_1_crypt          = 13,
    Hitag_1_security       = 14,
    HID_Proxkey            = 15,
    HID_IClass             = 16,
    Legic_advant_prime     = 17,
    Mifare_desfire         = 18,
    Mifare_plus            = 19,
    Mifare_ultra_light     = 20,
    ICode                  = 21,
    MYD                    = 22,
    ISO_Tags               = 23,
    Bluebox_id_tronic      = 24,
    Unique_FIT             = 25,
    Java_card              = 26,
    OCL                    = 27,
    Wiegand                = 28,
    Clock_data             = 29,
    Hitag_1_Feig           = 30,
    Preconfigured_reader   = 31
};
#endif

/** Diese Struktur enthält die Grundeinstellungen zum Gerät.

Diese Struktur beinhaltet alle Gruneinstellungen des Geräts, sowie globale Einstellungen die ggf. an anderen
Stellen angepasst werden können.
*/
struct StructConfig
{
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_1[3]{};
#endif
    uint8_t backlightLCD{};                     ///< Hintergrundbeleuchtung LCD
    uint16_t waitingTimeForReply{};             ///< Wartezeit auf Antwort vom PC
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_2{};
#endif
    uint8_t onlineOperation{};                  ///< Für die Definition des Online- / Offline-Betriebs
    uint16_t passwordSystemMenu{};              ///< Passwort für BIOS
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_3[7]{};
#endif
    uint8_t operatingMode{};                    ///< Betriebsmodus des Gerätes
                                                ///< |Wert|Bedeutung  |
                                                ///< |:---|:----------|
                                                ///< |0   |Normal     |
                                                ///< |1   |PZE-Modus 1|
                                                ///< |2   |PZE-Modus 2|
                                                ///< |3   |MDE        |
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_4[11]{};
#endif
    uint8_t deactivateOnOffButton{};            ///< Ein-/Austaste aktivieren
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_5{};
#endif
    uint8_t showInfoPage{};                     ///< Infoseite anzeigen
    uint8_t presentationModeUnique{};           ///< Darstellungsformat des UNIQUE Transponders
                                                ///< |Wert|Bedeutung                       |
                                                ///< |:---|:-------------------------------|
                                                ///< |0   |Dezimal (13 Stellen)            |
                                                ///< |1   |Hexadezimal (10 Stellen)        |
                                                ///< |3   |Hexadezimal gedreht (10 Stellen)|
                                                ///< |4   |Rexroth Codec (13 Stellen)      |
    uint8_t transponderModeUnique{};            ///< Modus des UNIQUE Transponders
                                                ///< |Wert|Bedeutung     |
                                                ///< |:---|:-------------|
                                                ///< |0   |32bit (4Byte) |
                                                ///< |1   |40bit (5 Byte)|
    StructTransponder transponderGlobal;        ///< globale Struktur, wie eine Karte gelesen wird
    StructTransponder transponderAccessControl; ///< globale Struktur, wie bei Zutritt gelesen wird
    uint8_t autoIncrementFlags{};               ///< Flags für Autoinkrement beim Karte schreiben
    uint16_t autoIncrementValue{};              ///< 16Bit-Wert um den inkrementiert wird
    uint8_t transponderCutFlags{};              ///< Flags für die Zuschneidefunktion der Legic/Mifare Leser
    uint8_t transponderCutFrom{};               ///< Start für Ausschneiden von Transponderwert bei Legic & Mifare
    uint8_t transponderCutLength{};             ///< Länge des auszuschneidenen Transponderwerts 
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_6{};
#endif
    uint8_t ioModuleActive{};                   ///< Betriebszusatnd des IO-Moduls
                                                ///< |Wert|Bedeutung  |
                                                ///< |:---|:----------|
                                                ///< |0   |deaktiviert|
                                                ///< |1   |aktiviert  |
    uint8_t transponderSegment{};               ///< Segmentnummer bei segmentierten Karten
    uint8_t transponderType{};                  ///< Transpondertyp
                                                ///< |Wert|Hex |Bedeutung                 |
                                                ///< |:---|:---|:-------------------------|
                                                ///< |0   |0x00|Unique (EM4102/EM4200)    |
                                                ///< |1   |0x01|Hitag2                    |
                                                ///< |2   |0x02|Legic Prime               |
                                                ///< |3   |0x03|Hitag1                    |
                                                ///< |4   |0x04|HitagS H32                |
                                                ///< |5   |0x05|Mifare Classic            |
                                                ///< |6   |0x06|Q5 / ATA5577              |
                                                ///< |7   |0x07|HitagS H56                |
                                                ///< |8   |0x08|HitagS H48                |
                                                ///< |9   |0x09|iButton                   |
                                                ///< |10  |0x0A|SimonVoss                 |
                                                ///< |11  |0x0B|Nedap                     |
                                                ///< |12  |0x0C|Titan (EM4450)            |
                                                ///< |13  |0x0D|Hitag1 (Crypt)            |
                                                ///< |14  |0x0E|Hitag1 (Security)         |
                                                ///< |15  |0x0F|HID (ProxKey)             |
                                                ///< |16  |0x10|HID (iClass)              |
                                                ///< |17  |0x11|Legic Advant (auch Prime) |
                                                ///< |18  |0x12|Mifare DESFire            |
                                                ///< |19  |0x13|Mifare Classic / Plus     |
                                                ///< |20  |0x14|Mifare UltraLight (auch C)|
                                                ///< |21  |0x15|iCode                     |
                                                ///< |22  |0x16|MyD                       |
                                                ///< |23  |0x17|ISO Tags (13,56 MHz)      |
                                                ///< |24  |0x18|ETSI Tags (865-867 MHz)   |
                                                ///< |25  |0x19|Unique FIT                |
                                                ///< |26  |0x1A|Java Card                 |
                                                ///< |27  |0x1B|Octopus Reader            |
                                                ///< |28  |0x1C|Wiegand                   |
                                                ///< |29  |0x1D|Clock Data                |
                                                ///< |30  |0x1E|Hitag1 (Feig)             |
                                                ///< |31  |0x1F|Vorkonfigurierter Leser   |
                                                ///< |63  |0x3F|Keiner                    |
                                                ///< Bit 8 gibt die Segmentverwendung an
                                                ///< Bit 7 steht für "nur Seriennummer lesen"
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_7{};
#endif
    uint8_t keySound{};                         ///< Tastenton
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_8[5]{};
#endif
    uint8_t signalLastFunctionKey{};            ///< Letzte Funktionstaste durch blinken abzeigen, wie AE3
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_9[2]{};
#endif
    uint32_t initialValue3{};                   ///< Startwert für Kartensegment 3 beschreiben (ist im AEIII+ die IP-Adresse)
    uint8_t enableEntrance{};                   ///< Zutrittskontolle Version 1 aktivieren
    uint8_t useExternalReader{};                ///< externer Leser verwenden
    uint8_t doorSignalTime{};                   ///< Dauer für Türöffner
    uint8_t doorMonitoring{};                   ///< Türüberwachung aktivieren
    uint8_t doorAlarmTime{};                    ///< Alarmzeit für geöffnete Tür
    uint8_t entranceMainMenu{};                 ///< Bit 0: Überprüfen von Zutritt im Hauptmenü

    uint8_t enableCut{};                        ///< Zuschneiden aktivieren
    uint8_t cutFrom{};                          ///< Zuschnitt von
    uint8_t cutUpTo{};                          ///< Zuschnitt bis
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused_10[5]{};
#endif
    uint32_t initialValue1;                     ///< Startwert für Kartensegment 1 beschreiben
    uint8_t entranceFlag{};                     ///< Bit 0: Prüfung der Feiertage bei Zeitsteuerung der digitalen Ausgänge
                                                ///< War Typ für Wert des Kartenpräfix
    uint8_t showBigWatchAlways{};               ///< Große Uhr immer anzeigen
    uint8_t timeoutPZE{};                       ///< Timeout nach Eingaben zur Prüfung der Tastenumschalttabelle
    uint8_t lampFlagsMDE{};                     ///< Flags für die (rote) Lampensteuerung der MDE
    uint8_t peepFlags{};                        ///< Flags der aktustischen Bestätigungssignale
                                                ///< |Bit|Bedeutung (wenn gesetzt)   |
                                                ///< |:-:|:--------------------------|
                                                ///< |1  |keine Bestätigung für Ok   |
                                                ///< |2  |keine Bestätigung für ERROR|
                                                ///< ###Bedeutung vor Version 04.01.05
                                                ///< Feste Stellenanzahl für Kartenwert (0 => aus; wird links mit Nullen aufgefüllt)
    uint16_t transponderUserPassword{};         ///< Passwort für Benutzermenü (Kartenprogrammiermenue/MMC/USB-Host etc.); 0 => aus; 16bit
    uint32_t initialValue2{};                   ///< Startwert für Kartensegment 2 beschreiben
    uint8_t enableEntrance2{};                  ///< Zutrittskontrolle Version 2 aktivieren
}; // 100 Byte

// ########################################################
// Erweiterter Flash

///////////////////////////////////////
// Struct: StructExtendedFlash

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define EXTENDED_CHECK      0xAAAAAAAE ///< StructExtendedFlash.ulCheck: Prüfwert für Gültigkeit (Versionskennung)

#define EXTF_EMPTY          0x00000000 ///< StructExtendedFlash.flags: Nicht vorhanden.
#define EXTF_NEW            0x00000001 ///< StructExtendedFlash.flags: Struktur angelegt.
#define EXTF_DOWNLOAD       0x00000010 ///< StructExtendedFlash.flags: Aus Geraet ausgelesene Daten.

/** Diese Struktur enthält die Kopfdaten des erweiterten Speichers.

Diese Struktur enthält die Adressen der einzelnen Blöcke. Sie ist die erste und einzige Struktur ihrer Art
im erweiterten Speicherblock (16kByte).
*/
struct StructExtendedFlash
{
    uint32_t check{};                               ///< muss immer mit @ref EXTENDED_CHECK belegt werden.
    uint32_t size{};                                ///< Größe der Stuktur @ref StructExtendedFlash
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint32_t offsetIOModule{};                      ///< Offset auf IOModul - Struktur
    uint32_t offsetIOModule8DigitalInputs{};        ///< Offset auf die 8 @ref StructDigitalInput - Strukturen des IOModul
    uint32_t offsetIntern4DigitalInputs{};          ///< Offset auf die 4 @ref StructDigitalInput - Strukturen für intern.
    uint32_t offsetEventChains{};                   ///< Offset auf PointerArray der Eingabeketten für MDE
    uint32_t offsetFreeScope{};                     ///< Offset auf nächsten freien Speicherbereich im ExtendedFlashMem
    uint32_t offsetAccessControlTablesV1{};         ///< Offset auf die Tabellen der Zutrittskontrolle Version 1.
    uint32_t offsetAccessControlEventChainsV1{};    ///< Offset auf OffsetArray der Eingabeketten für Zutrittskontrolle Version 1.
    uint32_t offsetGlobalVariableValues{};          ///< Offset auf Struktur mit Standardwerten der globalen Variablen
    uint32_t offsetAccessControlTablesV2{};         ///< Offset auf die Tabellen für Zutrittskontrolle Version 2.
    uint32_t offsetAccessControlEventChainsV2{};    ///< Offset auf OffsetArray der Eingabeketten für Zutrittskontrolle Version 2.
    uint32_t offsetExtendedStruct{};                ///< Offset auf Struktur für erweiterte Funktionalitäten
    uint32_t offsetIntern4DigitalInputsPlus{};      ///< Offset auf die weiteren 4 @ref StructDigitalInput - Strukturen für intern.
    uint32_t offsetIntern2AnalogInputs{};           ///< Offset auf die 2 @ref StructAnalogInput - Strukturen für intern.
    uint32_t offsetEnergieSchemas{};                ///< Offset auf die Energieschemen
    uint32_t offsetMachineDefinitions{};            ///< Offset auf die Maschinendefinitionen
    uint32_t offsetEnergieMemo{};                   ///< Offset auf Zeichenfolge für Memo der Energieverwaltung
    uint32_t offsetTransponderConfiguration{};      ///< Offset auf die Transponderkonfigurationen
    // Die folgenden Zeiger werden von DatafoxStudioIV-Versionen < 04.03.03.02 ebenfalls gelesen, aber überschrieben.
    uint8_t unused_1[20];
#endif
    uint32_t flags;                                 ///< Flags für Struktur und erweiterten Speicher
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    // Ab der Version 04.03.03.01 auf 200 Byte vergrößert.
    // Die folgenden Zeiger werden von DatafoxStudioIV-Versionen < 04.03.03.02 nicht gelesen.
    uint32_t offsetInternDigitalInputs{};           ///< Offset auf @ref StructHeaderOffsetArray für die @ref StructDigitalInput - Strukturen intern (ersetzt pulMDEDigital).
    uint32_t offsetInternDigitalInputsExt{};        ///< Offset auf @ref StructHeaderOffsetArray für die @ref StructDigitalInputExt - Strukturen intern (ersetzt pulMDEDigitalExt).
    uint32_t offsetInternAnalogInputs{};            ///< Offset auf @ref StructHeaderOffsetArray für die @ref StructAnalogInput - Strukturen intern (ersetzt pulMDEAnalog).
    uint32_t offsetTimers{};                        ///< Offset auf @ref StructHeaderOffsetArray für die @ref StructTimer - Stukturen (ersetzt pulExtended.timer[4]).
    uint32_t offsetVibrationSensor{};               ///< Offset auf eine @ref StructVibrationSensor für den Vibrationssensor.
    uint32_t offsetSystemMessages{};                ///< Offset auf @ref StructHeaderOffsetArray für die @ref StructSystemMessage - Strukturen für Systemmeldungen.
    uint8_t unused_2[76]{};
#endif
}; // 200 Byte
#endif

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define sizeof_StructExtendedFlash_1    100 // Strukturgroesse der Version 1 ist: 100Byte
#define sizeof_StructExtendedFlash_2    200 // Strukturgroesse der Version 2 ist: 200Byte
#endif

#ifndef DOXYGEN_SHOULD_SKIP_THIS
///////////////////////////////////////
// Struct: StructHeaderOffsetArray

/** Diese Struktur enthält die Kopfdaten für folgende OffsetArray - Strukturdaten.

Diese Struktur ist eine vereinheitlichte Hilfstruktur für OffsetArray-Strukturdaten,
wie z. B. für @ref StructDigitalInput, @ref StructDigitalInputExt, @ref StructAnalogInput, @ref StructTimer, etc.
*/
struct StructHeaderOffsetArray
{
    uint32_t size{};        ///< Version / Größe der Stuktur @ref StructHeaderOffsetArray
    uint32_t count{};       ///< Anzahl der Pointer in dem Array
    uint32_t offsetArray{}; ///< Basisoffset auf die Sturkturbasis, zum Pointerarray.
}; // 12 Byte
#endif

///////////////////////////////////////
// Struct: StructIOModule

/** Diese Struktur enthält die Informationen zum IO-Module.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructIOModule
{
    // Spannungseingänge
    uint8_t voltageInputsActive{};          ///< Flags welcher Eingang aktiv ist.
    char voltageInputsName[8][17]{};        ///< Bezeichner der Eingänge.
                                            ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
    float voltageInputsMinValue[8]{};       ///< Minimalwert für die Berechnung.
    float voltageInputsMaxValue[8]{};       ///< Maximalwert für die Berechnung.
    char voltageInputsUnit[8][5]{};         ///< Einheit - suffix der Berechnung.
                                            ///< Die Einheit darf maximal 4 Zeichen (ASCII) lang sein.
    uint8_t voltageInputsShowOnScreen{};    ///< Flags ob die Ausgabe auf dem Display erlaubt ist.

    // Stromeingänge
    uint8_t currentInputsActive{};          ///< Flags welcher Eingang aktiv ist.
    char currentInputsName[8][17]{};        ///< Bezeichner der Eingänge
                                            ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
    float currentInputsMinValue[8]{};       ///< Minimalwert für die Berechnung.
    float currentInputsMaxValue[8]{};       ///< Maximalwert für die Berechnung.
    char currentInputsUnit[8][5]{};         ///< Einheit - suffix der Berechnung.
                                            ///< Die Einheit darf maximal 4 Zeichen (ASCII) lang sein.
    uint8_t currentInputsShowOnScreen{};    ///< Flags ob die Ausgabe auf dem Display erlaubt ist.
}; // 484 Byte


///////////////////////////////////////
// Struct: StructDigitalInput

/** Diese Struktur enthält die Informationen für die Behandlung eines digitalen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructDigitalInput
{
    uint8_t useAs{};                        ///< Verwendung als Start/Stop, Zaehler etc..
    uint8_t startLowHigh{};                 ///< auszuführende Eingabekette bei Flankenwechsel low -> high
    uint8_t startHighLow{};                 ///< Auszuführende Eingabekette bei Flankenwechsel high -> low
    uint16_t countDivisor{};                ///< bei Verwendung als Zähler, dessen Teiler
    uint16_t cycle{};                       ///< bei Verwendung als Zähler, dessen Zyklus.
    uint16_t timeout{};                     ///< bei Verwendung als Zähler mit Start/Stop, dessen Timeout.
    uint32_t countDivisor32Bit{};           ///< bei Verwendung als Zähler, dessen Teiler als 32bit
    uint16_t debounceHigh{};                ///< Endprellzeit für Flankenwechsel Low-High
    uint16_t debounceLow{};                 ///< Endprellzeit für Flankenwechsel High-Low
    uint8_t invokeInputChainCountDivisor{}; ///< auszuführende Eingabekette bei erreichtem Zähler
    uint8_t invokeInputChainCycle{};        ///< auszuführende Eingabekette bei erreichtem Zyklus
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[2]{};
#endif
}; // 21 Byte

///////////////////////////////////////
// Struct: StructDigitalInputExt

/** Diese Struktur enthält zusätzliche Informationen für die Behandlung eines digitalen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructDigitalInputExt
{
    uint16_t cavity{};  ///< Die zu verwendende Kavität.
    uint16_t strokes{}; ///< Die zu verwendende Hubzahl.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[17]{};
#endif
}; // 21 Byte

///////////////////////////////////////
// Struct: StructAnalogInput

/** Diese Struktur enthält die Informationen für die Behandlung eines analogen Einganges.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructAnalogInput
{
    uint8_t useAs{};                ///< Eingangsverwendung als ...
    uint8_t show{};                 ///< Flags ob Ausgabe auf Display erlaubt.
    uint8_t sampleRate{};           ///< Abtastrate für den analogen Eingang
    char name[17]{};                ///< Bezeichner der Eingänge.
                                    ///< Der Name darf maximal 16 Zeichen (ASCII) lang sein.
    char unit[5]{};                 ///< Einheit - suffix der Berechnung.
                                    ///< Die Einheit darf maximal 4 Zeichen (ASCII) lang sein.
    float minLimit{};               ///< Minimalwert für Berechnung.
    float maxLimit{};               ///< Maximalwert für Berechnung.
    float minInput{};               ///< Minimalwert für Eingangsspannung.
    float maxInput{};               ///< Maximalwert für Eingangsspannung.
    float limitUpperAlarm{};        ///< Wert für die obere Alarmgrenze
    float limitUpperWarning{};      ///< Wert für die obere Warngrenze
    float limitLowerWarning{};      ///< Wert für die untere Warngrenze
    float limitLowerAlarm{};        ///< Wert für die untere Alarmgrenze
    uint8_t limitTolerance{};       ///< Grenzwertabweichung
    uint8_t invokeInputChain[5]{};  ///< Auszuführende Eingabekette bei Statuswechsel
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[27]{};
#endif
}; // 92 Byte


///////////////////////////////////////
// Struct: StructVibrationSensor

/** Diese Struktur enthält die Informationen für die Behandlung eines Vibrationssensors.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructVibrationSensor
{
    uint8_t useAs{};                ///< Eingangsverwendung als ...
    uint32_t level{};               ///< Wert für die Erkennung einer ausgelösten Vibration
    uint16_t timeout{};             ///< Erkennungszeitraum in Sekunden
    uint8_t invokeInputChain[2]{};  ///< auszuführende Ketten eine für Start und eine für Stopp der Vibration
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[23]{};           ///< Reserviert.
#endif
}; // 32 Byte


///////////////////////////////////////
// Struct: StructValueGlobalVar

/** Diese Struktur enthält die ggf. zu setzenden Standardwerte der globalen Variablen.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructValueGlobalVar
{
    uint8_t flags{};        ///< Bitfeld gibt an welche Variablen gesetzt werden sollen.
    char value[8][21]{};    ///< Zu setzender Standardwert der Variablen.
                            ///< Der Wert darf maximal 20 Zeichen (ASCII) lang sein.
}; // 169 Byte


///////////////////////////////////////
// Struct: StructFingerprint

/** Diese Struktur enthält die Einstellparameter für ein Fingerprint-Modul

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructFingerprint
{
    uint8_t method{};           ///< Methode des Fingerprints
    uint8_t segment{};          ///< Startsegment der abgelegten Fingertemplates auf Ausweis
    uint8_t securityLevel{};    ///< Sicherheitslevel; der sich aus Falschrueckweisungrate et Falschakzeptanzrate ergibt.
    float securityFRR{};        ///< Falschrückweisungsrate
    float securityFAR{};        ///< Falschakzeptanzrate
    uint8_t templateFormat{};   ///< Formattemplate
                                ///< |Wert|Bedeutung                  |
                                ///< |:---|:--------------------------|
                                ///< |0   |DIN V66400 (161 Byte)      |
                                ///< |1   |Idencom-Compact (216 Byte) |
                                ///< |2   |Idencom-Standard (561 Byte)|
                                ///< |3   |Staturn-Standard (635 Byte)|
    uint8_t quality{};          ///< Mindestbildqualität beim Scannen
    uint8_t minutiaeCount{};    ///< Mindestanzahl der Minutien beim Scannen
    uint8_t passwordGroup{};    ///< Zu verwendende Passwortgruppe beim Schreiben des Ausweises
    uint8_t type{};             ///< Typ des Fingerprintmodul
                                ///< |Wert|Bedeutung             |
                                ///< |:---|:---------------------|
                                ///< |0   |Idencom Modul         |
                                ///< |1   |Saturn01 Flächensensor|
    uint8_t segment2{};         ///< Hier ist bei Ausweisen z.B. MifareDesfire wo die Templates nicht hintereinander eingetragen sind
                                ///< die Position des zweiten Template eingetragen
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    char unused[3]{};
#endif
}; // 20 Byte

/** Zusatztyp für ein Fingertemplate vom Typ DIN.
*/
struct StructFingertemplateDIN
{
    uint32_t pidHi{};
    uint32_t pidLo{};
    uint8_t fid{};
    uint8_t quality{};
    uint8_t count{};
    uint8_t data[50][3]{};
}; // 161 Byte

/** Zusatztyp für ein Fingertemplate vom Typ Idencom.
*/
struct StructFingertemplateIdencom
{
    uint32_t pidHi{};
    uint32_t pidLo{};
    uint8_t fid{};
    uint8_t quality{};
    uint8_t count{};
    uint8_t data[41][5]{};
}; // 216 Byte

/** Zusatztyp für ein Fingertemplate vom Typ Idencom Standard.
*/
struct StructFingertemplateIdencomStandard
{
    uint32_t pidHi{};
    uint32_t pidLo{};
    uint8_t fid{};
    uint8_t quality{};
    uint8_t count{};
    uint8_t data[50][11]{};
}; // 561 Byte

/** Zusatztyp für ein Fingertemplate vom Typ Saturn 01.
*/
struct StructFingertemplateSaturn01
{
    uint32_t pidHi{};
    uint32_t pidLo{};
    uint8_t fid{};
    uint8_t quality{};
    uint8_t count{};
    uint8_t data[39][16]{};
}; // 635 Byte

/** Zusatztyp für ein Fingertemplate vom Typ Id.
*/
struct StructFingertemplateId
{
    char pid[21]{};
    char fid[4]{};
}; // 25 Byte

///////////////////////////////////////
// Struct: StructTimeboyAdaption

/** Diese Struktur enthält die Einstellparameter für eine Timeboyadaption an MasterIV-Geräte.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructTimeboyAdaption
{
    uint8_t method{};               ///< Methode der Adaption
    uint8_t count{};                ///< Anzahl der Steckplätze bei Mehrfachdocking
    uint8_t records{};              ///< Datenablage auf MasterIV oder Timeboy
    uint8_t invokeInputChainPush{}; ///< auszuführende Eingabekette, wenn Timeboy gesteckt wird
    uint8_t invokeInputChainPull{}; ///< auszuführende Eingabekette, wenn Timeboy gezogen wird
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    char unused[15]{};
#endif
}; // 20 Byte

///////////////////////////////////////
// Struct: StructGPRS

/** Diese Struktur enthaelt die Einstellparameter für eine GPRS-Kommunikation

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructGPRS
{
    uint8_t method{};           ///< Methode der GPRS-Kommunikation
    uint8_t invokeInputChain{}; ///< auszuführende Eingabekette, wenn Alive-Timeout abgelaufen
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    char unused[8]{};
#endif
}; // 10 Byte

///////////////////////////////////////
// Struct: StructEnergieOption

/** Diese Struktur enthält die Einstellparameter für die Energieoptionen zum Modbil-MasterIV.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructEnergieOption
{
    uint8_t mode{};             ///< Aktivierung der Option
    uint8_t digitalIn{};        ///< Eingabekette für das An/Aus der Zuendung
    uint16_t timeout{};         ///< Timout, nach dem die Module nach einem An, eingeschaltet werden
    uint8_t invokeRelease{};    ///< Eingabekette zur Fahrzeugfreigabe.
    uint8_t invokeGPS{};        ///< Eingabekette bei gültigen GPS-Daten.
    uint8_t digitalOut{};       ///< Zu schaltendes Relais für Selbstabschaltung
    uint8_t flags{};            ///< Flags für die Energieoption
                                ///< |Bit|Bedeutung                                                    |
                                ///< |:-:|:------------------------------------------------------------|
                                ///< |1  |Einzel-(0) oder Dauerfreischaltung(1).                       |
                                ///< |2  |GPS-Start/Stopp für An/Aus der Zündung nutzen.               |
                                ///< |3  |Immer Bewegung, Erkennung für An/Aus der Zündung deaktiviert.|
                                ///< |4-8|-FREI-                                                       |
    uint8_t powerLow{};         ///< Tiefentladeschutz. Wert in Volt. 10,2V wird als 102 also *10 abgelegt.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[7]{};
#endif
}; // 16 Byte

///////////////////////////////////////
// Struct: StructTimer

/** Diese Struktur enthält die Einstellparameter für die Timer.

Die Struktur ist im extendet Speicherblock abgelegt.
*/
struct StructTimer
{
    uint8_t useAs{};            ///< Verwendung als Timer in Eingabeketten etc.
    uint16_t cycle{};           ///< Bei Verwendung als Timer, dessen Zyklus.
    uint16_t timeout{};         ///< Bei Verwendung als Timer, dessen Timeout.
    uint8_t invokeInputChain{}; ///< Auszufüehrende Eingabekette, wenn Timeout abgelaufen, oder danach der Zyklus abgelaufen.
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[10]{};
#endif
}; // 16 Byte

///////////////////////////////////////
// Struct: StructGPS

/** Diese Struktur enthält die Einstellparameter für die GPS-Signalverarbeitung.

Die Struktur ist im extendet Speicherblock abgelegt.
*/
struct StructGPS
{
    uint8_t startSpeed{};               ///< Zu erreichende Geschwindigkeit als Starterkennung. Einheit: km/h
    uint16_t startMove{};               ///< Zu erreichende Bewegungsentfernung als alternative Starterkennung. Einheit: Meter
    uint8_t invokeInputChainStart{};    ///< Auszuführende Eingabekette für Start Erkennung, 0 = keine Zuordnung
    uint16_t stopTimeout{};             ///< Timeoutzeit für die stopSpeed unterschritten sein muss, damit ein Stopperkennung stattfindet. Einheit: Sekunden
    uint8_t stopSpeed{};                ///< Geschwindigkeit die für das Erzeugen einer Stopperkennung unterschritten werden muss. Einheit: km/h
    uint8_t invokeInputChainStop{};     ///< Auszuführende Eingabekette für Stopperkennung, 0 = keine Zuordnung
    uint16_t cycle{};                   ///< Zykluszeit, in der ein Zyklusdatensatz erzeugt werden soll, wenn keine anderen Fahrdatensätze erzeugt werden. Einheit: Sekunden.
    uint8_t invokeInputChainCycle{};    ///< Auszuführende Eingabekette für Zyklusdatensatz, 0 = keine Zuordnung
    uint8_t course{};                   ///< Wert für Kursänderung.
    uint8_t invokeInputChainCourse{};   ///< Auszuführende Eingabekette für Kursdatensatz, 0 = keine Zuordnung
    uint16_t move{};                    ///< Wert für Bewegung. Einheit: Meter.
    uint8_t invokeInputChainMove{};     ///< Auszuführende Eingabekette für Bewegungsdatensatz, 0 = keine Zuordnung
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[2]{};
#endif
}; // 18 Byte

///////////////////////////////////////
// Struct: StructMachineDefinition

/** Diese Struktur enthält Informationen über die Definition einer Maschine.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructMachineDefinition
{
    uint8_t flags{};    ///< Gibt Informationsbits an.
                        ///< |Bit|Bedeutung       |
                        ///< |:-:|:---------------|
                        ///< |1  |0 -> deaktiviert|
                        ///< |^  |1 -> aktiviert  |
    char name[17]{};    ///< Bezeichnung der Maschine
                        ///< Die Bezeichnung darf maximal 16 Zeichen (ASCII) lang sein.
    uint8_t inputs{};   ///< Gibt die zugeordneten digitalen Eingänge an.
                        ///< |Bit|Bedeutung|
                        ///< |:-:|:--------|
                        ///< |1  |Eingang 1|
                        ///< |2  |Eingang 2|
                        ///< |3  |Eingang 3|
                        ///< |4  |Eingang 4|
                        ///< |5  |Eingang 5|
                        ///< |6  |Eingang 6|
                        ///< |7  |Eingang 7|
                        ///< |8  |Eingang 8|
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[5]{};
#endif
}; // 24 Byte

///////////////////////////////////////
// Struct: StructKeyBox

/** Diese Struktur enthält zusätzliche Informationen für die verschlüsselte Werteablage.
*/
struct StructKeyBox
{
    uint8_t random{};   ///< Beinhaltet einen Zufallswert.
    uint8_t flags{};    ///< Flagwerte.
                        ///< |Bit|Bedeutung (wenn gesetzt)|
                        ///< |:-:|:-----------------------|
                        ///< |1  |Passwortschutz aktiv    |
    char key[17]{};     ///< Beinhaltet den Schlüssel
                        ///< Der Schlüssel darf maximal 16 Zeichen (ASCII) lang sein.
    uint16_t crc{};     ///< CRC über die Struktur
}; // 21 Byte

///////////////////////////////////////
// Struct: StructExtended

/** Diese Struktur enthält zusaetzliche Informationen.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
In dieser Struktur werden alle weiteren Informationen abgelegt.
*/
struct StructExtended
{
    uint16_t doubleInputTime{};                 ///< Zeit für "Doppelte Eingabe" PZE1/PZE2 Modus in Sekunden
    uint16_t confirmationTime{};                ///< Zeit für "Bestätigungsfelder" in Sekunden
    // Sommer/Winterzeitumschaltung
    uint8_t daylightActive{};                   ///< Aktivieren der Sommer-/Winterzeitumschaltung (daylight saving)
                                                ///< |Wert|Bedeutung                         |
                                                ///< |---:|:---------------------------------|
                                                ///< |0   |deaktiviert                       |
                                                ///< |1   |aktiv und von Firmware auszuwerten|
    uint8_t daylight1Month{};                   ///< Monat in dem umgestellt wird (1 - 12).
                                                ///< Umstellung von Winterzeit (Normalzeit) auf Sommerzeit [+1 h].
    uint8_t daylight1Week{};                    ///< Monatswoche in der umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet).
                                                ///< Umstellung von Winterzeit (Normalzeit) auf Sommerzeit [+1 h].
    uint8_t daylight1Day{};                     ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag).
                                                ///< Umstellung von Winterzeit (Normalzeit) auf Sommerzeit [+1 h].
    uint8_t daylight1Hour{};                    ///< Stunde für Umschaltung (0 - 23).
                                                ///< Umstellung von Winterzeit (Normalzeit) auf Sommerzeit [+1 h].
    uint8_t daylight2Month{};                   ///< Monat in dem umgestellt wird (1 - 12).
                                                ///< Umstellung von Sommerzeit auf Winterzeit (Normalzeit) [-1 h].
    uint8_t daylight2Week{};                    ///< Monatswoche in der umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet).
                                                ///< Umstellung von Sommerzeit auf Winterzeit (Normalzeit) [-1 h].
    uint8_t daylight2Day{};                     ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag).
                                                ///< Umstellung von Sommerzeit auf Winterzeit (Normalzeit) [-1 h].
    uint8_t daylight2Hour{};                    ///< Stunde für Umschaltung (0 - 23)
                                                ///< Umstellung von Sommerzeit auf Winterzeit (Normalzeit) [-1 h].
    char transponderPassword32[9]{};            ///< Passwort für Hitag2 (32bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 8 Zeichen (ASCII) lang sein.
    char transponderPasswordA0[13]{};           ///< Passwort für Mifare (48bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 12 Zeichen (ASCII) lang sein.
    uint8_t phgAESKey[16]{};                    ///< Schlüssel für Kommunikationsverschlüsselung der PHG Leser
    StructFingerprint fingerprint;              ///< Datenstruktur zum Fingerprint
    StructTimeboyAdaption adaption;             ///< Anbindungsstruktur von Timboys an MasterIV
    StructGPRS gprs;                            ///< Struktur für Zusatzangaben bezüglich GPRS, wie z. B. Alive.
    StructEnergieOption energieOption;          ///< Struktur für Energieoption bei Mobil-MasterIV
    StructTimer timer[4]{};                     ///< Struktur für Timer in der Signalverarbeitung (vorgesehen sind 4 Timer a 16 Byte)
    uint8_t transponderPasswordFlag{};          ///< Passwortflag für Passwortverwendung von Mifare
                                                ///< |Bit|Bedeutung                                    |||
                                                ///< |:-:|:--------------------------------------------|||
                                                ///< |1-2|Passwortverwendung für Lesen/Schreiben A0/B0 |||
                                                ///< |^  |0              |1              |2              |
                                                ///< |^  |KeyA/KeyA      |KeyA/KeyB      |KeyB/KeyB      |
                                                ///< |3-4|Passwortverwendung für Lesen/Schreiben A1/B1 |||
                                                ///< |^  |0              |1              |2              |
                                                ///< |^  |KeyA/KeyA      |KeyA/KeyB      |KeyB/KeyB      |
                                                ///< |5-6|Passwortverwendung für Lesen/Schreiben A2/B2 |||
                                                ///< |^  |0              |1              |2              |
                                                ///< |^  |KeyA/KeyA      |KeyA/KeyB      |KeyB/KeyB      |
                                                ///< |7  |-FREI-                                       |||
                                                ///< |8  |Autologin                                    |||
                                                ///< |^  |Verwendung aller Passwörter für das Lesen    |||
    char transponderPasswordB0[13]{};           ///< Passwort für Mifare (48bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 12 Zeichen (ASCII) lang sein.
    char transponderPasswordA1[13]{};           ///< Passwort für Mifare (48bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 12 Zeichen (ASCII) lang sein.
    char transponderPasswordB1[13]{};           ///< Passwort für Mifare (48bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 12 Zeichen (ASCII) lang sein.
    char transponderPasswordA2[13]{};           ///< Passwort für Mifare (48bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 12 Zeichen (ASCII) lang sein.
    char transponderPasswordB2[13]{};           ///< Passwort für Mifare (48bit) in ASCII (Hex) abgelegt.
                                                ///< Das Passwort darf maximal 12 Zeichen (ASCII) lang sein.
    int32_t transponderSectorTrailer{};         ///< Zugriffseinstellungen für den globale Sektor.
    uint8_t transponderInitialValue[17]{};      ///< Startwert von Mifare-Ausweis
    uint8_t memoryUsage{};                      ///< Speicheraufteilung zwischen Daten/Listen in %. Wert gibt die % für Daten reserviert an.
    uint8_t entranceOnline{};                   ///< Betriebsmodus der Zutrittskontrolle.
                                                ///< |Wert|Bedeutung                 |
                                                ///< |---:|:-------------------------|
                                                ///< |0   |Offline                   |
                                                ///< |1   |Online                    |
                                                ///< |2   |Online/Offline mit Timeout|
    uint8_t entranceOnlineTimeout{};            ///< Timeout für die Umschaltung zwischen Online/Offline Betrieb der Zutrittskontrolle.
    StructDigitalInputExt digitalExtInput[8]{}; ///< Erweiterte Strukturen für die digitalen Eingänge.
    char fileName[44]{};                        ///< Dateiname der Setupdatei. Wird vor der Übertragung durch Laderoutine der Setupdatei gesetzt.
    char fileTimestamp[20]{};                   ///< Modifizierungszeitstempel der Setupdatei. Wird vor der Übertragung durch Laderoutine der Setupdatei gesetzt.
    uint8_t passwordSetup[17]{};                ///< Passwort für die Kommunikation
    StructGPS gps;                              ///< Struktur für GPS-Datensatzerzeugung
    uint8_t userPasswordEnable{};               ///< Benutzerpasswort verwenden, ansonsten wird das Systempasswort fürs Userpasswort verwendet.
    uint8_t escJumpToMainmenu{};                ///< Im PZE-Modus wird durch drücken der ESC-Taste das Hauptmenü angezeigt.
    uint8_t ledPerFieldFunction{};              ///< AE/BDE-MasterIV: LED steuerung mit Feldfunktion "LED schalten" steuern.
    uint16_t dataWriteOKTime{};                 ///< Die Anzeigezeit der Meldung "Datenspeicherung erfolgreich".
                                                ///< |Wert    |Bedeutung              |
                                                ///< |-------:|:----------------------|
                                                ///< |0       |aus                    |
                                                ///< |500     |default                |
                                                ///< |500-5000|Zeit in 250ms Schritten|
                                                ///< Einheit: ms
    uint8_t dataWriteOKFontType{};              ///< Der verwendete Zeichensatztyp bei Meldung "Datenspeicherung erfolgreich".
    uint32_t powerActiv{};                      ///< Gibt an ob die Energieverwaltung des Setups aktiviert ist.
                                                ///< Inklusive Flags für die aktivierten Module.
    uint8_t powerClockTimerCount{};             ///< Gibt an wie viele Timer die zu einer Bestimmten Uhrzeit auslösen existieren dürfen.
    uint8_t spsQuery{};                         ///< Abfrage einer angeschlossenen SPS. Feldfunktion SPS-Abfrage.
    uint8_t dataOnCard{};                       ///< Datenarrays auf Transponderkarten schreiben.
    uint8_t acExtension{};                      ///< Erweiterte Zutrittskontrolle verwenden.
    uint16_t acTagTimeToClose{};                ///< Die Zeit wie lange der Ausweis vor einen ZK Leser gehalten werden muss bis eine Schließung erfolgt.
    uint16_t acAlarmTimeout{};                  ///< Auszeit nachdem ein Alarmdatensatz generiert wird, weil ein digitaler Eingang zu lange offen war.
    uint8_t batteryLoadOptimization{};          ///< Ladeerhaltung voller Akkus optimieren.
    uint8_t showSmallWatch{};                   ///< Zusätzlich zur Anzeige der großen Uhr wird eine kleine Uhr mit Sekundenanzeige eingeblendet.
    uint8_t dontDimBacklightForCharge{};        ///< Die Hintergrundbeleuchtung soll nicht gedimmt werden um die Aufladung der Akku's zu beschleunigen.
    uint8_t flagsEnergie{};                     ///< Zusätzliche Flags für bestimmtes Geräteverhalten in der Energieverwaltung.
                                                ///< |Bit|Bedeutung (wenn gesetzt)                                                         |
                                                ///< |:-:|:--------------------------------------------------------------------------------|
                                                ///< |1  |TimeboyIV soll GPS abschalten, wenn er sich in der Dockingstation befindet.      |
                                                ///< |2  |TimeboyIV soll Mobilfunk abschalten, wenn er sich in der Dockingstation befindet.|
                                                ///< |3-8|-FREI-                                                                           |
    uint16_t displayOffAfter{};                 ///< Display ausschalten nach [Wert] Minuten ohne Bedienung. 0 = niemals aus bis 990 = 16,5 Stunden
    uint8_t flagsShowSetupNote{};               ///< Flag ob ein Hinweis zu Änderungen im Setup angezeigt werden soll.
                                                ///< |Bit|Bedeutung (wenn gesetzt)                                      |
                                                ///< |:-:|:-------------------------------------------------------------|
                                                ///< |1  |Energiesparoption "Display aus nach n Minuten ohne Bedienung."|
                                                ///< |2  |Standardwert wurde auf 120 Minuten gesetzt                    |
    uint8_t phgCryptUseFlags{};                 ///< Flags für die Zusatzsicherheitsoptionen bei ZK-Anlagen mit PHG-Lesern.
                                                ///< |Bit|Bedeutung (wenn gesetzt)                                                     |
                                                ///< |:-:|:----------------------------------------------------------------------------|
                                                ///< |1  |Initialisierungsvector soll zusätzliche zur Verschlüsselung verwendet werden.|
                                                ///< |2  |Sessionkeys sollen zusätzlich zur Verschlüsselung verwendet werden.          |
    uint8_t readerPollingFlags{};               ///< Flags für das Polling der Leser im Bus.
                                                ///< |Bit|Bedeutung (wenn gesetzt)                                      |
                                                ///< |:-:|:-------------------------------------------------------------|
                                                ///< |1  |Maximale Pollingrate.                                         |
                                                ///< |^  |Achtung: Dieses Bit wird ab der Version 04.03.09 nicht weiter |
                                                ///< |^  |unterstützt und von der Firmware als fest gesetzt ausgewertet.|
                                                ///< |2  |Nur Leser aus der Reader-Liste im Bus suchen.                 |
    uint8_t recordPriorityColumn[20]{};         ///< Spaltenindex für Priorität des Datensatzes zur Datensatzbeschreibung. 0=Keines zugewiesen, 1..MAX_FIELDS+1=Feldindex
    uint8_t cameraUse{};                        ///< Verwendung der eventuell integrierten Kamera.
                                                ///< |Wert|Bedeutung                                                   |
                                                ///< |---:|:-----------------------------------------------------------|
                                                ///< |0   |keine Verwendung                                            |
                                                ///< |1   |einfache Bildaufnahme per Feldfunktion                      |
                                                ///< |2   |einfache und stichprobenartige Bildaufnahme per Feldfunktion|
    uint8_t cameraGrab{};                       ///< Bei @ref cameraUse == 2 gibt es den Prozentwert der Stichprobe an.

    uint8_t memoryUsageExtData{};               ///< Für Daten zu Datensätzen (Kamerabilder in diesem Moment erstmal) 0 - 90
    uint8_t dateTimeFormatSettings{};           ///< Zur Einstellung des Datum- und Zeit-Formats an den Geräten
                                                ///< |Wert|Bedeutung                                                                       |
                                                ///< |---:|:-------------------------------------------------------------------------------|
                                                ///< |0   |augenblickliches StandardFormat = Tag.Monat.Jahr und Uhr 24 Stunden Anzeige     |
                                                ///< |1   |internationales Datumsformat Jahr-Monat-Tag und Uhr 24 Stunden Anzeige          |
                                                ///< |2   |USA Datumsformat Monat/Tag/Jahr und Uhr 24 Stunden Anzeige                      |
                                                ///< |3   |USA Datumsformat Monat-Tag-Jahr und Uhr 24 Stunden Anzeige                      |
                                                ///< |4   |GB Datumsformat Tag/Monat/Jahr und Uhr 24 Stunden Anzeige                       |
                                                ///< |5   |GB Datumsformat Tag-Monat-Jahr und Uhr 24 Stunden Anzeige                       |
                                                ///< |16  |deutsches Datumsformat Tag.Monat.Jahr und Uhr 12 Stunden Anzeige und AM/PM      |
                                                ///< |17  |internationales Datumsformat Jahr-Monat-Tag und Uhr 12 Stunden Anzeige und AM/PM|
                                                ///< |18  |USA Datumsformat Monat/Tag/Jahr und Uhr 12 Stunden Anzeige und AM/PM            |
                                                ///< |19  |USA Datumsformat Monat-Tag-Jahr und Uhr 12 Stunden Anzeige und AM/PM            |
                                                ///< |20  |GB Datumsformat Tag/Monat/Jahr und Uhr 12 Stunden Anzeige und AM/PM             |
                                                ///< |21  |GB Datumsformat Tag-Monat-Jahr und Uhr 12 Stunden Anzeige und AM/PM             |
    StructKeyBox protectionPassword;            ///< Passwort für die verschlüsselte Werteablage.
    uint8_t useSignature{};                     ///< Verwendung der Feldfunktion 'Unterschrift'.
                                                ///< |Wert|Bedeutung         |
                                                ///< |---:|:-----------------|
                                                ///< |0   |keine Verwendung  |
                                                ///< |1   |Verwendung möglich|
    uint8_t timerTimeBaseDivisor{1};            ///< Zur Einstellung der Zeitbasis der Timer für die Konfiguration der Startverzögerung und des Zyklus (1 - 10)
                                                ///< Die konfigurierten Werte (Startverzögerung und Zyklus) werden mit der Zeitbasis multipliziert.
                                                ///< |Wert|Zeitbasis|
                                                ///< |---:|--------:|
                                                ///< |1   |1000ms   |
                                                ///< |2   |500ms    |
                                                ///< |3   |333ms    |
                                                ///< |4   |250ms    |
                                                ///< |5   |200ms    |
                                                ///< |6   |167ms    |
                                                ///< |7   |143ms    |
                                                ///< |8   |125ms    |
                                                ///< |9   |111ms    |
                                                ///< |10  |100ms    |
    uint8_t commissioningMode;                  ///< Inbetriebnahmemodus bei ZK-Geräten aktiv.
    uint8_t useLongBarcode{};                   ///< Verwendung der Feldfunktion 'Langen Barcode lesen'.
                                                ///< |Wert|Bedeutung         |
                                                ///< |---:|:-----------------|
                                                ///< |0   |keine Verwendung  |
                                                ///< |1   |Verwendung möglich|

#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t unused[410]{};
#endif
}; // 1024 Byte

///////////////////////////////////////
// Struct: StructSystemMessage

enum class EnumSystemMessageType : uint8_t
{
    None = 0,
    Information = 1,    ///< 1 = Information
    Error = 2           ///< 2 = Fehler
};
enum class EnumSystemMessageInformationId : uint8_t
{
    None = 0,
    GetListPerHttp = 100    ///< 100 = Übernahme von Listen per HTTP

};
enum class EnumSystemMessageErrorId : uint8_t
{
    None = 0,
    GetListPerHttp = 100    ///< 100 = Übernahme von Listen per HTTP
};
union UnionSystemMessageId
{
    uint8_t param1{};
    EnumSystemMessageInformationId information;
    EnumSystemMessageErrorId error;
};

/** Diese Struktur enthält zusätzliche Informationen für die Behandlung einer Systemmeldung.

Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
struct StructSystemMessage
{
    EnumSystemMessageType type{ EnumSystemMessageType::None };  ///< Der Nachrichten-Type.
    UnionSystemMessageId id{};  ///< Die Nachrichtenkennung, abhängig von der Typangabe.
    uint32_t flags{};           ///< Reservierte Flagvariable.
    uint8_t invokeInputChain{}; ///< auszuführende Ereigniskette, wenn diese Meldung eintritt.
}; // 7 Byte


#ifndef DOXYGEN_SHOULD_SKIP_THIS
// Power
#define POWER_PROFIL_COUNT                         4  ///< Anzahl der möglichen Energieschemen
#define POWER_PROFIL_UNITS                         14 ///< Anzahl der auswählbaren Module CPU, Display usw.
#define POWER_PROFIL_ITEMS                         14 ///< Anzahl der auswählbaren Energieoptionen für AN, AUS usw.
#define POWER_EVENT_COUNT                          22 ///< Anzahl der definierten Events
#define POWER_EVENT_MAX                            22 ///< Anzahl der max. selektierbaren Events pro Energieschema
#define POWER_EVENT_TIMER_A_MAX    POWER_PROFIL_COUNT ///< Anzahl der Systemtimer, mögliche mit Timeout
#define POWER_EVENT_TIMER_B_MAX                     8 ///< Anzahl der Systemtimer, mögliche per Uhrzeit

//Auswahlpunkte der Module
#define POWER_ITEM_NR_OFF                          0  ///< Aus
#define POWER_ITEM_NR_ON                           1  ///< Ein
#define POWER_ITEM_NR_KEEP                         2  ///< Beibehalten aus vorausgegangenem Schema
#define POWER_ITEM_NR_SLEEP                        3  ///< Sleep-Mode
#define POWER_ITEM_NR_SERVICE_TIMEOUT              4  ///< Bedienung-TimeOut
#define POWER_ITEM_NR_CHAIN_FIELD                  5  ///< EIN, wenn Modul in aktiver Feldfunktion benutzt wird
#define POWER_ITEM_NR_DISPLAY_DARK                 6  ///< Beleuchtung Display Dunkel
#define POWER_ITEM_NR_SEND_RECORDS                 7  ///< Kommunikation "Ein" wenn Datensätze vorhanden sind, "Aus" nach Timeout
#define POWER_ITEM_NR_COM_ON                       8  ///< Kommunikation Zyklisch "Ein", nach Datenübertragung wieder "Aus" nach TimeOut, Aktivierungszeit in h oder als Uhrzeit
#define POWER_ITEM_NR_USBHOST_MENUE                9  ///< Nur Ein im BIOS-Menü
#define POWER_ITEM_NR_GPS_CYCLE                    10 ///< Zyklischer Betrieb Ein bis gültige Daten vorhanden sind und Zeitwert für Aus muss angegeben werden, Aktivierungszeit in h oder als Uhrzeit
#define POWER_ITEM_NR_DIGITAL_IN                   11 ///< Digitaler Eingang wird verwendet
#define POWER_ITEM_NR_MACHINE_STOP                 12 ///< Maschine steht(Stopp)
#define POWER_ITEM_NR_TB_TIMEOUT                   13 ///< Timeboy Steckevent EIN und nach TimeOut z.B. 10h wieder AUS
#define POWER_ITEM_NR_TOFF                         14 ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
#define POWER_ITEM_NR_TON                          15 ///< EIN, wenn digitaler Eingang x min. 1s HIGH ist und eingestellter Timeout abgelaufen. AUS wenn digitaler Eingang x wieder min. 1s LOW ist.
#define POWER_ITEM_NR_STANDBY                      16 ///< Modul wird nur bei Bedarf aktiviert. Kann eventl. Verzögerungen hervorrufen.
#define POWER_ITEM_NR_TB_SLOT                      17 ///< EIN, wenn digitaler Eingang x min. 1s HIGH wird. AUS wenn digitaler Eingang x wieder min. 1s LOW ist und eingestellter Timeout abgelaufen.
///< Zusaetzlich wird bei AUS, wenn Listendaten für Timeboys vorliegen, alle 2 Minuten nach zu aktualisierenden Timeboys geschaut.

//Module, Baugruppen
#define POWER_MODULE_NR_CPU                         1 ///< CPU
#define POWER_MODULE_NR_TIMER                       2 ///< Timer 
#define POWER_MODULE_NR_INPUTS                      3 ///< Eingänge
#define POWER_MODULE_NR_GLOBAL                      4 ///< Global Alles
#define POWER_MODULE_NR_12V_EXT                     5 ///< 12V extern
#define POWER_MODULE_NR_5V_EXT                      6 ///< 5V extern
#define POWER_MODULE_NR_COMMUNICATION               7 ///< Kommunikation
#define POWER_MODULE_NR_DISPLAY_LIGHT               8 ///< Display Beleuchtung
#define POWER_MODULE_NR_DISPLAY_TEXT                9 ///< Display Text
#define POWER_MODULE_NR_FINGERPRINT                10 ///< Biokey
#define POWER_MODULE_NR_TP                         11 ///< Transponder
#define POWER_MODULE_NR_GPS                        12 ///< GPS
#define POWER_MODULE_NR_USBHOST                    13 ///< USB-Host
#define POWER_MODULE_NR_MMC                        14 ///< MMC

//Flags Module
#define POWER_MODULE_CPU                        0x00000001 ///< CPU
#define POWER_MODULE_TIMER                      0x00000002 ///< Timer 
#define POWER_MODULE_INPUTS                     0x00000004 ///< Eingänge
#define POWER_MODULE_GLOBAL                     0x00000008 ///< Global Alles
#define POWER_MODULE_5V_EXT                     0x00000010 ///< 5V extern
#define POWER_MODULE_12V_EXT                    0x00000020 ///< 12V extern
#define POWER_MODULE_COMMUNICATION              0x00000040 ///< Kommunikation
#define POWER_MODULE_DISPLAY_LIGHT              0x00000080 ///< Display Beleuchtung
#define POWER_MODULE_DISPLAY_TEXT               0x00000100 ///< Display Text
#define POWER_MODULE_FINGERPRINT                0x00000200 ///< Biokey
#define POWER_MODULE_TP                         0x00000400 ///< Transponder
#define POWER_MODULE_GPS                        0x00000800 ///< GPS
#define POWER_MODULE_USBHOST                    0x00001000 ///< USB-Host
#define POWER_MODULE_MMC                        0x00002000 ///< MMC

//Events
#define POWER_EVENT_NR_TIMER_A                   0x01 ///< Systemtimer Typ A mit Aktivzeit
#define POWER_EVENT_NR_TIMER_B                   0x02 ///< Systemtimer Typ B mit Uhrzeit
#define POWER_EVENT_NR_KEY_PRESSED               0x03 ///< Eingabeereignis(Tastatur, Biokey, Transponder, Barcode)
#define POWER_EVENT_NR_KEY_TIMEOUT_1             0x04 ///< Eingabe TimeOut 1
#define POWER_EVENT_NR_KEY_TIMEOUT_2             0x05 ///< Eingabe TimeOut 2
#define POWER_EVENT_NR_USB                       0x06 ///< Stromversorgung über USB aktiviert
#define POWER_EVENT_NR_BATTERY                   0x07 ///< Stromversorgung über Batterie aktiviert
#define POWER_EVENT_NR_LINE_IN                   0x08 ///< Stromversorgung über Netzteil aktiviert
#define POWER_EVENT_NR_BAT_LOW                   0x09 ///< Batteriezustand niedrig
#define POWER_EVENT_NR_BAT_EMPTY                 0x0A ///< Batteriezustand leer
#define POWER_EVENT_NR_SWITCH_ON                 0x0B ///< Ein-/Ausschalter schaltet EIN
#define POWER_EVENT_NR_SWITCH_OFF                0x0C ///< Ein-/Ausschalter schaltet AUS
#define POWER_EVENT_NR_TB_PLUGIN                 0x0D ///< Timeboy-Steck-Event
#define POWER_EVENT_NR_TB_PLUGOUT                0x0E ///< Timeboy-Zieh-Event
#define POWER_EVENT_NR_MDE_START                 0x0F ///< Start-Erkennung MDE
#define POWER_EVENT_NR_MDE_STOP                  0x10 ///< Stopp-Erkennung MDE
#define POWER_EVENT_NR_ANALOG_OVER_VALUE         0x11 ///< Analogwertüberwachung Messwert Überschreitung
#define POWER_EVENT_NR_ANALOG_UNDER_VALUE        0x12 ///< Analogwertüberwachung Messwert Unterschreitung
#define POWER_EVENT_NR_MOVE_START                0x13 ///< Bewegungserkennung Start
#define POWER_EVENT_NR_MOVE_STOP                 0x14 ///< Bewegungserkennung Stopp
#define POWER_EVENT_NR_COMMUNICATION             0x15 ///< Kommunikation
#define POWER_EVENT_NR_COMMUNICATION_TIMEOUT     0x16 ///< KommunikationsTimeOut

//Flags der Energieverwaltung
#define POWER_EVENT_TIMER_A                     0x00000001 ///< Systemtimer Typ A mit Aktivzeit
#define POWER_EVENT_TIMER_B                     0x00000002 ///< Systemtimer Typ B mit Uhrzeit
#define POWER_EVENT_KEY_PRESSED                 0x00000004 ///< Eingabeereignis(Tastatur, Biokey, Transponder, Barcode)
#define POWER_EVENT_KEY_TIMEOUT_1               0x00000008 ///< Eingabe TimeOut 1
#define POWER_EVENT_KEY_TIMEOUT_2               0x00000010 ///< Eingabe TimeOut 2
#define POWER_EVENT_USB                         0x00000020 ///< Stromversorgung über USB aktiviert
#define POWER_EVENT_BATTERY                     0x00000040 ///< Stromversorgung über Batterie aktiviert
#define POWER_EVENT_LINE_IN                     0x00000080 ///< Stromversorgung über Netzteil aktiviert
#define POWER_EVENT_BAT_LOW                     0x00000100 ///< Batteriezustand niedrig
#define POWER_EVENT_BAT_EMPTY                   0x00000200 ///< Batteriezustand leer
#define POWER_EVENT_SWITCH_ON                   0x00000400 ///< Ein-/Ausschalter schaltet EIN
#define POWER_EVENT_SWITCH_OFF                  0x00000800 ///< Ein-/Ausschalter schaltet AUS
#define POWER_EVENT_TB_PLUGIN                   0x00001000 ///< Timeboy-Steck-Event
#define POWER_EVENT_TB_PLUGOUT                  0x00002000 ///< Timeboy-Zieh-Event
#define POWER_EVENT_MDE_START                   0x00004000 ///< Start-Erkennung MDE
#define POWER_EVENT_MDE_STOP                    0x00008000 ///< Stopp-Erkennung MDE
#define POWER_EVENT_ANALOG_OVER_VALUE           0x00010000 ///< Analogwertüberwachung Messwert Überschreitung
#define POWER_EVENT_ANALOG_UNDER_VALUE          0x00020000 ///< Analogwertüberwachung Messwert Unterschreitung
#define POWER_EVENT_MOVE_START                  0x00040000 ///< Bewegungserkennung Start
#define POWER_EVENT_MOVE_STOP                   0x00080000 ///< Bewegungserkennung Stopp
#define POWER_EVENT_COMMUNICATION               0x00100000 ///< Kommunikation
#define POWER_EVENT_COMMUNICATION_TIMEOUT       0x00200000 ///< KommunikationsTimeOut
#endif

/** Diese Struktur enthält Einstellungen und Adressen der Energieschemen.
*/
typedef struct StructPowerSetupNode
{
    uint8_t version{};          ///< Versionserkennung
    uint8_t countProfile{};     ///< Anzahl der benutzten Schemen
    uint8_t countEvents{};      ///< Anzahl der benutzten Events
    uint8_t active{};           ///< Schema welches nach dem Start des Setup zuerst aktiviert werden soll
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t temp[2]{};          ///< Reserve
    uint32_t ptrProfileArray{}; ///< Zeiger auf das 1. vorhandene Schema mit der Struktur PowerSetup
    uint32_t ptrEventArray{};   ///< Zeiger auf das 1. vorhandene Event mit der Struktur PowerEvent
#endif
} PowerSetupNode; // 14 Byte

/** Diese Struktur enthält Einstellungen des einzenlen Moduls.
*/
typedef struct StructPowerModule
{
    uint8_t item{};         ///< Auswahl der Energieoption EIN, AUS usw.
    uint32_t time{};        ///< Wert der Zeit in Sekunden
    uint8_t value{};        ///< Wert für Helligkeit oder andere Größen
    uint32_t interval{};    ///< Wert des Einschaltintervall der Zeit in Sekunden (value = 0) oder als Uhrzeit (value = 1)
} PowerModule; // 10 Byte

/** Diese Struktur enthält Einstellungen aller Module.
*/
typedef struct StructPowerSetup
{
    uint8_t name[32]{};         ///< Name des Schemas
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t temp[2]{};          ///< Reserve  
#endif
    PowerModule cpu;            ///< Einstellungen für Mircrocontroller
    PowerModule timer;          ///< Einstellungen für Timer der Signalverarbeitung
    PowerModule inputs;         ///< Einstellungen für Eingänge der Signalverarbeitung
    PowerModule deviceGlobal;   ///< Einstellungen für Globale Spannungsversorgung der Baugruppe
    PowerModule U5V;            ///< Einstellungen für 5V extern
    PowerModule U12V;           ///< Einstellungen für 12V extern
    PowerModule communication;  ///< Einstellungen für Kommunikation
    PowerModule displayLight;   ///< Einstellungen für Displaybeleuchtung
    PowerModule displayText;    ///< Einstellungen für Display Ein/Aus
    PowerModule fingerprint;    ///< Einstellungen für Fingerprintmodul Biokey3000
    PowerModule transponder;    ///< Einstellungen für Transponderleser
    PowerModule gps;            ///< Einstellungen für GPS Modul Lea 5H von ublox
    PowerModule usbHost;        ///< Einstellungen für USB-Host VDrive2
    PowerModule mmc;            ///< Einstellungen für interne MMC/SD-Karte
} PowerSetup; // 174 Byte

/** Diese Struktur enthält die Definition und Einstellungen eines Events.
*/
typedef struct StructPowerEvent
{
    uint8_t type{};             ///< Eventtyp
    uint8_t profile{ 0xFF };    ///< Event gültig für Energieschema Nr.
    uint32_t time{};            ///< Wert der Zeit in Sekunden, auch bei Format Uhrzeit
    uint8_t nextprofile{};      ///< nächstes Energieschema nach Ablauf der Aktivzeit
    uint8_t value{};            ///< Wert für Analogwertüberwachung
    uint8_t channel{};          ///< Eingang für den Analogwert
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    uint8_t tmp[2]{};           ///< Reserve ab V2 der Strukturen
#endif
} PowerEvent; // 11 Byte

typedef struct StructPowerFlags
{
    uint8_t busy{};         ///< Modul arbeitet gerade, TimeOut-Zähler = 0
    uint32_t timestamp{};   ///< Zeitwert ab wann Modul nicht mehr arbeitet
    uint32_t state{};       ///< Modul Ein(TRUE) oder Aus(FALSE)
} PowerFlags; // 9 Byte

typedef struct StructPowerState
{
    PowerFlags cpu;             ///< Status für Mircrocontroller
    PowerFlags timer;           ///< Status für Timer der Signalverarbeitung
    PowerFlags inputs;          ///< Status für Eingänge der Signalverarbeitung
    PowerFlags deviceGlobal;    ///< Status für Globale Spannungsversorgung der Baugruppe
    PowerFlags U5V;             ///< Status für 5V extern
    PowerFlags U12V;            ///< Status für 12V extern
    PowerFlags communication;   ///< Status für Kommunikation
    PowerFlags displayLight;    ///< Status für Displaybeleuchtung
    PowerFlags displayText;     ///< Status für Display Ein/Aus
    PowerFlags fingerprint;     ///< Status für Fingerprintmodul Biokey3000
    PowerFlags transponder;     ///< Status für Transponderleser
    PowerFlags gps;             ///< Status für GPS Modul Lea 5H von ublox
    PowerFlags usbHost;         ///< Status für USB-Host VDrive2
    PowerFlags mmc;             ///< Status für interne MMC/SD-Karte
} PowerState; // 126 Byte

typedef struct StructEventTimers
{
    uint32_t timerA{};          ///< Zeitwert des Systemtimers Typ A
    uint32_t timerB{};          ///< Zeitwert des Systemtimers Typ B
    uint32_t timerService{};    ///< Zeitwert ab wann keine Bedienung mehr erfolgte
    uint32_t timerTimeboy{};    ///< Zeitwert ab wann der Timeboy gesteckt wurde
    uint32_t timerMove{};       ///< Zeitwert ab wann Gerät nicht mehr bewegt wurde
    uint32_t timerCom{};        ///< Zeitwert ab wann Gerät nicht mehr kommunizierte
} PowerEventTimers; // 24 Byte

/** Diese Struktur enthält die wichtigsten Grundeinstellungen des Energiemanagements.
*/
typedef struct StructPowerProperties
{
    uint8_t profileMaxCount{};  ///< Anzahl der benutzbaren Schemen
    uint8_t activeProfile{};    ///< akuelles Energieschema
    PowerSetup profile{};       ///< Struktur der einzelnen Module mit ihren Optionen
    uint32_t moduleBusy{};      ///< Modul arbeitet gerade
    uint32_t moduleIdle{};      ///< Modul im Leerlauf
    PowerState modules{};       ///< Struktur der einzelnen Module mit aktuellen Werten
    uint8_t eventMaxCount{};    ///< Anzahl der Events im Setup
    uint8_t eventCount{};       ///< Anzahl der Events im aktuellen Schema
    uint32_t eventStates{};     ///< Energie-Events der Module
    PowerEventTimers timeEvent; ///< Event-Timer einzelner Module
} PowerProperties; // 340 Byte

/** Diese Struktur enthält die Einstellungen des Memofelds der Energieverwaltung.
*/
typedef struct StructPowerMemo
{
    char memo[255]{};   ///< Beinhaltet eine Notiz bzw. Information der Energieverwaltung.
                        ///< Der Inhalt darf maximal 254 Zeichen (ASCII) lang sein.
} PowerMemo; // 255 Byte

#pragma pack()

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#endif
#endif